import { world, Player, system } from "@minecraft/server";
import { ActionFormData, ModalFormData, MessageFormData } from "@minecraft/server-ui";
import Database from "./Database/index.js"

import { CustomUI } from "./customForm.js";
import { getScore, setScore } from "./library/lib/index";
import { openInviteForm } from "./function.js";



// Add player join handler
world.afterEvents.playerSpawn.subscribe((eventData) => {
    const player = eventData.player;
    if (!player) return;
    
    try {
        // Only tag if this is a fresh join (not a respawn)
        if (!player.hasTag('joined')) {
            player.addTag('joined');
            
            // Check notifications after a short delay
            system.runTimeout(() => {
                if (player.isValid()) {
                    checkPlayerNotifications(player);
                    initializeScoreboards();
                }
            }, 40); // 2-second delay
        }
    } catch (error) {
        console.error("Error in player join handler:", error);
    }
});

// Add player join handler to update their war status
world.afterEvents.playerJoin.subscribe(({ player }) => {
    try {
        const playerClan = Database.get(config.clanNameKey, player);
        if (!playerClan) return;

        // Check if player's clan is in an active war
        for (const [warId, warData] of activeWarData.entries()) {
            if (warData.clan1 === playerClan || warData.clan2 === playerClan) {
                // Update player about ongoing war
                const timeRemaining = warData.duration - Math.floor((Date.now() - warData.startTime) / 1000);
                if (timeRemaining > 0) {
                    player.sendMessage(
                        `§e⚔ Your clan is in an active war!\n` +
                        `§b${warData.clan1} §f(${warData.clan1Kills}) vs §b${warData.clan2} §f(${warData.clan2Kills})\n` +
                        `§eTime remaining: ${formatTime(timeRemaining)}`
                    );
                }
                break;
            }
        }
    } catch (error) {
        console.error('Error handling player join:', error);
    }
});

function initializeScoreboards() {
    try {
        // Create necessary scoreboards if they don't exist
        createScoreboardIfNotExists("clanWarKills", "Clan War Kills");
        createScoreboardIfNotExists(config.CLAN_SCORE, "Clan Score");
        createScoreboardIfNotExists(config.MONEY_SCORE, "Money");
        createScoreboardIfNotExists(config.WINS_SCORE, "Wins");
        createScoreboardIfNotExists(config.LOSSES_SCORE, "Losses");
        createScoreboardIfNotExists(config.TOTAL_GAMES_SCORE, "Total Games");
        createScoreboardIfNotExists("clanwins", "Clan Wins");
        createScoreboardIfNotExists("clanlost", "Clan Losses");
        createScoreboardIfNotExists("clankills", "Clan Kills");

        // Get all clan names from the database
        const clans = Database.entries()
            .filter(entry => entry.key.startsWith(config.clansDBKey))
            .map(entry => {
                try {
                    return JSON.parse(entry.value).name;
                } catch (error) {
                    return null;
                }
            })
            .filter(name => name !== null);

        // Initialize scores for all players and clans
        const commands = [
            // Player scoreboards
            `scoreboard players add @a ${config.MONEY_SCORE} 0`,
            `scoreboard players add @a ${config.WINS_SCORE} 0`, 
            `scoreboard players add @a ${config.LOSSES_SCORE} 0`,
            `scoreboard players add @a ${config.TOTAL_GAMES_SCORE} 0`,
            `scoreboard players add @a ${config.CLAN_SCORE} 0`,
            `scoreboard players add @a clanWarKills 0`,
            `scoreboard players add @a casinobalance 0`,
            `scoreboard players add @a balance1 0`,
            
            // Clan scoreboards - add for each clan
            ...clans.flatMap(clanName => [
                `scoreboard players add "${clanName}" clanwins 0`,
                `scoreboard players add "${clanName}" clanlost 0`,
                `scoreboard players add "${clanName}" clankills 0`,
                `scoreboard players add "${clanName}" ${config.CLAN_SCORE} 0`
            ])
        ];

        // Run commands to initialize scores
        const overworld = world.getDimension("overworld");
        for (const command of commands) {
            overworld.runCommandAsync(command).catch(error => {
                console.warn(`Failed to run command ${command}:`, error);
            });
        }

        // Additional check for new players every 5 seconds
        system.runInterval(() => {
            for (const player of world.getAllPlayers()) {
                try {
                    const objectives = [
                        config.MONEY_SCORE,
                        config.WINS_SCORE,
                        config.LOSSES_SCORE,
                        config.TOTAL_GAMES_SCORE,
                        config.CLAN_SCORE,
                        "clanWarKills"
                    ];

                    for (const objective of objectives) {
                        const obj = world.scoreboard.getObjective(objective);
                        if (obj && !obj.hasParticipant(player.scoreboardIdentity)) {
                            obj.setScore(player.scoreboardIdentity, 0);
                        }
                    }
                } catch (error) {
                    console.warn(`Error initializing scores for ${player.name}:`, error);
                }
            }
        }, 100); // Check every 5 seconds (100 ticks)

    } catch (error) {
        console.error('Error in scoreboard initialization:', error);
    }
}

// Call the function when world initializes
world.afterEvents.worldInitialize.subscribe(() => {
    initializeScoreboards();
    loadWarDataFromDatabase();
    verifyAndFixRoles();

});


// Add these global variables at the top of your file, after the imports
export const activeWars = new Map(); // For player-specific war data
export const activeWarData = new Map(); // For war-specific data
const surrenderVotes = new Map(); // For surrender voting system

const config = {
    MONEY_SCORE: 'money', // Scoreboard objective for money
    clansDBKey: "clans",
    clanMembersDBKey: "clanMembers",
    clanNameKey: "clanName",
    inviteCodeKey: "inviteCode",
    defaultClanDescription: "No description provided.",
    clanWarInvitationsDBKey: "clanWarInvitations",
    WINS_SCORE: 'wins',
    LOSSES_SCORE: 'losses',
    CLAN_SCORE: 'clanScore',
    CLAN_FUNDS: 'clanFunds',
    notificationsDBKey: "notifications" ,
    horizontalKB: 0.2789,
    verticalKB: 0.1023,
    sprintMultiplier: 1.05,
    TOTAL_GAMES_SCORE: 'clanGames', // Add this new config
};

// UUID-based rank system based on clan wins
const uuidRanks = [
    "Rookie",           // 0-2 wins
    "Bronze I",         // 3-5 wins
    "Bronze II",        // 6-8 wins
    "Bronze III",       // 9-11 wins
    "Silver I",         // 12-16 wins
    "Silver II",        // 17-21 wins
    "Silver III",       // 22-26 wins
    "Gold I",           // 27-31 wins
    "Gold II",          // 32-36 wins
    "Gold III",         // 37-41 wins
    "Platinum I",       // 42-46 wins
    "Platinum II",      // 47-51 wins
    "Platinum III",     // 52-56 wins
    "Diamond I",        // 57-61 wins
    "Diamond II",       // 62-66 wins
    "Diamond III",      // 67-71 wins
    "Master",           // 72-76 wins
    "Grandmaster",      // 77-81 wins
    "Elite",            // 82-86 wins
    "Legend"            // 87+ wins
];



// Event listener for opening the shop via a specific item (e.g., a book)
world.afterEvents.itemUse.subscribe(({ itemStack: item, source: player }) => {
    if (item.typeId === 'clan:crystal') {
        showMainMenu(player);
    }
});

system.runInterval(() => {
    for (let player of world.getPlayers()) {
        if (player.hasTag("clancreate")) {
            showMainMenu(player);
            player.removeTag("clancreate");
        }
    }
},30);


// Function to retrieve a player's rank
function getRank(player) {
    const allTags = player.getTags();
    const foundTag = allTags.find(tag => tag.startsWith('rank:'));
    return foundTag ? foundTag.substring(5) : 'Member'; // Default to 'Member' if no rank found
}

// Function to set or update a player's rank
function setRank(player, rank) {
    // Remove any existing rank tag before adding a new one
    const rankTags = player.getTags().filter(tag => tag.startsWith('rank:'));
    rankTags.forEach(tag => player.removeTag(tag)); // Remove old rank tag(s)
    // Add the new rank as a tag
    player.addTag(`rank:${rank}`);
}


// Function to generate a random six-digit clan invite code (e.g., clan-123456)
function generateInvitationCode() {
    const randomNum = Math.floor(100000 + Math.random() * 900000); // Generates a random six-digit number
    return `clan-${randomNum}`;
}

export async function showMainMenu(player) {
    if (!player) return;

    const ui = new CustomUI(2, "Clan System");
    const moneyObj = world.scoreboard.getObjective(config.MONEY_SCORE);
    const money = moneyObj ? moneyObj.getScore(player) : 0;
    
    ui.body("Manage your clan");
    ui.scores(money, 0);
    let buttonIndex = 0;

    // Check if the player is in a clan
    const playerClanName = Database.get(config.clanNameKey, player);
    
    // Array to track buttons for easier mapping
    const buttons = [];

    if (!playerClanName) {
        // Not in clan buttons
        buttons.push("create");
        ui.button("Create", 'textures/items/note', false, buttonIndex++);
        
        buttons.push("join");
        ui.button("Join", 'textures/items/trophy', false, buttonIndex++);
    }

    buttons.push("clans");
    ui.button("Clans", 'textures/ui/icon_multiplayer', false, buttonIndex++);

    if (playerClanName) {
        const playerInviteCode = Database.get(config.inviteCodeKey, player);
        if (playerInviteCode) {
            const clanDataRaw = Database.get(`${config.clansDBKey}.${playerInviteCode}`);
            if (clanDataRaw) {
                try {
                    const clanData = JSON.parse(clanDataRaw);
                    const logo = clanData?.logo || 'textures/items/star';
                    const buttonLabel = playerClanName.toLowerCase().endsWith('clan') 
                        ? playerClanName 
                        : `${playerClanName} Clan`;
                    
                    buttons.push("myClan");
                    ui.button(buttonLabel, logo, false, buttonIndex++);
                } catch (error) {
                    console.error('Error parsing clan data:', error);
                }
            }
        }
    }

    buttons.push("notifications");
    ui.button("Notifications", 'textures/items/message', false, buttonIndex++);

    if (player.hasTag('admin')) {
        buttons.push("admin");
        ui.button("§cAdmin Delete", 'textures/ui/magnifying_glass', false, buttonIndex++);
    }
    // Add admin reset button at the end if player is admin
    if (player.hasTag('admin')) {
        ui.button("§c[ADMIN] Reset Players", 'textures/ui/realms_red_x', false, buttonIndex++);
        buttons.push("adminReset");
    }
    buttons.push("info");
    ui.button("Info & Help", 'textures/ui/Caution', false, buttonIndex++);

    ui.show(player).then(result => {
        if (!result || result.selection === null) return;

        const selectedButton = buttons[result.selection];
        
        switch (selectedButton) {
            case "create":
                showCreateClanForm(player);
                break;
            case "join":
                showJoinClanForm(player);
                break;
            case "clans":
                showAllClans(player);
                break;
            case "myClan":
                showClanDetailsForm(player);
                break;
            case "notifications":
                showNotifications(player);
                break;
            case "admin":
                showAdminClanDeletionForm(player);
                break;
            case "info":
                showAddonInfo(player);
                break;
            case "adminReset":
                    if (player.hasTag('admin')) {
                        showPlayerSelectionForReset(player);
                    }
                    break;
            default:
                console.warn(`Unknown button selection: ${selectedButton}`);
        }
    }).catch(error => {
        console.error('Error in showMainMenu:', error);
        player.sendMessage('§cAn error occurred while displaying the main menu.');
    });
}

function showPlayerSelectionForReset(adminPlayer) {
    try {
        // Get all online players
        const onlinePlayers = world.getAllPlayers();
        
        // Get all players with clan data from database
        const allPlayersWithClanData = new Set();
        
        // Add online players
        onlinePlayers.forEach(player => {
            if (hasAnyClanData(player)) {
                allPlayersWithClanData.add({
                    name: player.name,
                    isOnline: true,
                    player: player
                });
            }
        });

        // Add offline players from database entries
        Database.entries().forEach(entry => {
            if (entry.key.startsWith(config.clanNameKey)) {
                const playerName = entry.key.split('.')[1];
                // Check if player is not already added (not online)
                if (!Array.from(allPlayersWithClanData).some(p => p.name === playerName)) {
                    allPlayersWithClanData.add({
                        name: playerName,
                        isOnline: false,
                        clanName: entry.value
                    });
                }
            }
        });

        // Convert Set to Array and sort alphabetically
        const playersArray = Array.from(allPlayersWithClanData).sort((a, b) => 
            a.name.localeCompare(b.name)
        );

        if (playersArray.length === 0) {
            adminPlayer.sendMessage("§eNo players with clan data found.");
            return;
        }

        const ui = new CustomUI(2, "§c[ADMIN] Select Players to Reset");
        
        let bodyText = "§fSelect players to reset clan data:\n" +
                      "§7(Green = Online, Red = Offline)\n\n";
        
        playersArray.forEach(playerData => {
            const clanName = playerData.isOnline ? 
                Database.get(config.clanNameKey, playerData.player) : 
                playerData.clanName;
            bodyText += `§7- ${playerData.isOnline ? '§a' : '§c'}${playerData.name} §7(${clanName})\n`;
        });
        
        ui.body(bodyText);

        // Add button for each player
        playersArray.forEach((playerData, index) => {
            const clanName = playerData.isOnline ? 
                Database.get(config.clanNameKey, playerData.player) : 
                playerData.clanName;
            
            ui.button(
                `${playerData.isOnline ? '§a' : '§c'}${playerData.name}\n§7Clan: ${clanName}`,
                playerData.isOnline ? 'textures/ui/icon_steve' : 'textures/ui/icon_steve_gray',
                false,
                index
            );
        });

        ui.show(adminPlayer).then(result => {
            if (!result || result.selection === null) return;

            const selectedPlayerData = playersArray[result.selection];
            if (selectedPlayerData) {
                if (selectedPlayerData.isOnline) {
                    showResetConfirmation(adminPlayer, selectedPlayerData.player);
                } else {
                    showOfflinePlayerResetConfirmation(adminPlayer, selectedPlayerData);
                }
            }
        });

    } catch (error) {
        console.error("Error in showPlayerSelectionForReset:", error);
        adminPlayer.sendMessage("§cAn error occurred while showing player selection.");
    }
}

// Add this new function for offline player reset confirmation
function showOfflinePlayerResetConfirmation(adminPlayer, playerData) {
    const ui = new CustomUI(1, "§cConfirm Offline Player Reset");
    
    ui.body(
        `§c⚠ WARNING ⚠\n\n` +
        `§fYou are about to reset:\n` +
        `§7Player: §c${playerData.name} §7(Offline)\n` +
        `§7Current Clan: §f${playerData.clanName}\n\n` +
        `§fThis will:\n` +
        `§7- Remove player from their clan\n` +
        `§7- Clear all clan-related data\n\n` +
        `§cThis action cannot be undone!`
    );

    ui.button("§c[DANGER] Confirm Reset", 'textures/ui/realms_red_x', false, 0);
    ui.button("§aCancel", 'textures/ui/cancel', false, 1);

    ui.show(adminPlayer).then(result => {
        if (!result || result.selection !== 0) {
            adminPlayer.sendMessage("§aReset cancelled.");
            return;
        }

        resetOfflinePlayerClanData(adminPlayer, playerData);
    });
}

// Add this new function to handle offline player reset
function resetOfflinePlayerClanData(adminPlayer, playerData) {
    try {
        // Remove all database entries for the player
        Database.entries().forEach(entry => {
            if (entry.key.includes(playerData.name) && 
               (entry.key.startsWith(config.clanNameKey) || 
                entry.key.startsWith(config.inviteCodeKey) || 
                entry.key.startsWith(config.notificationsDBKey))) {
                Database.delete(entry.key);
            }
        });

        // Remove player from clan members list
        const clanEntries = Database.entries()
            .filter(entry => entry.key.startsWith(config.clansDBKey));

        clanEntries.forEach(entry => {
            try {
                const clanData = JSON.parse(entry.value);
                if (clanData.members && clanData.members.includes(playerData.name)) {
                    clanData.members = clanData.members.filter(member => member !== playerData.name);
                    if (clanData.roles) {
                        delete clanData.roles[playerData.name];
                    }
                    Database.set(entry.key, JSON.stringify(clanData));
                }
            } catch (error) {
                console.error(`Error updating clan data for ${entry.key}:`, error);
            }
        });

        adminPlayer.sendMessage(`§aSuccessfully reset clan data for offline player §f${playerData.name}`);

    } catch (error) {
        console.error("Error in resetOfflinePlayerClanData:", error);
        adminPlayer.sendMessage("§cAn error occurred while resetting offline player data.");
    }
}
function showResetConfirmation(adminPlayer, targetPlayer) {
    const clanName = Database.get(config.clanNameKey, targetPlayer) || "Unknown Clan";
    
    const ui = new CustomUI(1, "§cConfirm Player Reset");
    
    ui.body(
        `§c⚠ WARNING ⚠\n\n` +
        `§fYou are about to reset:\n` +
        `§7Player: §f${targetPlayer.name}\n` +
        `§7Current Clan: §f${clanName}\n\n` +
        `§fThis will:\n` +
        `§7- Remove player from their clan\n` +
        `§7- Clear all clan-related tags\n` +
        `§7- Reset all clan-related data\n\n` +
        `§cThis action cannot be undone!`
    );

    ui.button("§c[DANGER] Confirm Reset", 'textures/ui/realms_red_x', false, 0);
    ui.button("§aCancel", 'textures/ui/cancel', false, 1);

    ui.show(adminPlayer).then(result => {
        if (!result || result.selection !== 0) {
            adminPlayer.sendMessage("§aReset cancelled.");
            return;
        }

        resetPlayerClanData(adminPlayer, targetPlayer);
    });
}

function resetPlayerClanData(adminPlayer, targetPlayer) {
    try {
        // Get clan data before reset for cleanup
        const clanName = Database.get(config.clanNameKey, targetPlayer);
        const inviteCode = Database.get(config.inviteCodeKey, targetPlayer);

        // Remove all clan-related tags
        const tags = targetPlayer.getTags();
        tags.forEach(tag => {
            if (tag.startsWith('clan:') || 
                tag.startsWith('role:') || 
                tag.startsWith('rank:')) {
                targetPlayer.removeTag(tag);
            }
        });

        // Clear clan database entries
        Database.delete(config.clanNameKey, targetPlayer);
        Database.delete(config.inviteCodeKey, targetPlayer);

        // Remove any pending notifications
        Database.entries()
            .filter(entry => entry.key.startsWith(`${config.notificationsDBKey}.`) && 
                           entry.key.includes(targetPlayer.name))
            .forEach(entry => Database.delete(entry.key));

        // If player was in a clan, remove them from the clan's member list
        if (clanName && inviteCode) {
            const clanDataKey = `${config.clansDBKey}.${inviteCode}`;
            const clanDataString = Database.get(clanDataKey);
            if (clanDataString) {
                try {
                    const clanData = JSON.parse(clanDataString);
                    clanData.members = clanData.members.filter(member => member !== targetPlayer.name);
                    if (clanData.roles) {
                        delete clanData.roles[targetPlayer.name];
                    }
                    Database.set(clanDataKey, JSON.stringify(clanData));
                } catch (error) {
                    console.error("Error updating clan data:", error);
                }
            }
        }

        // Notify players
        targetPlayer.sendMessage("§cYour clan data has been reset by an administrator.");
        adminPlayer.sendMessage(`§aSuccessfully reset clan data for §f${targetPlayer.name}`);

    } catch (error) {
        console.error("Error in resetPlayerClanData:", error);
        adminPlayer.sendMessage("§cAn error occurred while resetting player data.");
    }
}

// Helper function to check if a player has clan data
function hasAnyClanData(player) {
    // Check for clan-related tags
    const hasClanTags = player.getTags().some(tag => 
        tag.startsWith('clan:') || 
        tag.startsWith('role:') || 
        tag.startsWith('rank:')
    );

    // Check for clan database entries
    const hasClanDB = Database.get(config.clanNameKey, player) || 
                     Database.get(config.inviteCodeKey, player);

    return hasClanTags || hasClanDB;
}

function showAdminClanDeletionForm(player) {
    if (!player.hasTag('admin')) {
        player.sendMessage("§cOnly administrators can use this feature.");
        return;
    }

    try {
        const form = new ModalFormData()
            .title("Admin Clan Deletion")
            .textField("Enter clan name to search:", "Search Clans");

        form.show(player).then(response => {
            if (response.canceled || !response.formValues[0]) return;

            const searchTerm = response.formValues[0].toLowerCase();

            // Find all matching clans in the database
            const matchingClans = Database.entries()
                .filter(entry => {
                    if (!entry.key.startsWith(config.clansDBKey)) return false;
                    try {
                        const data = JSON.parse(entry.value);
                        return data.name.toLowerCase().includes(searchTerm);
                    } catch {
                        return false;
                    }
                })
                .map(entry => {
                    return {
                        key: entry.key,
                        data: JSON.parse(entry.value)
                    };
                });

            if (matchingClans.length === 0) {
                player.sendMessage(`§cNo clans found matching "${response.formValues[0]}"`);
                return;
            }

            // Show matching clans in a selection UI
            const ui = new CustomUI(2, "Select Clan to Delete");
            ui.body(`§eFound ${matchingClans.length} matching clan(s):`);

            matchingClans.forEach((clan, index) => {
                ui.button(
                    `§6${clan.data.name}\n§fOwner: ${clan.data.owner}\n§fMembers: ${clan.data.members.length}`, 
                    clan.data.logo || 'textures/items/star',
                    false, 
                    index
                );
            });

            ui.show(player).then(result => {
                if (!result || result.selection === null) return;

                const selectedClan = matchingClans[result.selection];
                const inviteCode = selectedClan.key.split('.')[1];

                // Show final confirmation UI
                const confirmUi = new CustomUI(1, "Confirm Clan Deletion");
                confirmUi.body(
                    `§cAre you sure you want to delete this clan?\n\n` +
                    `§6Clan Name: §f${selectedClan.data.name}\n` +
                    `§6Owner: §f${selectedClan.data.owner}\n` +
                    `§6Members: §f${selectedClan.data.members.length}\n` +
                    `§6Invite Code: §f${inviteCode}\n\n` +
                    `§cThis action cannot be undone!`
                );

                confirmUi.button("§cDelete Clan", 'textures/ui/realms_red_x', false, 0);
                confirmUi.button("Cancel", 'textures/ui/cancel', false, 1);

                confirmUi.show(player).then(confirmResult => {
                    if (!confirmResult || confirmResult.selection !== 0) {
                        player.sendMessage("§aClan deletion canceled.");
                        return;
                    }

                    // Delete the clan
                    try {
                        // Delete main clan entry
                        Database.delete(selectedClan.key);

                        // Clear clan data for all members
                        if (selectedClan.data.members && Array.isArray(selectedClan.data.members)) {
                            selectedClan.data.members.forEach(memberName => {
                                const member = findPlayerByName(memberName);
                                if (member) {
                                    // Clear database entries
                                    Database.delete(config.clanNameKey, member);
                                    Database.delete(config.inviteCodeKey, member);

                                    // Remove all clan-related tags
                                    member.getTags().forEach(tag => {
                                        if (tag.startsWith('clan:') || 
                                            tag.startsWith('role:') || 
                                            tag.startsWith('rank:')) {
                                            member.removeTag(tag);
                                        }
                                    });

                                    // Clear any other clan-related data
                                    removeClanTagFromPlayer(member, inviteCode);
                                    member.sendMessage("§cThe clan you were part of has been deleted.");
                                } else {
                                    // Handle offline players - store in pending removals
                                    const pendingRemovalKey = `pendingClanRemoval.${memberName}`;
                                    Database.set(pendingRemovalKey, JSON.stringify({
                                        action: 'delete',
                                        timestamp: Date.now()
                                    }));
                                }
                            });
                        }

                        // Delete all related clan data
                        Database.delete(`${config.clansDBKey}.${inviteCode}_funds`);
                        Database.delete(`${config.clansDBKey}.activeWar.${selectedClan.data.name}`);
                        Database.delete(`${config.clansDBKey}.${inviteCode}.kills`);
                        
                        // Delete any pending notifications
                        Database.entries()
                            .filter(entry => entry.key.startsWith(`${config.notificationsDBKey}.${selectedClan.data.name}`))
                            .forEach(entry => Database.delete(entry.key));

                        player.sendMessage(`§aSuccessfully deleted clan "${selectedClan.data.name}".`);

                    } catch (error) {
                        console.error('Error deleting clan:', error);
                        player.sendMessage("§cAn error occurred while deleting the clan.");
                    }
                });
            });

        }).catch(error => {
            console.error('Error in admin clan deletion:', error);
            player.sendMessage("§cAn error occurred while processing your request.");
        });

    } catch (error) {
        console.error('Error in showAdminClanDeletionForm:', error);
        player.sendMessage("§cAn error occurred while showing the form.");
    }
}

// Helper function to remove clan tags from player
function removeClanTagFromPlayer(player, inviteCode) {
    try {
        // Remove all clan-related tags
        const tagsToRemove = player.getTags().filter(tag => 
            tag.startsWith('clan:') || 
            tag.startsWith('role:') || 
            tag.startsWith('rank:')
        );
        
        tagsToRemove.forEach(tag => player.removeTag(tag));

        // Clear any other clan-related data
        Database.delete(config.clanNameKey, player);
        Database.delete(config.inviteCodeKey, player);
        
        // Remove any pending notifications
        Database.entries()
            .filter(entry => entry.key.startsWith(`${config.notificationsDBKey}.`) && 
                           entry.key.includes(player.name))
            .forEach(entry => Database.delete(entry.key));

    } catch (error) {
        console.error('Error removing clan tags:', error);
    }
}
function showNotifications(player) {
    try {
        const playerClanName = Database.get(config.clanNameKey, player);
        if (!playerClanName) {
            player.sendMessage("§7You are not in a clan.");
            return;
        }
        
        // Retrieve notifications for the player's clan with deduplication for fund notifications
        let seenFundNotifications = new Set();
        let notifications = Database.entries().filter(entry => {
            try {
                const isForClan = entry.key.startsWith(`${config.notificationsDBKey}.${playerClanName}`);
                const isFundNotification = entry.key.includes('fund_');
                
                if (isFundNotification) {
                    if (seenFundNotifications.has(entry.key)) {
                        return false;
                    }
                    seenFundNotifications.add(entry.key);
                }
                
                return isForClan || (isFundNotification && entry.key.includes(playerClanName));
            } catch (error) {
                return false;
            }
        });

        // If player is admin, also show clan reports
        if (player.hasTag('admin')) {
            const adminNotifications = Database.entries().filter(entry =>
                entry.key.startsWith(`${config.notificationsDBKey}.admin`)
            );
            notifications = notifications.concat(adminNotifications);
        }

        // Just send a message if there are no notifications
        if (!notifications || notifications.length === 0) {
            player.sendMessage("§7There are no notifications at the moment.");
            return;
        }

        // Create UI with type 3 for notifications list
        const ui = new CustomUI(3, "Notifications");
        let bodyText = "";
        let reportNotifications = [];
        let joinRequests = [];
        let warInvites = [];
        let clanWarnings = [];
        let fundNotifications = [];
        let roleChangeNotifications = []; // Add array for role change notifications

        notifications.forEach(notification => {
            try {
                const request = JSON.parse(notification.value);
                if (request.type === 'warInvite') {
                    warInvites.push({ notification, request });
                    bodyText += `§b${request.invitingClan} invited you to a war with bet $${request.betAmount}\n`;
                } else if (request.type === 'clanReport' && player.hasTag('admin')) {
                    reportNotifications.push({ notification, request });
                    bodyText += `§c[REPORT] ${request.clanName} - ${request.reportType}\n`;
                } else if (request.type === 'clanWarning') {
                    clanWarnings.push({ notification, request });
                    bodyText += `§e[WARNING] from Administrator\n`;
                } else if (request.type === 'fundTransaction') {
                    fundNotifications.push({ notification, request });
                    const transactionType = request.transactionType;
                    const color = transactionType === 'deposit' ? '§a' : '§c';
                    const action = transactionType === 'deposit' ? 'deposited' : 'withdrew';
                    bodyText += `${color}${request.playerName} ${action} $${request.amount}\n`;
                } else if (request.type === 'roleChange') { // Add handling for role change notifications
                    roleChangeNotifications.push({ notification, request });
                    bodyText += `§d[ROLE] ${request.changedPlayer}'s role changed\n`;
                } else if (!request.type) { // Assuming these are join requests
                    joinRequests.push({ notification, request });
                    bodyText += `§a${request.playerName} (Rank: ${request.playerRank})\n`;
                }
            } catch (error) {
                console.error('Error parsing notification:', error);
            }
        });

        if (!bodyText) {
            player.sendMessage("§7There are no notifications at the moment.");
            return;
        }

        ui.body(bodyText);

        // Add buttons for different notification types
        let buttonIndex = 0;

        // Add join request buttons
        joinRequests.forEach((item) => {
            ui.button(
                `§e${item.request.playerName}§a has requested to join the clan.`, 
                'textures/ui/invite_base', 
                false, 
                buttonIndex++
            );
        });

        // Add war invite buttons
        warInvites.forEach((item) => {
            ui.button(
                `§b${item.request.invitingClan}§a is inviting you for war bet: §e($${item.request.betAmount})`,
                'textures/ui/invite_base',
                false,
                buttonIndex++
            );
        });

        // Add report buttons (admin only)
        if (player.hasTag('admin')) {
            reportNotifications.forEach((item) => {
                ui.button(
                    `Report: ${item.request.clanName}`,
                    'textures/ui/warning_alex',
                    false,
                    buttonIndex++
                );
            });
        }

        // Add warning buttons
        clanWarnings.forEach((item) => {
            ui.button(
                `Warning from Admin`,
                'textures/ui/warning_alex',
                false,
                buttonIndex++
            );
        });

        // Add fund transaction buttons
        fundNotifications.forEach((item) => {
            const transactionType = item.request.transactionType;
            const color = transactionType === 'deposit' ? '§a' : '§c';
            const action = transactionType === 'deposit' ? 'deposited' : 'withdrew';
            const icon = transactionType === 'deposit' ? 'textures/items/money-bag' : 'textures/items/money-bag';
            
            ui.button(
                `${color}${item.request.playerName} ${action} $${item.request.amount}`,
                icon,
                false,
                buttonIndex++
            );
        });

        // Add role change notification buttons
        roleChangeNotifications.forEach((item) => {
            ui.button(
                `§d${item.request.changedPlayer}'s role changed`,
                'textures/ui/permissions_member_star',
                false,
                buttonIndex++
            );
        });

        // Only show UI if there are buttons
        if (buttonIndex > 0) {
            ui.show(player).then(result => {
                if (!result || result.selection === null) return;

                const totalRegularNotifs = joinRequests.length + warInvites.length;
                const totalReportNotifs = reportNotifications.length;
                const totalWarningNotifs = clanWarnings.length;
                const totalFundNotifs = fundNotifications.length;
                const selectedIndex = result.selection;

                if (selectedIndex < joinRequests.length) {
                    handleJoinRequest(player, joinRequests[selectedIndex].notification, joinRequests[selectedIndex].request);
                } else if (selectedIndex < totalRegularNotifs) {
                    const warIndex = selectedIndex - joinRequests.length;
                    handleWarInvitation(player, warInvites[warIndex].notification, warInvites[warIndex].request);
                } else if (player.hasTag('admin') && selectedIndex < totalRegularNotifs + totalReportNotifs) {
                    const reportIndex = selectedIndex - totalRegularNotifs;
                    handleReportNotification(player, reportNotifications[reportIndex].notification, reportNotifications[reportIndex].request);
                } else if (selectedIndex < totalRegularNotifs + totalReportNotifs + totalWarningNotifs) {
                    const warningIndex = selectedIndex - (totalRegularNotifs + totalReportNotifs);
                    handleWarningNotification(player, clanWarnings[warningIndex].notification, clanWarnings[warningIndex].request);
                } else if (selectedIndex < totalRegularNotifs + totalReportNotifs + totalWarningNotifs + totalFundNotifs) {
                    const fundIndex = selectedIndex - (totalRegularNotifs + totalReportNotifs + totalWarningNotifs);
                    handleFundNotification(player, fundNotifications[fundIndex].notification, fundNotifications[fundIndex].request);
                } else {
                    const roleIndex = selectedIndex - (totalRegularNotifs + totalReportNotifs + totalWarningNotifs + totalFundNotifs);
                    if (roleIndex >= 0 && roleIndex < roleChangeNotifications.length) {
                        handleRoleChangeNotification(player, roleChangeNotifications[roleIndex].notification, roleChangeNotifications[roleIndex].request);
                    }
                }
            }).catch(error => {
                console.error('Error displaying notifications:', error);
                player.sendMessage('§cAn error occurred while displaying notifications.');
            });
        } else {
            player.sendMessage("§7There are no actionable notifications at the moment.");
        }
    } catch (error) {
        console.error('Error in showNotifications:', error);
        player.sendMessage('§cAn error occurred while checking notifications.');
    }
}
function handleJoinRequest(adminPlayer, notificationKey, request) {
    try {
        // Get admin's clan name and data
        const adminClanName = Database.get(config.clanNameKey, adminPlayer);
        if (!adminClanName) {
            adminPlayer.sendMessage('§cError: Could not find your clan information.');
            return;
        }

        // Get clan data to check ownership
        const clanData = getClanData(adminClanName);
        if (!clanData) {
            adminPlayer.sendMessage('§cError: Could not find clan data.');
            return;
        }

        // Check if the player is the clan owner
        if (clanData.owner !== adminPlayer.name) {
            adminPlayer.sendMessage('§cOnly the clan owner can accept or decline join requests.');
            return;
        }

        const ui = new CustomUI(1, `Request from ${request.playerName}`);

        ui.body(
            `§6Player: §f${request.playerName}\n` +
            `§6Rank: §f${request.playerRank}\n\n` +
            `Do you want to accept this player into your clan?`
        );

        ui.button("Accept", 'textures/ui/confirm', false, 0);
        ui.button("Decline", 'textures/ui/cancel', false, 1);

        ui.show(adminPlayer).then(result => {
            if (!result || result.selection === null) return;

            const requestingPlayer = findPlayerByName(request.playerName);

            if (result.selection === 0) {
                // Get the clan's invite code if not provided in request
                const inviteCode = request.inviteCode || Database.get(config.inviteCodeKey, adminPlayer);
                
                if (!inviteCode) {
                    adminPlayer.sendMessage('§cError: Could not find clan invite code.');
                    return;
                }

                if (requestingPlayer) {
                    // Check if player is already in a clan
                    const playerCurrentClan = Database.get(config.clanNameKey, requestingPlayer);
                    if (playerCurrentClan) {
                        adminPlayer.sendMessage(`§c${request.playerName} is already in a clan.`);
                        return;
                    }

                    // Set player's clan data
                    Database.set(config.clanNameKey, adminClanName, requestingPlayer);
                    Database.set(config.inviteCodeKey, inviteCode, requestingPlayer);

                    // Update clan members list if not already included
                    if (!clanData.members.includes(request.playerName)) {
                        clanData.members.push(request.playerName);
                        Database.set(`${config.clansDBKey}.${inviteCode}`, JSON.stringify(clanData));
                    }

                    // Tag the player with the clan
                    tagPlayerWithClan(requestingPlayer, inviteCode);

                    // Set initial role
                    if (!clanData.roles) clanData.roles = {};
                    clanData.roles[request.playerName] = "member";
                    Database.set(`${config.clansDBKey}.${inviteCode}`, JSON.stringify(clanData));
                    requestingPlayer.addTag("role:member");

                    // Notify the players
                    adminPlayer.sendMessage(`§a${request.playerName} has been added to your clan.`);
                    requestingPlayer.sendMessage(`§aYour request to join ${adminClanName} has been accepted!`);
                    requestingPlayer.playSound("random.levelup");
                } else {
                    adminPlayer.sendMessage(`§e${request.playerName} is offline. They will be added when they next join.`);
                    // Store pending join data for offline player
                    const pendingJoinKey = `pendingJoin.${request.playerName}`;
                    const pendingData = {
                        clanName: adminClanName,
                        inviteCode: inviteCode,
                        timestamp: Date.now()
                    };
                    Database.set(pendingJoinKey, JSON.stringify(pendingData));
                }

                // Delete the notification
                Database.delete(notificationKey.key);
            } else {
                // Decline the request
                adminPlayer.sendMessage(`§c${request.playerName}'s request has been declined.`);
                if (requestingPlayer) {
                    requestingPlayer.sendMessage(`§cYour request to join ${adminClanName} was declined.`);
                }
                Database.delete(notificationKey.key);
            }
        });

    } catch (error) {
        console.error('Error in handleJoinRequest:', error);
        adminPlayer.sendMessage('§cAn error occurred while processing the request.');
    }
}


function showCreateClanForm(player) {
    if (!player || !player.name) return;

    // Check if player already has a clan
    const existingClan = Database.get(config.clanNameKey, player);
    if (existingClan) {
        player.sendMessage("§cYou are already in a clan and cannot create another one.");
        return;
    }

    const form = new ModalFormData()
        .title("Create a Clan")
        .textField("Clan Name:", "Enter a name", "")
        .textField("Clan Description:", "Enter a description", "");

    form.show(player).then(({ canceled, formValues }) => {
        if (canceled) return;

        const [name, description] = formValues;
        const cleanName = name ? name.trim() : "";

        // Validate the input fields
        if (!cleanName) {
            player.sendMessage("§cPlease enter a clan name.");
            return showCreateClanForm(player);
        }

        if (!description || description.trim() === "") {
            player.sendMessage("§cPlease enter a clan description.");
            return showCreateClanForm(player);
        }

        // Check if clan name already exists
        const existingClans = Database.entries()
            .filter(entry => entry.key.startsWith(config.clansDBKey))
            .map(entry => JSON.parse(entry.value));

        if (existingClans.some(clan => clan.name && clan.name.toLowerCase() === cleanName.toLowerCase())) {
            player.sendMessage("§cA clan with this name already exists. Please choose a different name.");
            return showCreateClanForm(player);
        }

        // Validate clan name length only
        if (cleanName.length < 3 || cleanName.length > 16) {
            player.sendMessage("§cClan name must be between 3 and 16 characters.");
            return showCreateClanForm(player);
        }

        // Show logo selection form after successful validation
        showLogoSelectionForm(player, cleanName, description.trim());
    }).catch(error => {
        console.error("Failed to create clan:", error);
        player.sendMessage("§cAn error occurred while creating the clan. Please try again.");
    });
}
// Function to initialize clan scoreboards
function initializeClanScoreboards(clanName) {
    try {
        const overworld = world.getDimension("overworld");
        overworld.runCommandAsync(`scoreboard players add "${clanName}" clanwins 0`);
        overworld.runCommandAsync(`scoreboard players add "${clanName}" clanlost 0`);
        overworld.runCommandAsync(`scoreboard players add "${clanName}" clankills 0`);
        overworld.runCommandAsync(`scoreboard players add @a money 0`);
    } catch (error) {
        console.error('Error initializing clan scoreboards:', error);
    }
}

// Find your clan creation function and add this after the clan is created
function showLogoSelectionForm(player, clanName, clanDescription) {
    const ui = new CustomUI(2, "Select Clan Logo");
    ui.body("Choose a logo for your clan:");

    let buttonIndex = 0;
    // Add 19 clan logo options
    for (let i = 1; i <= 19; i++) {
        ui.button(`${clanName}`, `textures/clans/clan${i}`, false, buttonIndex++);
    }

    ui.show(player).then(result => {
        if (result.canceled) return;

        const selectedLogo = result.selection !== null ? 
            `textures/clans/clan${result.selection + 1}` : 
            'textures/items/star';

        const inviteCode = generateInvitationCode();
        const clan = {
            name: clanName,
            description: clanDescription,
            owner: player.name,
            members: [player.name],
            logo: selectedLogo,
            rank: "Unranked",
            roles: {
                [player.name]: "Owner"
            }
        };

        // Save clan data
        Database.set(`${config.clansDBKey}.${inviteCode}`, JSON.stringify(clan));
        Database.set(config.clanNameKey, clanName, player);
        Database.set(config.inviteCodeKey, inviteCode, player);
        Database.set(`${config.clansDBKey}.${inviteCode}.kills`, "0");

        // Initialize clan scoreboards
        initializeClanScoreboards(clanName);

        // Tag the player and set role
        tagPlayerWithClan(player, inviteCode);
        setRank(player, "Owner");

        player.sendMessage(`§aSuccessfully created clan ${clanName}!`);
    }).catch(error => {
        console.error("Failed to create clan:", error);
        player.sendMessage("§cAn error occurred while creating the clan. Please try again.");
    });
}

function showJoinClanForm(player) {
    if (!player) return; // Ensure player exists

    // Check if the player is already in a clan
    const currentClan = Database.get(config.clanNameKey, player);
    if (currentClan) {
        player.sendMessage("§cYou are already in a clan and cannot join another.");
        return;
    }

    const form = new ModalFormData()
        .title("Join a Clan")
        .textField("Invite Code:", "Enter invite code", "");

    form.show(player).then(response => {
        if (response.canceled) return;

        const inviteCode = response.formValues ? response.formValues[0] : null;
        const clanDataString = Database.get(`${config.clansDBKey}.${inviteCode}`);
        const clan = clanDataString ? JSON.parse(clanDataString) : null;

        if (!clan) {
            player.sendMessage("§cInvalid invite code!");
            return;
        }

        // Create join request notification
        const requestData = {
            playerName: player.name,
            playerRank: getRank(player),
            timestamp: Date.now(),
            inviteCode: inviteCode
        };

        // Store the notification
        const notificationKey = `${config.notificationsDBKey}.${clan.name}.join_${Date.now()}`;
        Database.set(notificationKey, JSON.stringify(requestData));

        // Notify online clan members about the join request
        for (const member of world.getAllPlayers()) {
            if (Database.get(config.clanNameKey, member) === clan.name) {
                member.sendMessage(`§e${player.name} has requested to join your clan!`);
                member.runCommandAsync("playsound random.orb @s");
            }
        }

        player.sendMessage(`§aJoin request sent to ${clan.name}! Please wait for approval.`);
    });
}


// Function to get clan rank based on wins
function getClanRankFromWins(wins) {
    // Legend and above (87+ wins)
    if (wins >= 87) return "Legend";
    if (wins >= 82) return "Elite";
    if (wins >= 77) return "Grandmaster";
    if (wins >= 72) return "Master";
    
    // Diamond tier (57-71 wins)
    if (wins >= 67) return "Diamond III";
    if (wins >= 62) return "Diamond II";
    if (wins >= 57) return "Diamond I";
    
    // Platinum tier (42-56 wins)
    if (wins >= 52) return "Platinum III";
    if (wins >= 47) return "Platinum II";
    if (wins >= 42) return "Platinum I";
    
    // Gold tier (27-41 wins)
    if (wins >= 37) return "Gold III";
    if (wins >= 32) return "Gold II";
    if (wins >= 27) return "Gold I";
    
    // Silver tier (12-26 wins)
    if (wins >= 22) return "Silver III";
    if (wins >= 17) return "Silver II";
    if (wins >= 12) return "Silver I";
    
    // Bronze tier (3-11 wins)
    if (wins >= 9) return "Bronze III";
    if (wins >= 6) return "Bronze II";
    if (wins >= 3) return "Bronze I";
    
    // Default rank (0-2 wins)
    return "Rookie";
}

// Update any references to getClanRankFromKills to use getClanRankFromWins
// For example, in showClanDetailsForm:
const clanRank = getClanRankFromWins(config.WINS_SCORE); // Instead of using clanWins

// The color function can remain the same since it's based on rank names
function getRankColor(rank) {
    if (rank.includes("Legend")) return "§d"; // Light Purple
    if (rank.includes("Elite") || rank.includes("Grandmaster")) return "§5"; // Dark Purple
    if (rank.includes("Master")) return "§6"; // Gold
    if (rank.includes("Diamond")) return "§b"; // Aqua
    if (rank.includes("Platinum")) return "§3"; // Dark Aqua
    if (rank.includes("Gold")) return "§e"; // Yellow
    if (rank.includes("Silver")) return "§7"; // Gray
    if (rank.includes("Bronze")) return "§c"; // Red
    return "§8"; // Dark Gray for Rookie
}

function showClanDetailsForm(player) {
    if (!player) return;

    const inviteCode = Database.get(config.inviteCodeKey, player);
    const clanDataString = Database.get(`${config.clansDBKey}.${inviteCode}`);
    const clan = clanDataString ? JSON.parse(clanDataString) : null;

    if (!clan) {
        player.sendMessage("Clan data not found.");
        return;
    }

    // Check if the player is the clan leader
    const isClanLeader = clan.owner === player.name;

    // Use a unique key for the clan's funds
    const clanFundsKey = `${config.clansDBKey}.${inviteCode}_funds`;
    let clanFunds = Database.has(clanFundsKey) ? Database.get(clanFundsKey) : "0";
    clanFunds = parseInt(clanFunds, 10) || 0;

    // Get scoreboard objectives
    const winsObj = world.scoreboard.getObjective('clanwins');
    const lossesObj = world.scoreboard.getObjective('clanlost');
    const gamesObj = world.scoreboard.getObjective(config.TOTAL_GAMES_SCORE);
    
    // Get clan's stats from scoreboard and database
    const clanWins = winsObj ? (winsObj.getScore(clan.name) || 0) : 0;
    const clanLosses = lossesObj ? (lossesObj.getScore(clan.name) || 0) : 0;
    const totalGames = gamesObj ? (gamesObj.getScore(clan.name) || 0) : 0;
    
    // Get stored kills
    const killsKey = `${config.clansDBKey}.${inviteCode}.kills`;
    const totalClanKills = parseInt(Database.get(killsKey) || "0", 10);
    
    // Calculate clan rank based on wins
    const clanRank = getClanRankFromWins(clanWins);
    const rankColor = getRankColor(clanRank);

    // Calculate win rate
    let winRate = 0;
    if (totalGames > 0) {
        winRate = ((clanWins / totalGames) * 100).toFixed(2);
    }

    // Get win rate color based on percentage
    let winRateColor = "§7"; // Default gray for 0%
    if (winRate > 0) {
        if (winRate >= 70) winRateColor = "§2"; // Dark green for ≥70%
        else if (winRate >= 50) winRateColor = "§a"; // Light green for ≥50%
        else if (winRate >= 30) winRateColor = "§e"; // Yellow for ≥30%
        else winRateColor = "§c"; // Red for <30%
    }
    
    // Create UI and display clan info
    const ui = new CustomUI(1, `${clan.name} clan!`);

    ui.body(
        `§iDescription:§b ${clan.description}\n§6[§5${rankColor}${clanRank}§6]\n` +
        `§iInvite Code:§b ${inviteCode}\n\n` +
        `§iOwner:§b ${clan.owner} §7${getRank(findPlayerByName(clan.owner)) || 'Member'}\n\n` +
        `§iTotal Kills:§c ${totalClanKills}\n` +
        `§iTotal Games:§b ${totalGames}\n` +
        `§iWar Wins:§a ${clanWins}\n` +
        `§iWar Losses:§c ${clanLosses}\n` +
        `§iWin Rate:${winRateColor} ${winRate}%%\n` +
        `§iClan Funds:§e $ ${clanFunds}\n` +
        `§iClan Rank: §5${rankColor} ${clanRank}`
    );

    let buttonIndex = 0;

    ui.button("Leave Clan", 'textures/ui/realms_red_x', false, buttonIndex++);
    ui.button("Funds", 'textures/items/money-bag', false, buttonIndex++);
    ui.button("Settings", 'textures/items/settings', false, buttonIndex++);

    if (isClanLeader) {
        ui.button("Start Clan War", 'textures/items/diamond_sword', false, buttonIndex++);
        ui.button("Delete Clan", 'textures/ui/cancel.png', false, buttonIndex++);
    }

    ui.show(player).then(result => {
        if (!result || result.selection === null) {
            console.error('No selection made.');
            return;
        }

        switch (result.selection) {
            case 0: // Leave Clan
                confirmLeaveClan(player, inviteCode, clan);
                break;
            case 1: // Clan Funds
                manageClanFunds(player);
                break;
            case 2: // Settings
                showClanSettings(player, clan);
                break;
            case 3: // Start Clan War (if leader)
                if (isClanLeader) {
                    showAllClans(player, clan);
                }
                break;
            case 4: // Delete Clan (if leader)
                if (isClanLeader) {
                    confirmDeleteClan(player, clan);
                }
                break;
        }
    }).catch(error => {
        console.error('Error in showClanDetailsForm:', error);
        player.sendMessage('§cAn error occurred while displaying clan details.');
    });
}

function showClanSettings(player, clan) {
    const ui = new CustomUI(2, "Clan Settings");
    const isClanLeader = clan.owner === player.name;

    ui.body(
        `§eClan Settings\n\n` +
        `§7Select an option below:`
    );

    let buttonIndex = 0;

    // View Members button (available to all)
    ui.button("View Members", 'textures/ui/icon_multiplayer', false, buttonIndex++);

    // Role Management (only for clan owner)
    if (isClanLeader) {
        ui.button("Manage Roles", 'textures/ui/icon_steve', false, buttonIndex++);
        ui.button("Edit Clan Info", 'textures/ui/subscription_glyph_color', false, buttonIndex++);
    }

    // Back button
    ui.button("Back", 'textures/ui/arrow_left', false, buttonIndex++);

    ui.show(player).then(result => {
        if (!result || result.selection === null) return;

        if (result.selection === 0) {
            // View Members
            showClanMembers(player, clan);
        } else if (isClanLeader && result.selection === 1) {
            // Manage Roles
            showRoleAssignmentForm(player);
        } else if (isClanLeader && result.selection === 2) {
            // Edit Clan Info
            showClanInfoEditForm(player, clan);
        } else {
            // Back to clan details
            showClanDetailsForm(player);
        }
    });
}
// Add role assignment function
function showRoleAssignmentForm(player) {
    try {
        // Check if player is clan owner
        const playerClanName = Database.get(config.clanNameKey, player);
        const clan = getClanData(playerClanName);
        
        if (!clan || clan.owner !== player.name) {
            player.sendMessage("§cOnly clan owners can assign roles!");
            return;
        }

        // Get all online clan members
        const onlineMembers = world.getAllPlayers().filter(p => 
            Database.get(config.clanNameKey, p) === playerClanName &&
            p.name !== player.name // Exclude the owner
        );

        if (onlineMembers.length === 0) {
            player.sendMessage("§cNo online clan members to assign roles to!");
            return;
        }

        // Create member selection form
        const memberForm = new CustomUI(1, "Select Member");
        
        memberForm.body("§eSelect a member to assign a role:");
        
        onlineMembers.forEach((member, index) => {
            const currentRank = getRank(member);
            memberForm.button(`${member.name} (${currentRank})`, 'textures/ui/icon_steve', false, index);
        });

        memberForm.show(player).then(memberResult => {
            if (!memberResult || memberResult.selection === null) return;

            const selectedMember = onlineMembers[memberResult.selection];
            
            // Show role selection form
            const roleForm = new CustomUI(1, "Assign Role");
            
            roleForm.body(
                `§eAssign role to §f${selectedMember.name}\n` +
                `§7Current role: §f${getRank(selectedMember)}\n\n` +
                `§7Select new role:`
            );

            const roles = ["admin", "officer", "member", "recruit"];
            const roleIcons = {
                "admin": "textures/ui/op",
                "officer": "textures/ui/dressing_room_skins",
                "member": "textures/ui/friend_glyph",
                "recruit": "textures/ui/player_new"
            };

            roles.forEach((role, index) => {
                roleForm.button(
                    `${role.charAt(0).toUpperCase() + role.slice(1)}`,
                    roleIcons[role],
                    false,
                    index
                );
            });

            roleForm.show(player).then(roleResult => {
                if (!roleResult || roleResult.selection === null) return;

                const newRole = roles[roleResult.selection];
                
                // Show confirmation form
                const confirmForm = new ModalFormData()
                    .title("Confirm Role Assignment")
                    .textField(
                        `§eAssign §f${newRole} §erole to §f${selectedMember.name}?\n` +
                        `§7Type 'confirm' to proceed:`,
                        "Type here"
                    );

                confirmForm.show(player).then(confirmResult => {
                    if (!confirmResult || confirmResult.canceled) return;
                    
                    if (confirmResult.formValues[0].toLowerCase() === 'confirm') {
                        // Remove old role tags
                        const oldRank = getRank(selectedMember);
                        selectedMember.removeTag(`rank:${oldRank}`);
                        
                        // Add new role tag
                        selectedMember.addTag(`rank:${newRole}`);

                        // Notify both players
                        player.sendMessage(`§aSuccessfully assigned §f${newRole} §arole to §f${selectedMember.name}`);
                        selectedMember.sendMessage(`§aYou have been promoted to §f${newRole} §aby §f${player.name}`);
                        
                        // Play sound effects
                        player.playSound("random.levelup");
                        selectedMember.playSound("random.levelup");

                        // Create notification
                        const notificationData = {
                            type: 'roleChange',
                            changedPlayer: selectedMember.name,
                            oldRole: oldRank,
                            newRole: newRole,
                            changedBy: player.name,
                            timestamp: Date.now()
                        };

                        // Store notification for all clan members
                        clan.members.forEach(memberName => {
                            const notificationKey = `${config.notificationsDBKey}.${clan.name}.role_${Date.now()}_${Math.random().toString(36).substr(2, 9)}`;
                            Database.set(notificationKey, JSON.stringify(notificationData));
                        });

                        // Notify online clan members
                        for (const member of world.getAllPlayers()) {
                            if (Database.get(config.clanNameKey, member) === clan.name &&
                                member.name !== player.name &&
                                member.name !== selectedMember.name) {
                                member.sendMessage(`§e${selectedMember.name} has been assigned the role of ${newRole} by ${player.name}`);
                            }
                        }

                    } else {
                        player.sendMessage("§cRole assignment cancelled.");
                    }
                });
            });
        });

    } catch (error) {
        console.error('Error in showRoleAssignmentForm:', error);
        player.sendMessage('§cAn error occurred while assigning roles.');
    }
}

// Add role change notification handler
function handleRoleChangeNotification(player, notification, roleData) {
    const ui = new CustomUI(1, "Role Change Notification");
    
    ui.body(
        `§eRole Change Update:\n\n` +
        `§f${roleData.changedPlayer} §7was changed from\n` +
        `§f${roleData.oldRole} §7to §f${roleData.newRole}\n` +
        `§7by §f${roleData.changedBy}\n\n` +
        `§7Time: ${new Date(roleData.timestamp).toLocaleString()}`
    );

    ui.button("Dismiss", 'textures/ui/check', false, 0);

    ui.show(player).then(result => {
        if (!result || result.selection !== 0) return;
        
        Database.delete(notification.key);
        player.sendMessage("§aNotification dismissed.");
    });
}

// Function to show a custom UI confirmation for deleting a clan
function confirmDeleteClan(player, targetClan) {
    if (!player || !targetClan) return;

    const ui = new CustomUI(1, "Delete Clan Confirmation");

    ui.body(
        `§rAre you sure you want to delete the clan: ${targetClan.name}?\n\n` +
        `§cThis action cannot be undone and will remove all data related to the clan.`
    );

    ui.button("§aConfirm", 'textures/ui/confirm.png', false, 0);
    ui.button("§cCancel", 'textures/ui/cancel.png', false, 1);

    ui.show(player).then(result => {
        if (!result || result.selection === null) {
            console.error('No selection made.');
            return;
        }

        if (result.selection === 0) {
            // Open another form for final confirmation
            finalDeleteConfirmation(player, targetClan);
        } else {
            player.sendMessage("§cClan deletion canceled.");
        }
    }).catch(error => {
        console.error("Error in confirmDeleteClan:", error);
        player.sendMessage("§cAn error occurred while confirming clan deletion.");
    });
}

function finalDeleteConfirmation(player, targetClan) {
    if (!player || !targetClan) return;

    const ui = new CustomUI(1, "Final Confirmation");

    ui.body(
        `§rThis is your last chance to cancel.\n\n` +
        `§cAre you absolutely sure you want to delete the clan: ${targetClan.name}?`
    );

    ui.button("§aYes, Delete", 'textures/ui/confirm.png', false, 0);
    ui.button("§cNo, Cancel", 'textures/ui/cancel.png', false, 1);

    ui.show(player).then(result => {
        if (!result || result.selection === null) {
            console.error('No selection made.');
            return;
        }

        if (result.selection === 0) {
            // Proceed with deletion
            deleteClan(player, targetClan);
            player.sendMessage("§aThe clan has been deleted successfully.");
        } else {
            player.sendMessage("§cClan deletion canceled.");
        }
    }).catch(error => {
        console.error("Error in finalDeleteConfirmation:", error);
        player.sendMessage("§cAn error occurred while confirming final deletion.");
    });
}

// Function to handle the actual deletion of the clan
function deleteClan(player, targetClan) {
    if (!player || !targetClan) {
        console.error('Player or targetClan is undefined in deleteClan');
        return;
    }

    try {
        // Find the clan data by name
        const allClans = Database.entries()
            .filter(entry => {
                // Only include main clan entries, exclude war and other special entries
                return entry.key.startsWith(config.clansDBKey) && 
                       !entry.key.includes('war_') && 
                       !entry.key.includes('activeWar') &&
                       !entry.key.includes('_funds');
            })
            .map(entry => {
                try {
                    return {
                        key: entry.key,
                        data: JSON.parse(entry.value)
                    };
                } catch (error) {
                    console.warn(`Skipping invalid JSON for key ${entry.key}`);
                    return null;
                }
            })
            .filter(entry => entry !== null);

        const clanEntry = allClans.find(entry => entry.data.name === targetClan.name);

        if (!clanEntry) {
            console.error(`Clan ${targetClan.name} not found when trying to delete.`);
            return;
        }

        // Get invite code from the key (format: clans.INVITECODE)
        const inviteCode = clanEntry.key.split('.')[1];
        console.log('Found clan invite code:', inviteCode);

        // Delete the main clan entry
        Database.delete(clanEntry.key);

        // Clear clan data for all members
        if (clanEntry.data.members && Array.isArray(clanEntry.data.members)) {
            clanEntry.data.members.forEach(memberName => {
                const member = findPlayerByName(memberName);
                if (member) {
                    // Only remove clan data if it matches this clan
                    const memberClanName = Database.get(config.clanNameKey, member);
                    if (memberClanName === targetClan.name) {
                        Database.delete(config.clanNameKey, member);
                        Database.delete(config.inviteCodeKey, member);
                        removeClanTagFromPlayer(member, inviteCode);
                        member.sendMessage("§cThe clan you were part of has been deleted.");
                    }
                }
            });
        }

        // Delete related clan data
        Database.delete(`${config.clansDBKey}.${inviteCode}_funds`);
        Database.delete(`${config.clansDBKey}.activeWar.${targetClan.name}`);
        Database.delete(`${config.clansDBKey}.${inviteCode}.kills`);

        player.sendMessage(`§aThe clan "${targetClan.name}" has been successfully deleted.`);

        // Refresh the clan list
        showAllClans(player);

    } catch (error) {
        console.error('Error in deleteClan:', error);
        player.sendMessage("§cAn error occurred while deleting the clan.");
    }
}
function showClanMembers(player, clan) {
    // Validate inputs and get clan data if needed
    if (!player) return;

    // Get clan data if not provided
    if (!clan) {
        try {
            const clanName = Database.get(config.clanNameKey, player);
            const inviteCode = Database.get(config.inviteCodeKey, player);
            if (!inviteCode) {
                console.error('No invite code found for player');
                player.sendMessage('§cError: Could not find your clan data.');
                return;
            }
            
            const clanDataString = Database.get(`${config.clansDBKey}.${inviteCode}`);
            if (!clanDataString) {
                console.error('No clan data found for invite code:', inviteCode);
                player.sendMessage('§cError: Could not find clan data.');
                return;
            }

            clan = JSON.parse(clanDataString);
            if (!clan || !clan.members) {
                console.error('Invalid clan data structure');
                player.sendMessage('§cError: Invalid clan data.');
                return;
            }
        } catch (error) {
            console.error('Error retrieving clan data:', error);
            player.sendMessage('§cAn error occurred while retrieving clan data.');
            return;
        }
    }

    const membersList = clan.members || [];
    const ui = new CustomUI(2, "Clan Members");
    const viewerRole = getRole(player);

    let bodyText = "Select a member to view their info:\n\n";
    membersList.forEach((memberName, index) => {
        // Check if member is online
        const isOnline = findPlayerByName(memberName) !== null;
        const memberRole = getOfflinePlayerRole(memberName, clan);
        // Add online/offline indicator and role to member name
        bodyText += `${isOnline ? '§a' : '§7'}${memberName} §f(${memberRole}) ${isOnline ? '§2(Online)' : '§8(Offline)'}\n`;
        ui.button(memberName, 'textures/ui/icon_steve', false, index);
    });

    ui.body(bodyText);

    ui.show(player).then(response => {
        if (!response || response.selection === null) {
            console.error('No selection made.');
            return;
        }

        // Pass the selected member's name and clan data to showPlayerInfoForm1
        const selectedMemberName = membersList[response.selection];
        if (selectedMemberName) {
            showPlayerInfoForm1(player, selectedMemberName, clan);
        }
    }).catch(error => {
        console.error('Error displaying clan members:', error);
        player.sendMessage('§cAn error occurred while displaying clan members.');
    });
}

// Helper function to get offline player role with better error handling
function getOfflinePlayerRole(playerName, clan) {
    if (!clan) return 'member'; // Default fallback
    if (clan.owner === playerName) return 'owner';
    if (clan.roles && clan.roles[playerName]) return clan.roles[playerName];
    return 'member'; // Default role
}


function showPlayerInfoForm1(viewerPlayer, targetPlayerName, clan) {
    // ... existing validation code ...

    const viewerRole = getRole(viewerPlayer);
    const targetPlayer = findPlayerByName(targetPlayerName);
    const targetRole = targetPlayer ? getRole(targetPlayer) : getOfflinePlayerRole(targetPlayerName, clan);
    const isOnline = targetPlayer !== null;

    const ui = new CustomUI(1, `Player Info: ${targetPlayerName}`);

    ui.body(
        `§eName: §f${targetPlayerName}\n` +
        `§eStatus: ${isOnline ? '§aOnline' : '§8Offline'}\n` +
        `§eClan Role: §f${targetRole}\n\n` +
        `§7Select an action:`
    );

    let buttonIndex = 0;
    const buttons = [];

    // Back button
    buttons.push("back");
    ui.button("Back", 'textures/ui/arrow_left', false, buttonIndex++);

    // Message button (only if player is online)
    if (isOnline) {
        buttons.push("message");
        ui.button("Send Message", 'textures/ui/message', false, buttonIndex++);
    }

    // Add kick button for owner (regardless of online status)
    if (viewerRole === 'owner' && targetPlayerName !== clan.owner) {
        buttons.push("kick");
        ui.button("§cKick Member", 'textures/ui/realms_red_x', false, buttonIndex++);
    }

    // Add promote button for owner
    if (viewerRole === 'owner' && targetPlayerName !== clan.owner) {
        buttons.push("promote");
        ui.button("Promote Member", 'textures/ui/permissions_member_star', false, buttonIndex++);
    }

    ui.show(viewerPlayer).then(result => {
        if (!result || result.selection === null) return;

        const selectedAction = buttons[result.selection];
        switch (selectedAction) {
            case "back":
                showClanMembers(viewerPlayer, clan);
                break;
            case "message":
                if (isOnline && targetPlayer) {
                    showMessageForm(viewerPlayer, targetPlayer);
                }
                break;
            case "kick":
                if (isOnline && targetPlayer) {
                    confirmOfflineMemberKick(viewerPlayer, targetPlayerName, clan);
                } else {
                    confirmOfflineMemberKick(viewerPlayer, targetPlayerName, clan);
                }
                break;
            case "promote":
                showPromoteForm(viewerPlayer, targetPlayerName, clan);
                break;
        }
    }).catch(error => {
        console.error('Error in showPlayerInfoForm1:', error);
        viewerPlayer.sendMessage('§cAn error occurred while displaying player info.');
    });
}

function confirmOfflineMemberKick(kicker, targetPlayerName, clan) {
    if (!kicker || !targetPlayerName || !clan) return;

    const ui = new CustomUI(1, "Confirm Kick Member");

    ui.body(
        `§cAre you sure you want to kick §f${targetPlayerName}§c from the clan?\n\n` +
        `§7This action cannot be undone.`
    );

    ui.button("§cConfirm Kick", 'textures/ui/realms_red_x', false, 0);
    ui.button("§aCancel", 'textures/ui/cancel', false, 1);

    ui.show(kicker).then(result => {
        if (!result || result.selection !== 0) return;

        try {
            const inviteCode = Database.get(config.inviteCodeKey, kicker);
            if (!inviteCode) {
                console.error('No invite code found for kicker');
                return;
            }

            // Remove from clan members list
            clan.members = clan.members.filter(member => member !== targetPlayerName);
            
            // Remove roles if they exist
            if (clan.roles && clan.roles[targetPlayerName]) {
                delete clan.roles[targetPlayerName];
            }

            // Update clan data
            Database.set(`${config.clansDBKey}.${inviteCode}`, JSON.stringify(clan));

            // If player is online, clear their clan data
            const targetPlayer = findPlayerByName(targetPlayerName);
            if (targetPlayer) {
                // Clear player-specific clan data
                Database.delete(config.clanNameKey, targetPlayer);
                Database.delete(config.inviteCodeKey, targetPlayer);

                // Remove clan tags
                removeClanTagFromPlayer(targetPlayer, inviteCode);
                
                // Remove role tags
                const tags = targetPlayer.getTags();
                tags.forEach(tag => {
                    if (tag.startsWith('role:')) {
                        targetPlayer.removeTag(tag);
                    }
                });

                // Notify kicked player
                targetPlayer.sendMessage(`§cYou have been kicked from the clan by ${kicker.name}`);
                targetPlayer.playSound("mob.enderdragon.growl");
            }

            // Notify all online clan members
            for (const member of world.getAllPlayers()) {
                if (Database.get(config.clanNameKey, member) === clan.name) {
                    member.sendMessage(`§c${targetPlayerName} has been kicked from the clan by ${kicker.name}`);
                }
            }

            // Return to clan members list
            showClanMembers(kicker, clan);

        } catch (error) {
            console.error('Error in confirmOfflineMemberKick:', error);
            kicker.sendMessage('§cAn error occurred while kicking the member.');
        }
    });
}

// Add message form
function showMessageForm(sender, recipient) {
    const form = new ModalFormData()
        .title("Send Clan Message")
        .textField("Enter your message:", "Type your message here");

    form.show(sender).then(response => {
        if (response.canceled || !response.formValues[0]) return;

        const message = response.formValues[0];
        recipient.sendMessage(`§6[Clan Message] §e${sender.name}: §f${message}`);
        sender.sendMessage(`§aMessage sent to ${recipient.name}`);
        
        // Play notification sound
        recipient.playSound("random.orb");
    });
}

// Add kick confirmation
function confirmKickMember(kicker, targetPlayer, clan) {
    const ui = new CustomUI(1, "Confirm Kick Member");

    ui.body(
        `§cAre you sure you want to kick §f${targetPlayer.name}§c from the clan?\n\n` +
        `§7This action cannot be undone.`
    );

    ui.button("§cConfirm Kick", 'textures/ui/realms_red_x', false, 0);
    ui.button("§aCancel", 'textures/ui/cancel', false, 1);

    ui.show(kicker).then(result => {
        if (!result || result.selection !== 0) return;

        // Remove player from clan
        const inviteCode = Database.get(config.inviteCodeKey, targetPlayer);
        Database.delete(config.clanNameKey, targetPlayer);
        Database.delete(config.inviteCodeKey, targetPlayer);
        targetPlayer.removeTag(`role:${getRole(targetPlayer)}`);

        // Notify all clan members
        for (const member of world.getAllPlayers()) {
            if (Database.get(config.clanNameKey, member) === clan.name) {
                member.sendMessage(`§c${targetPlayer.name} has been kicked from the clan by ${kicker.name}`);
            }
        }

        // Notify kicked player
        targetPlayer.sendMessage("§cYou have been kicked from the clan.");
        
        // Update clan members list
        clan.members = clan.members.filter(member => member !== targetPlayer.name);
        Database.set(`${config.clansDBKey}.${inviteCode}`, JSON.stringify(clan));
    });
}

// Add promote form
function showPromoteForm(owner, targetPlayer, clan) {
    const currentRole = getRole(targetPlayer);
    const ui = new CustomUI(1, "Promote Member");

    ui.body(
        `§ePromote §f${targetPlayer.name}\n` +
        `§7Current Role: §f${currentRole}\n\n` +
        `§7Select new role:`
    );

    const roles = ["admin", "officer", "member"];
    const roleIcons = {
        "admin": "textures/ui/op",
        "officer": "textures/ui/dressing_room_skins",
        "member": "textures/ui/friend_glyph"
    };

    roles.forEach((role, index) => {
        if (role !== currentRole) { // Only show roles different from current
            ui.button(
                `${role.charAt(0).toUpperCase() + role.slice(1)}`,
                roleIcons[role],
                false,
                index
            );
        }
    });

    ui.show(owner).then(result => {
        if (!result || result.selection === null) return;

        const newRole = roles[result.selection];
        
        // Update role
        targetPlayer.removeTag(`role:${currentRole}`);
        targetPlayer.addTag(`role:${newRole}`);

        // Notify all clan members
        for (const member of world.getAllPlayers()) {
            if (Database.get(config.clanNameKey, member) === clan.name) {
                member.sendMessage(`§e${targetPlayer.name} has been promoted to ${newRole} by ${owner.name}`);
            }
        }

        // Play sound effects
        owner.playSound("random.levelup");
        targetPlayer.playSound("random.levelup");
    });
}

// Helper function to find player object by name
function findPlayerByName(name) {
    const players = getAllOnlinePlayers(); // Get list of all online players

    // Return the player object matching the name or null if not found
    return players.find(player => player.name === name) || null;
}

// Placeholder function to retrieve all online players
function getAllOnlinePlayers() {
    // Replace with your actual method or API call to fetch players
    // For example, if in Minecraft, you may need something like:
    return world.getPlayers(); // Minecraft Bedrock example
}

function leaveClan(player, inviteCode) {
    if (!player || !inviteCode) return; // Ensure player and invite code exist

    const clanDataString = Database.get(`${config.clansDBKey}.${inviteCode}`);
    const clan = clanDataString ? JSON.parse(clanDataString) : null;
    if (!clan) return; // Ensure clan exists

    clan.members = clan.members.filter(member => member !== player.name);

    // Update or remove clan data based on the remaining members
    if (clan.members.length === 0) {
        Database.delete(`${config.clansDBKey}.${inviteCode}`);
    } else {
        Database.set(`${config.clansDBKey}.${inviteCode}`, JSON.stringify(clan));
    }

    // Clear player-specific clan data
    Database.delete(config.clanNameKey, player);
    Database.delete(config.inviteCodeKey, player);

    // Remove the clan tag from the player
    removeClanTagFromPlayer(player, inviteCode);

    player.sendMessage("You left the clan.");
}


// Function to add the clan invite code as a tag to a player
function tagPlayerWithClan(player, inviteCode) {
    if (!player || !inviteCode) return;
    
    // Add the invite code as a tag (e.g., clan-123456)
    player.addTag(inviteCode);
}



async function showAllClans(player) {
    try {
        const allEntries = Database.entries();
        const clanEntries = allEntries.filter(entry => {
            // Only include main clan entries, exclude war and other special entries
            return entry.key.startsWith(config.clansDBKey) && 
                   !entry.key.includes('war_') && 
                   !entry.key.includes('activeWar') &&
                   !entry.key.includes('_funds');
        });

        if (clanEntries.length === 0) {
            player.sendMessage("§cNo clans available.");
            return;
        }

        const allClans = clanEntries.map(entry => {
            try {
                if (typeof entry.value === "string" && isValidJSON(entry.value)) {
                    return JSON.parse(entry.value);
                } else {
                    console.warn(`Skipping non-JSON entry for key ${entry.key}`);
                    return null;
                }
            } catch (error) {
                console.error(`Error parsing clan data for key ${entry.key}:`, error);
                return null;
            }
        }).filter(clan => clan !== null && clan.name);

        if (allClans.length === 0) {
            player.sendMessage("§cNo valid clans available.");
            return;
        }
        const ui = new CustomUI(2, "Available Clans");
        let bodyText = "Select a clan to view or join:\n\n";

        allClans.forEach((clan, index) => {
            const logo = clan.logo || 'textures/items/star';
            bodyText += `§a${clan.name} - ${clan.description || config.defaultClanDescription}\n`;
            ui.button(clan.name, logo, false, index);
        });

        ui.body(bodyText);
        ui.button("Close", 'textures/ui/cancel.png', false, allClans.length);

        const response = await ui.show(player);
        if (!response || response.selection === null) {
            return;
        }

        if (response.selection === allClans.length) {
            player.sendMessage("Closed the clan list.");
        } else {
            const selectedClan = allClans[response.selection];
            if (selectedClan) {
                showClanOptionsForm(player, selectedClan);
            }
        }

    } catch (error) {
        console.error("Error displaying available clans:", error);
        player.sendMessage("§cAn error occurred while displaying available clans.");
    }
}

function isValidJSON(str) {
    try {
        JSON.parse(str);
        return true;
    } catch (e) {
        return false;
    }
}

function showClanOptionsForm(player, targetClan) {
    if (!player || !targetClan || !targetClan.name) {
        console.error('Player or clan is undefined in showClanOptionsForm');
        return;
    }

    try {
        // Get admin's clan name and data
        const adminClanName = Database.get(config.clanNameKey, player);
       
        // Get complete clan data from database using name
        const allEntries = Database.entries();
        const clanEntry = allEntries.find(entry => {
            if (!entry.key.startsWith(config.clansDBKey)) return false;
            try {
                const data = JSON.parse(entry.value);
                return data.name === targetClan.name;
            } catch {
                return false;
            }
        });

        if (!clanEntry) {
            console.error('Clan not found in database');
            player.sendMessage('§cError: Clan not found.');
            return;
        }

        // Parse clan data and add invite code
        const inviteCode = clanEntry.key.split('.')[1];
        targetClan = {
            ...JSON.parse(clanEntry.value),
            inviteCode
        };

        // Get the inviting player's clan data (but don't require it)
        const invitingPlayerClanName = Database.get(config.clanNameKey, player);
        const invitingClan = invitingPlayerClanName ? getClanData(invitingPlayerClanName) : null;

        // Get scoreboard objectives
        const winsObj = world.scoreboard.getObjective('clanwins');
        const lossesObj = world.scoreboard.getObjective('clanlost');
        const gamesObj = world.scoreboard.getObjective(config.TOTAL_GAMES_SCORE);
        
        // Get clan's stats from scoreboard and database
        const clanWins = winsObj ? (winsObj.getScore(targetClan.name) || 0) : 0;
        const clanLosses = lossesObj ? (lossesObj.getScore(targetClan.name) || 0) : 0;
        const totalGames = gamesObj ? (gamesObj.getScore(targetClan.name) || 0) : 0;
        
        // Get stored kills
        const killsKey = `${config.clansDBKey}.${inviteCode}.kills`;
        const totalClanKills = parseInt(Database.get(killsKey) || "0", 10);
        
        // Calculate clan rank based on wins
        const clanRank = getClanRankFromWins(clanWins);
        const rankColor = getRankColor(clanRank);

        // Calculate win rate
        let winRate = 0;
        if (totalGames > 0) {
            winRate = ((clanWins / totalGames) * 100).toFixed(2);
        }

        // Get win rate color based on percentage
        let winRateColor = "§7"; // Default gray for 0%
        if (winRate > 0) {
            if (winRate >= 70) winRateColor = "§2"; // Dark green for ≥70%
            else if (winRate >= 50) winRateColor = "§a"; // Light green for ≥50%
            else if (winRate >= 30) winRateColor = "§e"; // Yellow for ≥30%
            else winRateColor = "§c"; // Red for <30%
        }

        const ui = new CustomUI(1, `Clan: ${targetClan.name}`);

        // Format the body text based on display settings
        let bodyText = `§iDescription:§e ${targetClan.description}\n`;
        
        if (targetClan.showInviteCode !== false) {
            bodyText += `§iInvite Code:§b ${inviteCode}\n`;
        }
        
        bodyText += `\n§iOwner:§b ${targetClan.owner}${getRankDisplay(targetClan.owner)}\n\n`;

        // Add each stat individually based on settings
        if (targetClan.showKills !== false) {
            bodyText += `§iTotal Kills:§c ${totalClanKills}\n`;
        }
        if (targetClan.showGames !== false) {
            bodyText += `§iTotal Games:§b ${totalGames}\n`;
        }
        if (targetClan.showWins !== false) {
            bodyText += `§iWar Wins:§a ${clanWins}\n`;
        }
        if (targetClan.showLosses !== false) {
            bodyText += `§iWar Losses:§c ${clanLosses}\n`;
        }
        if (targetClan.showWinRate !== false) {
            bodyText += `§iWin Rate:${winRateColor} ${winRate}%%\n`;
        }
        if (targetClan.showFunds !== false) {
            bodyText += `§iClan Funds:§e $${getClanFunds(targetClan)}\n`;
        }
        if (targetClan.showRank !== false) {
            bodyText += `§iClan Rank:${rankColor} ${clanRank}`;
        }

        ui.body(bodyText);

        let buttonIndex = 0;
        const buttons = [];

        // Only show Join button if player isn't in a clan
        if (!invitingClan) {
            ui.button("Join Clan", 'textures/ui/confirm', false, buttonIndex);
            buttons.push("join");
            buttonIndex++;
        }

        // View Members (available to all)
        ui.button("View Members", 'textures/ui/permissions_member_star', false, buttonIndex);
        buttons.push("members");
        buttonIndex++;

        // Only show War button if player is in a clan
        if (invitingClan) {
            ui.button("Invite to War", 'textures/items/diamond_sword', false, buttonIndex);
            buttons.push("war");
            buttonIndex++;
        }

        // Report Clan (available to all)
        ui.button("Report Clan", 'textures/ui/realms_red_x', false, buttonIndex);
        buttons.push("report");
        buttonIndex++;

        // Show delete button only for clan owner or admin
        if (targetClan.owner === player.name || player.hasTag('admin')) {
            ui.button("Delete Clan", 'textures/ui/realms_red_x', false, buttonIndex);
            buttons.push("delete");
            buttonIndex++;
        }

        // Add back button at the end
        ui.button("§7Back", 'textures/ui/arrow_left', false, buttonIndex);
        buttons.push("back");
        buttonIndex++;

        ui.show(player).then(response => {
            if (!response || response.selection === null) return;

            const selectedAction = buttons[response.selection];

            switch (selectedAction) {
                case "back":
                    showAllClans(player);
                    break;
                    
                case "join":
                    if (!invitingClan) {
                        confirmJoinClan(player, targetClan);
                    }
                    break;
                    
                case "members":
                    showClanMembers(player, targetClan);
                    break;
                    
                case "war":
                    if (invitingClan) {
                        openClanWarInviteForm(invitingClan, targetClan, player);
                    }
                    break;
                    
                case "report":
                    handleClanReport(player, targetClan);
                    break;
                    
                case "delete":
                    if (targetClan.owner === player.name || player.hasTag('admin')) {
                        finalDeleteConfirmation(player, targetClan);
                    }
                    break;
            }
        }).catch(error => {
            console.error('Error in showClanOptionsForm:', error);
            player.sendMessage('§cAn error occurred while showing clan options.');
        });

    } catch (error) {
        console.error('Error in showClanOptionsForm:', error);
        player.sendMessage('§cAn error occurred while showing clan options.');
    }
}

// Helper function to safely get rank display
function getRankDisplay(playerName) {
    const player = findPlayerByName(playerName);
    if (!player) return ' §7(Offline)';
    return ` §7(${getRank(player) || 'Member'})`;
}

// Function to open the form to invite another clan to a war
function openClanWarInviteForm(invitingClan, targetClan, player) {
    if (!invitingClan || !targetClan) {
        player.sendMessage("§cError: Could not retrieve clan data.");
        return;
    }

    // Get current clan funds
    const invitingClanFunds = getClanFunds(invitingClan);
    const targetClanFunds = getClanFunds(targetClan);

    const form = new ModalFormData()
    .title("Clan War Invitation")
    .textField(`Enter clan war bet amount:`, "Bet Amount")
    .dropdown("Select war duration:", [
        "15 Minutes (Quick War)",
        "30 Minutes (Short War)",
        "1 Hour (Standard War)",
        "2 Hours (Extended War)",
        "12 Hours (Half-Day War)",
        "24 Hours (Day War)",
        "3 Days (Extended War)",
        "7 Days (Week War)"
    ])
    .dropdown("Select preparation time:", [
        "1 Minute",
        "5 Minutes",
        "10 Minutes",
        "30 Minutes",
        "1 Hour",
        "2 Hours"
    ]);

    form.show(player).then(response => {
        if (response.canceled) {
            player.sendMessage("§cClan war invitation cancelled.");
            return;
        }

        const betAmount = parseInt(response.formValues[0], 10);
        const warDurationOptions = [
            900,    // 15 minutes (Quick War)
            1800,   // 30 minutes (Short War)
            3600,   // 1 hour (Standard War)
            7200,   // 2 hours (Extended War)
            43200,  // 12 hours (Half-Day War)
            86400,  // 24 hours (Day War)
            259200, // 3 days (Extended War)
            604800  // 7 days (Week War)
        ];
        
        const prepTimeOptions = [
            60,     // 1 minute
            300,    // 5 minutes
            600,    // 10 minutes
            1800,   // 30 minutes
            3600,   // 1 hour
            7200    // 2 hours
        ];

        const warDuration = warDurationOptions[response.formValues[1]];
        const prepTime = prepTimeOptions[response.formValues[2]];

        // Validate inputs
        if (isNaN(betAmount) || betAmount <= 0) {
            player.sendMessage("§cPlease enter a valid bet amount.");
            return;
        }

        if (betAmount > invitingClanFunds) {
            player.sendMessage(`§cYour clan doesn't have enough funds. Available: $${invitingClanFunds}`);
            return;
        }

        // Create war invitation data with explicit preparation time
        const warInvitationData = {
            type: 'warInvite',
            invitingClan: invitingClan.name,
            targetClan: targetClan.name,
            fromPlayer: player.name,
            betAmount: betAmount,
            warDuration: warDuration,
            prepTime: prepTime, // Make sure this is explicitly included
            timestamp: Date.now()
        };

        // Store the invitation in the target clan's notifications
        const notificationKey = `${config.notificationsDBKey}.${targetClan.name}.war_${Date.now()}`;
        Database.set(notificationKey, JSON.stringify(warInvitationData));

        // Format time displays for messages
        const prepTimeDisplay = formatTimeDisplay(prepTime);
        const warDurationDisplay = formatTimeDisplay(warDuration);

        // Notify all online members of the target clan
        for (const targetPlayer of world.getAllPlayers()) {
            const targetPlayerClan = Database.get(config.clanNameKey, targetPlayer);
            if (targetPlayerClan === targetClan.name) {
                targetPlayer.sendMessage(
                    `§e${invitingClan.name} has invited your clan to a war!\n` +
                    `§eBet amount: $${betAmount}\n` +
                    `§eWar Duration: ${warDurationDisplay}\n` +
                    `§ePreparation Time: ${prepTimeDisplay}`
                );
                targetPlayer.runCommandAsync("playsound random.orb @s");
            }
        }

        // Notify the inviting player
        player.sendMessage(
            `§aWar invitation sent to ${targetClan.name}\n` +
            `§aBet: $${betAmount}\n` +
            `§aWar Duration: ${warDurationDisplay}\n` +
            `§aPrep Time: ${prepTimeDisplay}`
        );

    }).catch(error => {
        console.error("Error in openClanWarInviteForm:", error);
        player.sendMessage("§cAn error occurred while creating the clan war invitation.");
    });
}

// Helper function to format time display
function formatTimeDisplay(seconds) {
    if (seconds >= 86400) { // Days
        return `${Math.floor(seconds / 86400)} day(s)`;
    } else if (seconds >= 3600) { // Hours
        return `${Math.floor(seconds / 3600)} hour(s)`;
    } else if (seconds >= 60) { // Minutes
        return `${Math.floor(seconds / 60)} minute(s)`;
    } else { // Seconds
        return `${seconds} second(s)`;
    }
}

// Function to confirm joining a clan
function confirmJoinClan(player, targetClan) {
    try {
        // Validate player and targetClan
        if (!player || !targetClan || typeof targetClan.name !== 'string') {
            console.error('Invalid player or target clan data.');
            player.sendMessage('§cAn error occurred while processing your request.');
            return;
        }

        const ui = new CustomUI(1, "Confirm Join Clan");

        // Ensure the body text is a valid string
        const bodyText = `Are you sure you want to request to join the clan: ${targetClan.name}?`;
        ui.body(bodyText);

        ui.button("Yes", 'textures/ui/confirm', false, 0);
        ui.button("No", 'textures/ui/cancel.png', false, 1);

        ui.show(player).then(result => {
            if (!result || result.selection === null) {
                console.error('No selection made.');
                return;
            }

            if (result.selection === 0) {
                // Send join request to clan admin
                sendJoinRequest(player, targetClan);
            } else {
                player.sendMessage('Join request canceled.');
            }
        }).catch(error => {
            console.error('Error in confirmJoinClan:', error);
            player.sendMessage('§cAn error occurred while confirming join request.');
        });
    } catch (error) {
        console.error('Error in confirmJoinClan:', error);
        player.sendMessage('§cAn unexpected error occurred.');
    }
}

// Function to send join request to clan admin
function sendJoinRequest(player, targetClan) {
    if (!player || !targetClan || !targetClan.owner) {
        console.warn("Invalid player or clan data in sendJoinRequest");
        return;
    }

    try {
        const clanAdmin = findPlayerByName(targetClan.owner);
        
        // Store the join request in the notifications database
        const notificationKey = `${config.notificationsDBKey}.${targetClan.name}.${player.name}`;
        const requestData = {
            playerName: player.name,
            playerRank: getRank(player),
            timestamp: Date.now()
        };

        Database.set(notificationKey, JSON.stringify(requestData)); // Stringify the data before storing

        // Notify the player that their request was sent
        player.sendMessage(`§aYour request to join ${targetClan.name} has been sent.`);

        // If admin is online, notify them immediately
        if (clanAdmin) {
            clanAdmin.sendMessage(`§a${player.name} has requested to join your clan.`);
        }

    } catch (error) {
        console.error("Error in sendJoinRequest:", error);
        player.sendMessage("§cAn error occurred while sending your join request.");
    }
}


//////UPDATE CLAN WAR SYSTEM

////

///


// Function to handle war invitation
function handleWarInvitation(player, notificationKey, request) {
    if (!player || !notificationKey || !request) {
        console.error('Invalid player or request data in handleWarInvitation');
        return;
    }

    try {
        // Get player's clan name
        const playerClanName = Database.get(config.clanNameKey, player);
        if (!playerClanName) {
            player.sendMessage("§cYou must be in a clan to accept war invitations.");
            return;
        }

        // Get clan data to check ownership
        const clanData = getClanData(playerClanName);
        if (!clanData) {
            player.sendMessage('§cError: Could not find clan data.');
            return;
        }

        // Check if the player is the clan owner
        if (clanData.owner !== player.name) {
            player.sendMessage('§cOnly the clan owner can accept or decline war invitations.');
            return;
        }

        // Check if clan has enough funds for the bet
        const clanFunds = getClanFunds(clanData);
        if (clanFunds < request.betAmount) {
            player.sendMessage(`§cYour clan doesn't have enough funds for this war. Required: $${request.betAmount}, Available: $${clanFunds}`);
            return;
        }

        // Create UI for war invitation
        const ui = new CustomUI(1, "War Invitation");
        ui.body(
            `§6War Invitation from: §f${request.invitingClan}\n` +
            `§6Sent by: §f${request.fromPlayer}\n` +
            `§6Bet Amount: §f$${request.betAmount}\n` +
            `§6Duration: §f${Math.floor(request.warDuration / 60)} minutes\n\n` +
            `§eClan Funds Available: $${clanFunds}\n\n` +
            `Do you want to accept this war invitation?`
        );

        ui.button("§aAccept", 'textures/ui/confirm', false, 0);
        ui.button("§cDecline", 'textures/ui/cancel', false, 1);

        ui.show(player).then(result => {
            if (result.canceled || result.selection === null) {
                return;
            }

            if (result.selection === 0) {
                // Double check funds before accepting
                const currentFunds = getClanFunds(clanData);
                if (currentFunds < request.betAmount) {
                    player.sendMessage(`§cInsufficient clan funds. Required: $${request.betAmount}, Available: $${currentFunds}`);
                    return;
                }

                // Accept war invitation
                acceptWarInvitation(player, request);
                
                // Notify all members of both clans
                notifyClansAboutWar(playerClanName, request.invitingClan, request.betAmount, request.warDuration);
                
                // Remove notification if accepted
                Database.delete(notificationKey.key);
            } else {
                // Decline war invitation
                player.sendMessage(`§cWar invitation from ${request.invitingClan} declined.`);
                const inviter = findPlayerByName(request.fromPlayer);
                if (inviter) {
                    inviter.sendMessage(`§c${playerClanName} has declined your war invitation.`);
                }
                
                // Remove notification if declined
                Database.delete(notificationKey.key);
            }
        });

    } catch (error) {
        console.error('Error in handleWarInvitation:', error);
        player.sendMessage('§cAn error occurred while handling the war invitation.');
    }
}

// Helper function to notify all clan members about the war
function notifyClansAboutWar(clan1Name, clan2Name, betAmount, duration) {
    const allPlayers = world.getAllPlayers();
    
    allPlayers.forEach(player => {
        const playerClan = Database.get(config.clanNameKey, player);
        if (playerClan === clan1Name || playerClan === clan2Name) {
            player.sendMessage(
                `§e=== WAR ANNOUNCEMENT ===\n` +
                `§6${clan1Name} §fvs §6${clan2Name}\n` +
                `§eBet Amount: §f$${betAmount}\n` +
                `§eDuration: §f${Math.floor(duration / 60)} minutes\n` +
                `§aWar will begin shortly!`
            );
            player.playSound("random.levelup");
        }
    });
}

// Helper function to get clan funds
function getClanFunds(clan) {
    try {
        if (!clan || !clan.name) {
            console.warn('Invalid clan data passed to getClanFunds');
            return 0;
        }

        // Filter out war-related entries and only look for clan data
        const clanData = Database.entries()
            .filter(entry => 
                entry.key.startsWith(config.clansDBKey) && 
                !entry.key.includes('war_') &&  // Exclude war-related entries
                !entry.key.includes('.wars.') && // Exclude war storage
                !entry.key.includes('.activeWar.') // Exclude active war data
            )
            .find(entry => {
                try {
                    const data = JSON.parse(entry.value);
                    return data && data.name === clan.name;
                } catch (error) {
                    console.warn(`Skipping invalid JSON entry for clan funds: ${entry.key}`);
                    return false;
                }
            });

        if (!clanData) {
            console.warn(`No valid clan data found for: ${clan.name}`);
            return 0;
        }

        const inviteCode = clanData.key.split('.')[1];
        const clanFundsKey = `${config.clansDBKey}.${inviteCode}_funds`;
        const funds = Database.get(clanFundsKey);
        
        return parseInt(funds || "0", 10);
    } catch (error) {
        console.error('Error in getClanFunds:', error);
        return 0;
    }
}



// Utility function to format time in mm:ss
function formatTime(seconds) {
    // Round to whole seconds
    seconds = Math.floor(seconds);
    
    const minutes = Math.floor(seconds / 60);
    const remainingSeconds = seconds % 60;

    // Pad with zeros if needed
    const formattedMinutes = String(minutes).padStart(2, '0');
    const formattedSeconds = String(remainingSeconds).padStart(2, '0');

    return `${formattedMinutes}:${formattedSeconds}`;
}



// Scoreboard utility functions
function createScoreboardIfNotExists(objectiveId, displayName) {
    try {
        let objective = world.scoreboard.getObjective(objectiveId);
        if (!objective) {
            objective = world.scoreboard.addObjective(objectiveId, displayName);
        }
        return objective;
    } catch (error) {
        console.error(`Error creating scoreboard ${objectiveId}:`, error);
        return null;
    }
}

// Function to get score
function getScore(player, objectiveId) {
    try {
        const objective = world.scoreboard.getObjective(objectiveId);
        if (!objective) return 0;
        return objective.getScore(player) || 0;
    } catch (error) {
        console.error(`Error getting score for ${objectiveId}:`, error);
        return 0;
    }
}

// Function to set score
function setScore(player, objectiveId, value) {
    try {
        const objective = world.scoreboard.getObjective(objectiveId);
        if (!objective) {
            console.warn(`Objective ${objectiveId} not found`);
            return;
        }
        objective.setScore(player, value);
    } catch (error) {
        console.error(`Error setting score for ${objectiveId}:`, error);
    }
}

// Add the getClanMembers function
function getClanMembers(clan) {
    if (!clan || !clan.members) {
        console.warn("Invalid clan data in getClanMembers");
        return [];
    }

    // Get online members
    const onlineMembers = clan.members
        .map(memberName => findPlayerByName(memberName))
        .filter(player => player !== null);

    return onlineMembers;
}

export async function handleClanWarSurrender(player) {
    try {
        if (!player) {
            console.warn('handleClanWarSurrender called with no player');
            return;
        }

        // Get player's clan name
        const playerClanName = Database.get(config.clanNameKey, player);
        if (!playerClanName) {
            player.sendMessage('§cYou are not in a clan!');
            return;
        }

        // Get clan data to check ownership
        const clanData = getClanData(playerClanName);
        if (!clanData) {
            player.sendMessage('§cError: Could not find clan data.');
            return;
        }

        // Check if the player is the clan owner
        if (clanData.owner !== player.name) {
            player.sendMessage('§cOnly the clan owner can surrender!');
            return;
        }

        console.log(`Clan owner ${player.name} from clan ${playerClanName} is attempting to surrender`);

        let warFound = false;
        for (const [warId, warData] of activeWarData) {
            const isClan1 = warData.clan1 === playerClanName;
            const isClan2 = warData.clan2 === playerClanName;

            if (isClan1 || isClan2) {
                warFound = true;
                const surrenderingClan = isClan1 ? warData.clan1 : warData.clan2;
                const winningClan = isClan1 ? warData.clan2 : warData.clan1;

                // Set surrender flag
                warData.surrendered = true;
                activeWarData.set(warId, warData);
                saveWarDataToDatabase();

                // Determine phase-specific message
                const phaseMessage = !warData.warStarted ? 
                    "during preparation phase" : 
                    `after ${formatTimeDisplay(Math.floor((Date.now() - warData.startTime) / 1000))}`;

                // Create surrender message
                const surrenderMessage = 
                    `§c=== WAR SURRENDER ===\n` +
                    `§e${surrenderingClan} §chas surrendered ${phaseMessage}!\n` +
                    `§a${winningClan} §ehas won the war!\n` +
                    `§6Bet Amount: §f$${warData.betAmount}`;

                // Notify all members of both clans
                const allMembers = [
                    ...getClanMembers(getClanData(surrenderingClan)), 
                    ...getClanMembers(getClanData(winningClan))
                ];

                allMembers.forEach(member => {
                    member.sendMessage(surrenderMessage);
                    member.runCommand(`playsound @s random.levelup`);
                    member.onScreenDisplay.setActionBar(`§c${surrenderingClan} surrendered! §a${winningClan} wins!`);
                });

                // Stop any ongoing timers
                if (warData.warInterval) {
                    system.clearRun(warData.warInterval);
                }
                if (warData.countdownInterval) {
                    system.clearRun(warData.countdownInterval);
                }

                // Handle rewards and bet distribution
                distributeClanWarRewards(getClanData(winningClan), getClanData(surrenderingClan), warData.betAmount);

                // Update clan stats
                updateClanStats(getClanData(winningClan), true);
                updateClanStats(getClanData(surrenderingClan), false);

                // Log the surrender
                console.log(`War ${warId} ended by surrender: ${surrenderingClan} surrendered to ${winningClan}`);

                // Cleanup the war data
                cleanupClanWar(warId);
                activeWarData.delete(warId);
                activeWars.forEach((value, key) => {
                    if (value.warId === warId) {
                        activeWars.delete(key);
                    }
                });

                break;
            }
        }

        if (!warFound) {
            player.sendMessage('§cYou are not in an active clan war!');
         
        }

    } catch (error) {
        console.error('Error in handleClanWarSurrender:', error);
        player.sendMessage('§cAn error occurred while processing the surrender.');
    }
}

// Update the chat handler to use tags
world.beforeEvents.chatSend.subscribe((eventData) => {
    const player = eventData.sender;
    const message = eventData.message.toLowerCase();

    // Check for surrender command
    if (message === '!surrender' || message === '!s') {
        eventData.cancel = true; // Cancel the message from being broadcast
        handleClanWarSurrender(player); // Directly call surrender handler
    }
});

// Update endClanWar to clean up database
function endClanWar(warId, winningClan, losingClan) {
    try {
        const warData = activeWarData.get(warId);
        if (!warData) return;

        // Clear any existing timers
        if (warData.warInterval) {
            system.clearRun(warData.warInterval);
        }

        // Handle rewards and notifications
        distributeClanWarRewards(getClanData(winningClan), getClanData(losingClan), warData.betAmount);
        updateClanStats(getClanData(winningClan), true);
        updateClanStats(getClanData(losingClan), false);

        // Clean up data
        activeWarData.delete(warId);
        surrenderVotes.delete(warId);
        saveWarDataToDatabase();

        // Notify players
        const allWarPlayers = world.getAllPlayers().filter(p => 
            warData.clan1Members.includes(p.name) || warData.clan2Members.includes(p.name)
        );

        allWarPlayers.forEach(p => {
            p.sendMessage(`§a⚔ War ended! ${winningClan} defeats ${losingClan}!`);
            p.sendMessage(`§eFinal score: ${warData.clan1Kills}-${warData.clan2Kills}`);
        });

    } catch (error) {
        console.error('Error in endClanWar:', error);
    }
}

// Add helper function to cleanup war data
function cleanupWar(warId) {
    const warData = activeWarData.get(warId);
    if (!warData) return;

    // Clear interval if it exists
    if (warData.warInterval) {
        system.clearRun(warData.warInterval);
    }

    // Clear player war associations
    [...warData.clan1Members, ...warData.clan2Members].forEach(memberName => {
        activeWars.delete(memberName);
    });

    // Clear war data
    activeWarData.delete(warId);
    surrenderVotes.delete(warId);
}







// Add helper function to get clan data
function getClanData(clanName) {
    try {
        if (!clanName) return null;

        const allClans = Database.entries()
            .filter(entry => 
                entry.key.startsWith(config.clansDBKey) && 
                !entry.key.includes('war_') && 
                !entry.key.includes('activeWar') &&
                !entry.key.includes('_funds')
            )
            .map(entry => {
                try {
                    return {
                        key: entry.key,
                        data: JSON.parse(entry.value)
                    };
                } catch (error) {
                    console.warn(`Invalid JSON in clan data for key ${entry.key}`);
                    return null;
                }
            })
            .filter(entry => entry !== null && entry.data.name === clanName);

        if (allClans.length === 0) return null;

        const clan = allClans[0].data;
        clan.inviteCode = allClans[0].key.split('.')[1];
        return clan;
    } catch (error) {
        console.error('Error in getClanData:', error);
        return null;
    }
}

// Update the startClanWar function
function startClanWar(clan1, clan2, betAmount, warDuration, prepTime = 15) {  // Added prepTime parameter
    try {
        if (!clan1 || !clan2 || clan1.name === clan2.name) {
            console.error("Invalid clan data or same clans in war");
            return;
        }

        // Create scoreboard for clan war kills if it doesn't exist
        createScoreboardIfNotExists("clanWarKills", "Clan War Kills");

        const allPlayers = world.getAllPlayers();
        const clan1Members = allPlayers.filter(p => Database.get(config.clanNameKey, p) === clan1.name);
        const clan2Members = allPlayers.filter(p => Database.get(config.clanNameKey, p) === clan2.name);

        const warId = `war_${Date.now()}_${Math.floor(Math.random() * 1000)}`;
        
        // Store war data globally with prepTime
        const warData = {
            warId: warId,
            clan1: clan1.name,
            clan2: clan2.name,
            clan1Members: clan1Members.map(m => m.name),
            clan2Members: clan2Members.map(m => m.name),
            betAmount: betAmount,
            duration: warDuration,
            startTime: Date.now(),
            prepTime: prepTime,
            warStarted: false,
            clan1Kills: 0,
            clan2Kills: 0
        };
        
        activeWarData.set(warId, warData);

        // Store war data for each player
        [...clan1Members, ...clan2Members].forEach(member => {
            if (member && member.isValid()) {
                setScore(member, "clanWarKills", 0);
                activeWars.set(member.name, {
                    warId: warId,
                    clan: Database.get(config.clanNameKey, member)
                });
            }
        });

        // Start countdown with prepTime
        startWarCountdown(warId, warData);
    } catch (error) {
        console.error("Error in startClanWar:", error);
    }
}

// Also update the acceptWarInvitation function to pass the prepTime
async function acceptWarInvitation(player, request) {
    try {
        const playerClanName = Database.get(config.clanNameKey, player);
        if (!playerClanName) {
            player.sendMessage("§cYou must be in a clan to accept war invitations.");
            return;
        }

        // Get both clans' data
        const invitingClanData = getClanData(request.invitingClan);
        const acceptingClanData = getClanData(playerClanName);

        if (!invitingClanData || !acceptingClanData) {
            player.sendMessage("§cError: Could not retrieve clan data.");
            return;
        }

        // Start the clan war with proper clan data and prep time
        startClanWar(invitingClanData, acceptingClanData, request.betAmount, request.warDuration, request.prepTime);

        // Notify both clans
        const allPlayers = world.getAllPlayers();
        allPlayers.forEach(p => {
            const pClanName = Database.get(config.clanNameKey, p);
            if (pClanName === invitingClanData.name || pClanName === acceptingClanData.name) {
                p.sendMessage(`§a§l⚔ Clan war between ${invitingClanData.name} and ${acceptingClanData.name} has begun!`);
                p.sendMessage(`§ePreparing for battle: ${formatTimeDisplay(request.prepTime)}`);
                p.playSound("random.levelup");
            }
        });
    } catch (error) {
        console.error("Error in acceptWarInvitation:", error);
        player.sendMessage("§cAn error occurred while accepting the war invitation.");
    }
}
function startWarCountdown(warId, warData) {
    // If this is a reload, use the existing countdown start time
    if (!warData.countdownStartTime) {
        warData.countdownStartTime = Date.now();
    }
    warData.countdownComplete = false;
    warData.surrendered = false;
    activeWarData.set(warId, warData);
    
    saveWarDataToDatabase();

    const countdownInterval = system.runInterval(() => {
        // Get fresh war data each tick to check for surrender
        const currentWarData = activeWarData.get(warId);
        if (!currentWarData || currentWarData.surrendered) {
            system.clearRun(countdownInterval);
            return;
        }

        // Calculate remaining time based on the original start time
        const timeElapsed = Math.floor((Date.now() - currentWarData.countdownStartTime) / 1000);
        const timeRemaining = Math.max(0, currentWarData.prepTime - timeElapsed);
        
        if (timeRemaining > 0) {
            const currentPlayers = world.getAllPlayers();
            const currentClan1Members = currentPlayers.filter(p => currentWarData.clan1Members.includes(p.name));
            const currentClan2Members = currentPlayers.filter(p => currentWarData.clan2Members.includes(p.name));

            [...currentClan1Members, ...currentClan2Members].forEach(member => {
                if (member && member.isValid()) {
                    // Format time display
                    let timeDisplay;
                    if (timeRemaining >= 3600) {
                        timeDisplay = `${Math.floor(timeRemaining / 3600)}h ${Math.floor((timeRemaining % 3600) / 60)}m`;
                    } else if (timeRemaining >= 60) {
                        timeDisplay = `${Math.floor(timeRemaining / 60)}m ${timeRemaining % 60}s`;
                    } else {
                        timeDisplay = `${timeRemaining}s`;
                    }

                    member.onScreenDisplay.setActionBar(
                        `§eClan War Preparation: §f${timeDisplay}\n` +
                        `§b${currentWarData.clan1} §fvs §b${currentWarData.clan2}\n` +
                        `§6Get ready for battle!`
                    );

                    if (timeRemaining <= 5 || 
                        (timeRemaining <= 60 && timeRemaining % 10 === 0) ||
                        (timeRemaining <= 300 && timeRemaining % 60 === 0)) {
                        member.playSound("note.pling");
                    }
                }
            });
        } else {
            system.clearRun(countdownInterval);
            
            // Check again for surrender before starting war
            if (!currentWarData.surrendered) {
                // Update war data
                currentWarData.countdownComplete = true;
                activeWarData.set(warId, currentWarData);
                saveWarDataToDatabase();
                
                // Notify all participants
                const currentPlayers = world.getAllPlayers();
                const warParticipants = currentPlayers.filter(p => 
                    currentWarData.clan1Members.includes(p.name) || 
                    currentWarData.clan2Members.includes(p.name)
                );

                warParticipants.forEach(member => {
                    member.sendMessage("§a§l⚔ The Clan War has begun! ⚔");
                    member.playSound("random.levelup");
                });

                startWarTimer(warId, currentWarData);
            }
        }
    }, 20);

    warData.countdownInterval = countdownInterval;
    activeWarData.set(warId, warData);
}

// Update startWarTimer to save data after changes
function startWarTimer(warId, warData) {
    warData.warStarted = true;
    warData.startTime = Date.now();

    
    saveWarDataToDatabase();

    const warInterval = system.runInterval(() => {
        const timeRemaining = warData.duration - Math.floor((Date.now() - warData.startTime) / 1000);
        
        if (timeRemaining > 0) {
            const currentPlayers = world.getAllPlayers();
            const currentClan1Members = currentPlayers.filter(p => warData.clan1Members.includes(p.name));
            const currentClan2Members = currentPlayers.filter(p => warData.clan2Members.includes(p.name));

            [...currentClan1Members, ...currentClan2Members].forEach(member => {
                if (member && member.isValid()) {
                    member.onScreenDisplay.setActionBar(
                        `§b${warData.clan1}: §a${warData.clan1Kills} §f| ${formatTime(timeRemaining)} | §b${warData.clan2}: §c${warData.clan2Kills}`
                    );
                }
            });
        } else {
            system.clearRun(warInterval);
            
            if (warData.clan1Kills === warData.clan2Kills) {
                handleWarDraw(warData);
            } else {
                const winningClan = warData.clan1Kills > warData.clan2Kills ? warData.clan1 : warData.clan2;
                const losingClan = warData.clan1Kills > warData.clan2Kills ? warData.clan2 : warData.clan1;
                endClanWar(warId, winningClan, losingClan);
            }
        }
    }, 20);

    warData.warInterval = warInterval;
    activeWarData.set(warId, warData);
    
}

world.afterEvents.entityDie.subscribe(async (data) => {
    const deadEntity = data.deadEntity;
    const damagingEntity = data.damageSource.damagingEntity;

    if (deadEntity?.typeId === "minecraft:player" && damagingEntity?.typeId === "minecraft:player") {
        try {
            const killerWarData = activeWars.get(damagingEntity.name);
            const victimWarData = activeWars.get(deadEntity.name);

            if (killerWarData?.warId && victimWarData?.warId && killerWarData.warId === victimWarData.warId) {
                const warData = activeWarData.get(killerWarData.warId);
                if (!warData || !warData.warStarted) return;

                // Increment killer's personal kills
                const currentKills = getScore(damagingEntity, "clanWarKills") || 0;
                setScore(damagingEntity, "clanWarKills", currentKills + 1);

                // Update clan kills in war data
                if (warData.clan1Members.includes(damagingEntity.name)) {
                    warData.clan1Kills++;
                } else if (warData.clan2Members.includes(damagingEntity.name)) {
                    warData.clan2Kills++;
                }

                // Save war data to database
                const warDataToSave = {
                    ...warData,
                    clan1Scores: {},
                    clan2Scores: {}
                };

                // Save individual scores for clan 1
                warData.clan1Members.forEach(memberName => {
                    const player = world.getAllPlayers().find(p => p.name === memberName);
                    if (player) {
                        warDataToSave.clan1Scores[memberName] = getScore(player, "clanWarKills") || 0;
                    }
                });

                // Save individual scores for clan 2
                warData.clan2Members.forEach(memberName => {
                    const player = world.getAllPlayers().find(p => p.name === memberName);
                    if (player) {
                        warDataToSave.clan2Scores[memberName] = getScore(player, "clanWarKills") || 0;
                    }
                });

                // Save to database
                Database.set(`warData_${warData.warId}`, JSON.stringify(warDataToSave));

                // Update the war data in the global map
                activeWarData.set(killerWarData.warId, warData);

                // Notify players
                damagingEntity.sendMessage(`§a+1 Kill for clan ${killerWarData.clan}!`);
                deadEntity.sendMessage(`§cYour clan lost a point!`);

                // Force action bar update for all participants
                const allPlayers = world.getAllPlayers();
                const clan1Members = allPlayers.filter(p => warData.clan1Members.includes(p.name));
                const clan2Members = allPlayers.filter(p => warData.clan2Members.includes(p.name));
                
                const timeRemaining = warData.duration - Math.floor((Date.now() - warData.startTime) / 1000);
                updateWarActionBar(clan1Members, clan2Members, warData, timeRemaining);

                // Save to database after all updates
                saveWarDataToDatabase();
            }
        } catch (error) {
            console.error('Error handling war kill:', error);
        }
    }
});

// Update the updateWarActionBar function
function updateWarActionBar(clan1Members, clan2Members, warData, timeRemaining) {
    try {
        // Use the stored war data scores directly
        const clan1Score = warData.clan1Kills;
        const clan2Score = warData.clan2Kills;

        // Update action bar for all members of both clans
        [...clan1Members, ...clan2Members].forEach(member => {
            if (member && member.isValid()) {
                member.onScreenDisplay.setActionBar(
                    `§b${warData.clan1}: §a${clan1Score} §f| ${formatTime(timeRemaining)} | §b${warData.clan2}: §c${clan2Score}`
                );
            }
        });
    } catch (error) {
        console.error('Error in updateWarActionBar:', error);
    }
}



function distributeClanWarRewards(winningClan, losingClan, betAmount) {
    try {
        if (!winningClan || !losingClan || !betAmount) {
            console.warn('Invalid parameters in distributeClanWarRewards');
            return;
        }

        // Get clan funds keys
        const winningClanFundsKey = `${config.clansDBKey}.${winningClan.inviteCode}_funds`;
        const losingClanFundsKey = `${config.clansDBKey}.${losingClan.inviteCode}_funds`;

        // Get current funds
        const currentWinnerFunds = parseInt(Database.get(winningClanFundsKey) || "0", 10);
        const currentLoserFunds = parseInt(Database.get(losingClanFundsKey) || "0", 10);

        // Winner gets their bet back plus the loser's bet
        const winnerNewBalance = currentWinnerFunds + betAmount;
        const loserNewBalance = currentLoserFunds - betAmount;

        // Update funds in database
        Database.set(winningClanFundsKey, winnerNewBalance.toString());
        Database.set(losingClanFundsKey, loserNewBalance.toString());

        // Update clan-wide scores
        const overworld = world.getDimension("overworld");
        
        // Add win for winner
        overworld.runCommandAsync(`scoreboard players add "${winningClan.name}" clanwins 1`);
        
        // Check current wins before deducting
        const winsObj = world.scoreboard.getObjective('clanwins');
        const currentWins = winsObj ? (winsObj.getScore(losingClan.name) || 0) : 0;
        
        // Add loss and only deduct win if they have wins
        overworld.runCommandAsync(`scoreboard players add "${losingClan.name}" clanlost 1`);
        if (currentWins > 0) {
            overworld.runCommandAsync(`scoreboard players remove "${losingClan.name}" clanwins 1`);
        }
        
        // Add total games played for both clans
        overworld.runCommandAsync(`scoreboard players add "${winningClan.name}" ${config.TOTAL_GAMES_SCORE} 1`);
        overworld.runCommandAsync(`scoreboard players add "${losingClan.name}" ${config.TOTAL_GAMES_SCORE} 1`);

        // Show animated titles for both clans
        const allPlayers = world.getAllPlayers();
        allPlayers.forEach(p => {
            const pClanName = Database.get(config.clanNameKey, p);
            if (pClanName === winningClan.name) {
                p.sendMessage(`§aYour clan won §6$${betAmount}§a from the war!`);
                p.runCommandAsync("playsound random.levelup @s");
                showWinnerAnimation(p, winningClan.name);
            } else if (pClanName === losingClan.name) {
                // Adjust message based on whether wins were deducted
                const lossMessage = currentWins > 0 
                    ? `§cYour clan lost §6$${betAmount}§c from the war and 1 win point.`
                    : `§cYour clan lost §6$${betAmount}§c from the war.`;
                p.sendMessage(lossMessage);
                showLoserAnimation(p, losingClan.name);
            }
        });

        // Log the transaction
        console.log(`War Rewards Distribution:
            Winner (${winningClan.name}): +${betAmount} (New balance: ${winnerNewBalance})
            Loser (${losingClan.name}): -${betAmount} (New balance: ${loserNewBalance})`);

        // Update clan stats
        updateClanStats(winningClan, true);
        updateClanStats(losingClan, false);

    } catch (error) {
        console.error('Error in distributeClanWarRewards:', error);
    }
}

function showWinnerAnimation(player, clanName) {
    let step = 0;
    const winsObj = world.scoreboard.getObjective('clanwins');
    const wins = winsObj ? (winsObj.getScore(clanName) || 0) : 0;
    const oldRank = getClanRankFromWins(wins - 1); // Previous rank
    const newRank = getClanRankFromWins(wins);      // New rank after win

    const animationSteps = [
        { rank: oldRank, stars: "✦✦✦✦✦", color: "§7" },     // Current rank max stars
        { rank: oldRank, stars: "✦✦✦✦✦", color: "§e" },     // Flash current
        { rank: newRank, stars: "✦✧✧✧✧", color: "§6" },     // New rank first star
        { rank: newRank, stars: "✦✦✧✧✧", color: "§6" },     // Adding stars
        { rank: newRank, stars: "✦✦✦✧✧", color: "§6" },
        { rank: newRank, stars: "✦✦✦✦✧", color: "§6" },
        { rank: newRank, stars: "✦✦✦✦✦", color: "§6" }      // Final state
    ];

    const interval = system.runInterval(() => {
        if (step >= animationSteps.length) {
            system.clearRun(interval);
            // Get updated rank and stars after animation
            const updatedWins = winsObj ? (winsObj.getScore(clanName) || 0) : 0;
            const updatedRank = getClanRankFromWins(updatedWins);
            let stars = calculateStarsForRank(updatedWins);
            
            player.onScreenDisplay.setActionBar(
                `§l${clanName} CLAN\n` +
                `§7[${updatedRank}]\n` +
                `§7${stars}`
            );
            return;
        }

        const current = animationSteps[step];
        player.onScreenDisplay.setActionBar(
            `§l${clanName} CLAN\n` +
            `${current.color}[${current.rank}]\n` +
            `${current.color}${current.stars}`
        );

        step++;
    }, 10);
}

function showLoserAnimation(player, clanName) {
    let step = 0;
    const winsObj = world.scoreboard.getObjective('clanwins');
    const wins = winsObj ? (winsObj.getScore(clanName) || 0) : 0;
    const oldRank = getClanRankFromWins(wins + 1); // Previous rank
    const newRank = getClanRankFromWins(wins);     // New rank after loss

    const animationSteps = [
        { rank: oldRank, stars: "✦✦✦✦✦", color: "§7" },     // Start with full stars
        { rank: oldRank, stars: "✦✦✦✦§8��", color: "§7" },   // Losing stars
        { rank: oldRank, stars: "✦✦✦§8✦✦", color: "§7" },
        { rank: oldRank, stars: "✦✦§8✦✦✦", color: "§7" },
        { rank: oldRank, stars: "✦§8✦✦✦✦", color: "§7" },
        { rank: newRank, stars: "✦✦✦", color: "§7" }        // New rank with default stars
    ];

    const interval = system.runInterval(() => {
        if (step >= animationSteps.length) {
            system.clearRun(interval);
            // Get updated rank and stars after animation
            const updatedWins = winsObj ? (winsObj.getScore(clanName) || 0) : 0;
            const updatedRank = getClanRankFromWins(updatedWins);
            let stars = calculateStarsForRank(updatedWins);
            
            player.onScreenDisplay.setActionBar(
                `§l${clanName} CLAN\n` +
                `§7[${updatedRank}]\n` +
                `§7${stars}`
            );
            return;
        }

        const current = animationSteps[step];
        player.onScreenDisplay.setActionBar(
            `§l${clanName} CLAN\n` +
            `${current.color}[${current.rank}]\n` +
            `${current.color}${current.stars}`
        );

        step++;
    }, 10);
}

// Helper function to calculate stars based on wins
function calculateStarsForRank(wins) {
    // Legend and above
    if (wins >= 87) return "✦✦✦✦✦";
    if (wins >= 82) return "✦✦✦✦✦";
    if (wins >= 77) return "✦✦✦✦✦";
    if (wins >= 72) return "✦✦✦✦✦";
    
    // Diamond tier
    if (wins >= 67) return "✦✦✦✦✦";
    if (wins >= 62) return "✦✦✦✦✦";
    if (wins >= 57) return "✦✦✦✦✦";
    
    // Platinum tier
    if (wins >= 52) return "✦✦✦✦✦";
    if (wins >= 47) return "✦✦✦✦✦";
    if (wins >= 42) return "✦✦✦✦✦";
    
    // Gold tier
    if (wins >= 37) return "✦✦✦✦✦";
    if (wins >= 32) return "✦✦✦✦✦";
    if (wins >= 27) return "✦✦✦✦✦";
    
    // Silver tier
    if (wins >= 22) return "✦✦✦✦✦";  // Silver III
    if (wins >= 17) return "✦✦✦✦✦";  // Silver II
    if (wins >= 16) return "✦✦✦✦✦";  // Silver I (max stars)
    if (wins >= 12) {                  // Silver I (progressive stars)
        const extraStars = wins - 12;
        return "✦".repeat(3 + extraStars);
    }
    
    // Bronze tier
    if (wins >= 9) return "✦✦✦✦✦";   // Bronze III
    if (wins >= 6) return "✦✦✦✦";     // Bronze II
    if (wins >= 3) return "✦✦✦";      // Bronze I
    
    // Rookie
    return "✦✦✦";
}
// Add the updateClanStats function
function updateClanStats(clan, isWinner) {
    try {
        if (!clan || !clan.name) return;

        const clanData = Database.entries()
            .find(entry => entry.value && JSON.parse(entry.value).name === clan.name);
        
        if (!clanData) return;

        const inviteCode = clanData.key.split('.')[1];
        const statsKey = `${config.clansDBKey}.${inviteCode}.stats`;
        
        // Get current stats
        let stats = {};
        try {
            const currentStats = Database.get(statsKey);
            stats = currentStats ? JSON.parse(currentStats) : { wins: 0, losses: 0, draws: 0 };
        } catch (error) {
            stats = { wins: 0, losses: 0, draws: 0 };
        }

        // Update stats based on result
        if (isWinner === null) {
            stats.draws = (stats.draws || 0) + 1;
        } else if (isWinner) {
            stats.wins = (stats.wins || 0) + 1;
        } else {
            stats.losses = (stats.losses || 0) + 1;
        }

        // Save updated stats
        Database.set(statsKey, JSON.stringify(stats));

    } catch (error) {
        console.error('Error in updateClanStats:', error);
    }
}

// Add this new confirmation function
function confirmLeaveClan(player, inviteCode, clan) {
    const ui = new ModalFormData()
        .title("Confirm Leave Clan")
        .textField("§c✦ Type 'yes' to confirm leaving the clan)", "Type here");

    ui.show(player).then(({ canceled, formValues }) => {
        if (canceled) return;

        const response = formValues[0].toLowerCase();
        const confirmResponses = ["yes", "oo", "sige"]; // Add more variations as needed

        if (confirmResponses.includes(response.toLowerCase())) {
            leaveClan(player, inviteCode);
            player.sendMessage("§aYou have left the clan.");
        } else {
            player.sendMessage("§eClan leave cancelled.");
        }
    });
}

// Function to handle kills and adjust scores
function handleKill(damagingEntity, deadEntity) {
    try {
        const killsDead = getScore(deadEntity, "kills") || 0;
        const killsKiller = getScore(damagingEntity, "kills") || 0;

        console.log(`Current kills - Dead: ${killsDead}, Killer: ${killsKiller}`);

        // If both players have fewer than 10 kills, killer gains +1 kill
        if (killsDead < 10 && killsKiller < 10) {
            setScore(damagingEntity, "kills", killsKiller + 1);
            damagingEntity.sendMessage(`§aYou gained 1 kill!`);
        } 
        // If both players have 11 or more kills, adjust based on the difference
        else if (killsKiller >= 10 && killsDead >= 10) {
            if (killsDead >= killsKiller) {
                // Killer gains 1 kill and dead player loses 10% of their kills
                setScore(damagingEntity, "kills", killsKiller + 1);
                const killsDeducted = Math.floor(killsDead * 0.10);
                setScore(deadEntity, "kills", killsDead - killsDeducted);
                damagingEntity.sendMessage(`§aYou gained 1 kill!`);
                deadEntity.sendMessage(`§cYou lost ${killsDeducted} kills.`);
            } else {
               
            }
        } 
        // If the dead player has 10 or more kills but the killer has fewer, the killer gains 1 kill
        else if (killsDead >= 10 && killsKiller < 10) {
            setScore(damagingEntity, "kills", killsKiller + 1);
            damagingEntity.sendMessage(`§aYou gained 1 kill!`);
        }

     

        // Update clan war scores if in a war
        updateClanWarScores(damagingEntity, deadEntity);

    } catch (error) {
        console.error('Error in handleKill:', error);
    }
}

function updateClanWarScores(killer, victim) {
    try {
        // Get clan names for both players
        const killerClan = Database.get(config.clanNameKey, killer);
        const victimClan = Database.get(config.clanNameKey, victim);

        if (!killerClan || !victimClan) return;

        // Check if there's an active war between these clans
        const warKey = `${config.clansDBKey}.activeWar.${killerClan}_${victimClan}`;
        const reverseWarKey = `${config.clansDBKey}.activeWar.${victimClan}_${killerClan}`;
        
        let warData = Database.get(warKey);
        if (!warData) {
            warData = Database.get(reverseWarKey);
        }

        if (!warData) return;

        try {
            warData = JSON.parse(warData);
        } catch (error) {
            console.error('Error parsing war data:', error);
            return;
        }

        // Update war kills
        if (!warData.kills) warData.kills = {};
        if (!warData.kills[killerClan]) warData.kills[killerClan] = 0;
        warData.kills[killerClan]++;

        // Store updated war data
        Database.set(warKey, JSON.stringify(warData));

        // Notify players about war kill
        killer.sendMessage(`§a+1 Kill for clan ${killerClan} in war against ${victimClan}!`);
        victim.sendMessage(`§c${victimClan} lost a point in war against ${killerClan}!`);

        // Broadcast war score
        broadcastWarScore(warData, killerClan, victimClan);

    } catch (error) {
        console.error('Error in updateClanWarScores:', error);
    }
}

// Helper function to get clan data by name
function getClanDataByName(clanName) {
    try {
        const allClans = Database.entries()
            .filter(entry => entry.key.startsWith(config.clansDBKey) && 
                   !entry.key.includes('war_') &&
                   !entry.key.includes('activeWar') &&
                   !entry.key.includes('_funds'))
            .find(entry => {
                try {
                    const data = JSON.parse(entry.value);
                    return data.name === clanName;
                } catch {
                    return false;
                }
            });

        return allClans ? JSON.parse(allClans.value) : null;
    } catch (error) {
        console.error('Error in getClanDataByName:', error);
        return null;
    }
}

// Function to broadcast war score
function broadcastWarScore(warData, clan1, clan2) {
    const score1 = warData.kills[clan1] || 0;
    const score2 = warData.kills[clan2] || 0;
    const message = `§6War Score: ${clan1}: §a${score1} §6vs ${clan2}: §c${score2}`;

    // Broadcast to all online players in both clans
    for (const player of world.getPlayers()) {
        const playerClan = Database.get(config.clanNameKey, player);
        if (playerClan === clan1 || playerClan === clan2) {
            player.sendMessage(message);
        }
    }
}

// Add this event listener for player deaths
world.afterEvents.entityDie.subscribe((event) => {
    const deadEntity = event.deadEntity;
    const damageSource = event.damageSource;
    const damagingEntity = damageSource.damagingEntity;

    // Check if this is a player vs player kill
    if (deadEntity?.typeId === "minecraft:player" && 
        damagingEntity?.typeId === "minecraft:player") {
        handleKill(damagingEntity, deadEntity);
    }
});



////////Knockback System ////////
////////////////////////////////


world.afterEvents.entityHitEntity.subscribe((event) => {
    const attacker = event.damagingEntity;
    const victim = event.hitEntity;
    
    if (!(attacker instanceof Player)) return;
    if (!attacker || !victim) return;

    const attackerLocation = attacker.location;
    const victimLocation = victim.location;

    let knockbackX = victimLocation.x - attackerLocation.x;
    let knockbackZ = victimLocation.z - attackerLocation.z;

    const magnitude = Math.sqrt(knockbackX * knockbackX + knockbackZ * knockbackZ);
    if (magnitude !== 0) {
        knockbackX /= magnitude;
        knockbackZ /= magnitude;
    }

    let horizontalKB = config.horizontalKB;
    let verticalKB = config.verticalKB;

    if (attacker.isSprinting) {
        horizontalKB *= config.sprintMultiplier;
    }

    victim.applyKnockback(knockbackX, knockbackZ, horizontalKB, verticalKB);
});

// Update the chat handler for surrender commands
world.beforeEvents.chatSend.subscribe((eventData) => {
    const player = eventData.sender;
    const message = eventData.message.toLowerCase();

    if (message === '!surrender' || message === '!s') {
        eventData.cancel = true;
        handleSurrenderCommand(player);
    } else if (message === '!agree') {
        eventData.cancel = true;
        handleAgreeCommand(player);
    }
});

// Function to handle the surrender command
function handleSurrenderCommand(player) {
    try {
        // Get player's clan name
        const playerClanName = Database.get(config.clanNameKey, player);
        if (!playerClanName) {
            player.sendMessage('§cYou are not in a clan!');
            return;
        }

        // Find active war for this clan
        let warData = null;
        let warId = null;

        for (const [wId, war] of activeWarData.entries()) {
            if (war.clan1 === playerClanName || war.clan2 === playerClanName) {
                warData = war;
                warId = wId;
                break;
            }
        }

        if (!warData) {
            player.sendMessage('§cYour clan is not in an active war!');
            return;
        }

        // Get current online clan members
        const onlineClanMembers = world.getAllPlayers().filter(p => 
            Database.get(config.clanNameKey, p) === playerClanName
        );

        // If player is alone, surrender immediately
        if (onlineClanMembers.length === 1) {
            const winningClan = warData.clan1 === playerClanName ? warData.clan2 : warData.clan1;
            
            // Notify all war participants
            const allWarPlayers = world.getAllPlayers().filter(p => 
                warData.clan1Members.includes(p.name) || warData.clan2Members.includes(p.name)
            );

            allWarPlayers.forEach(p => {
                p.sendMessage(`§c${playerClanName} has surrendered! §a${winningClan} wins the war!`);
                p.runCommandAsync("playsound random.levelup @s");
            });

            // End the war
            endClanWar(warId, winningClan, playerClanName);
            return;
        }

        // Initialize surrender votes if not exists
        if (!surrenderVotes.has(warId)) {
            surrenderVotes.set(warId, new Set());
        }

        // Add player's vote
        const votes = surrenderVotes.get(warId);
        if (votes.has(player.name)) {
            player.sendMessage('§cYou have already voted to surrender!');
            return;
        }

        votes.add(player.name);

        // Notify clan members
        onlineClanMembers.forEach(member => {
            if (member.name !== player.name) {
                member.sendMessage(`§e${player.name} wants to surrender! Type §6!agree §eto vote for surrender.`);
                member.sendMessage(`§e${votes.size}/${onlineClanMembers.length} votes needed type !agree to vote.`);
            }
        });

        player.sendMessage(`§aYou have voted to surrender. ${onlineClanMembers.length - votes.size} more votes needed.`);

    } catch (error) {
        console.error('Error in handleSurrenderCommand:', error);
        player.sendMessage('§cAn error occurred while processing surrender command.');
    }
}

// Function to handle the agree command
function handleAgreeCommand(player) {
    try {
        const playerClanName = Database.get(config.clanNameKey, player);
        if (!playerClanName) return;

        // Find active war and votes for this clan
        let warData = null;
        let warId = null;

        for (const [wId, war] of activeWarData.entries()) {
            if (war.clan1 === playerClanName || war.clan2 === playerClanName) {
                warData = war;
                warId = wId;
                break;
            }
        }

        if (!warData || !surrenderVotes.has(warId)) {
            player.sendMessage('§cThere is no active surrender vote!');
            return;
        }

        const votes = surrenderVotes.get(warId);
        if (votes.has(player.name)) {
            player.sendMessage('§cYou have already voted to surrender!');
            return;
        }

        // Add vote
        votes.add(player.name);

        // Get current online clan members
        const onlineClanMembers = world.getAllPlayers().filter(p => 
            Database.get(config.clanNameKey, p) === playerClanName
        );

        // Check if all members have voted
        if (votes.size === onlineClanMembers.length) {
            // Execute surrender
            const winningClan = warData.clan1 === playerClanName ? warData.clan2 : warData.clan1;

            // Notify all players in both clans
            const allWarPlayers = world.getAllPlayers().filter(p => 
                warData.clan1Members.includes(p.name) || warData.clan2Members.includes(p.name)
            );

            allWarPlayers.forEach(p => {
                p.sendMessage(`§c${playerClanName} has surrendered! §a${winningClan} wins the war!`);
                p.runCommandAsync("playsound random.levelup @s");
            });

            // End the war
            endClanWar(warId, winningClan, playerClanName);
            
            // Clear votes
            surrenderVotes.delete(warId);
        } else {
            // Notify remaining players
            onlineClanMembers.forEach(member => {
                member.sendMessage(`§e${player.name} has agreed to surrender! ${onlineClanMembers.length - votes.size} more votes needed.`);
            });
        }

    } catch (error) {
        console.error('Error in handleAgreeCommand:', error);
        player.sendMessage('§cAn error occurred while processing your vote.');
    }
}

// Add handleWarDraw function
function handleWarDraw(warData) {
    try {
        if (!warData) return;

        // Get clan data for both clans
        const clan1Data = getClanData(warData.clan1);
        const clan2Data = getClanData(warData.clan2);

        if (!clan1Data || !clan2Data) {
            console.error('Could not retrieve clan data for war draw');
            return;
        }

        // Return bet amounts to both clans
        const clan1FundsKey = `${config.clansDBKey}.${clan1Data.inviteCode}_funds`;
        const clan2FundsKey = `${config.clansDBKey}.${clan2Data.inviteCode}_funds`;

        const clan1CurrentFunds = parseInt(Database.get(clan1FundsKey) || "0", 10);
        const clan2CurrentFunds = parseInt(Database.get(clan2FundsKey) || "0", 10);

        // Return bet amounts
        Database.set(clan1FundsKey, (clan1CurrentFunds + warData.betAmount).toString());
        Database.set(clan2FundsKey, (clan2CurrentFunds + warData.betAmount).toString());

        // Notify all players in both clans
        const allWarPlayers = world.getAllPlayers().filter(p => 
            warData.clan1Members.includes(p.name) || warData.clan2Members.includes(p.name)
        );

        allWarPlayers.forEach(p => {
            const pClanName = Database.get(config.clanNameKey, p);
            p.sendMessage(`§eThe war ended in a draw! Final score: ${warData.clan1Kills}-${warData.clan2Kills}`);
            p.sendMessage(`§6Your bet amount of $${warData.betAmount} has been returned.`);
            p.runCommandAsync("playsound random.levelup @s");
        });

        // Update stats for both clans (optional - you might want to track draws separately)
        updateClanStats(clan1Data, null); // Pass null to indicate a draw
        updateClanStats(clan2Data, null);

    } catch (error) {
        console.error('Error in handleWarDraw:', error);
    }
}

// Add this function to update clan kills
function updateClanKills(clanName) {
    try {
        // Get clan data
        const allClans = Database.entries()
            .filter(entry => entry.key.startsWith(config.clansDBKey))
            .find(entry => {
                const data = JSON.parse(entry.value);
                return data.name === clanName;
            });

        if (!allClans) return;

        const clan = JSON.parse(allClans.value);
        const inviteCode = allClans.key.split('.')[1];

        // Update kills in clan data
        const killsKey = `${config.clansDBKey}.${inviteCode}.kills`;
        const currentKills = parseInt(Database.get(killsKey) || "0", 10);
        Database.set(killsKey, (currentKills + 1).toString());

        // Also update scoreboard
        world.getDimension("overworld").runCommandAsync(`scoreboard players add "${clanName}" clankills 1`);

    } catch (error) {
        console.error('Error in updateClanKills:', error);
    }
}

// Update your kill event handler to use the new function
world.afterEvents.entityDie.subscribe((event) => {
    const deadEntity = event.deadEntity;
    const killer = event.damageSource.damagingEntity;

    if (deadEntity?.typeId === "minecraft:player" && 
        killer?.typeId === "minecraft:player") {
        
        // Get killer's clan
        const killerClanName = Database.get(config.clanNameKey, killer);
        if (killerClanName) {
            updateClanKills(killerClanName);
        }

        // ... rest of your kill handling code ...
    }
});

// Add to your config object
const reportTypes = [
    "Inappropriate Clan Name",
    "Toxic Behavior",
    "Exploiting/Cheating",
    "Other"
];

// Function to handle clan reporting
function handleClanReport(player, targetClan) {
    const form = new ModalFormData()
        .title("Report Clan")
        .dropdown("Report Type", reportTypes)
        .textField("Describe the issue:", "Enter details here");

    form.show(player).then(response => {
        if (response.canceled || !response.formValues[1]) return;

        const reportType = reportTypes[response.formValues[0]];
        const reportDetails = response.formValues[1];
        
        // Create report notification
        const reportData = {
            type: 'clanReport',
            reporterId: player.name,
            clanName: targetClan.name,
            reportType: reportType,
            details: reportDetails,
            timestamp: Date.now()
        };

        // Generate unique report ID
        const reportId = `report_${Date.now()}_${Math.floor(Math.random() * 1000)}`;
        Database.set(`${config.notificationsDBKey}.admin.${reportId}`, JSON.stringify(reportData));

        player.sendMessage("§aYour report has been submitted to the administrators.");
    });
}

// Function to handle report notifications
function handleReportNotification(player, notification, reportData) {
    const ui = new CustomUI(1, `Clan Report: ${reportData.clanName}`);
    
    ui.body(
        `§eReporter: §f${reportData.reporterId}\n` +
        `§eType: §f${reportData.reportType}\n` +
        `§eDetails: §f${reportData.details}\n` +
        `§eTimestamp: §f${new Date(reportData.timestamp).toLocaleString()}`
    );

    ui.button("Inspect Clan", 'textures/ui/magnifying_glass', false, 0);
    ui.button("Delete Clan", 'textures/ui/realms_red_x', false, 1);
    ui.button("Remove Report", 'textures/ui/cancel', false, 2);
    ui.button("Send Warning", 'textures/ui/warning_alex', false, 3);

    ui.show(player).then(result => {
        if (!result || result.selection === null) return;

        const targetClan = getClanData(reportData.clanName);
        if (!targetClan && result.selection !== 2) {
            player.sendMessage("§cClan no longer exists!");
            return;
        }

        switch (result.selection) {
            case 0: // Inspect
                showClanOptionsForm(player, targetClan);
                break;
            case 1: // Delete Clan
                confirmDeleteClan(player, targetClan);
                break;
            case 2: // Remove Report
                Database.delete(notification.key);
                player.sendMessage("§aReport has been removed.");
                break;
            case 3: // Warning
                sendClanWarning(player, targetClan, reportData);
                break;
        }
    });
}

// Function to send warning to clan
function sendClanWarning(admin, targetClan, reportData) {
    const form = new ModalFormData()
        .title("Send Warning")
        .textField("Warning Message:", "Enter warning message");

    form.show(admin).then(response => {
        if (response.canceled || !response.formValues[0]) return;

        const warningData = {
            type: 'clanWarning',
            message: response.formValues[0],
            adminName: admin.name,
            timestamp: Date.now()
        };

        const warningId = `warning_${Date.now()}_${Math.floor(Math.random() * 1000)}`;
        Database.set(`${config.notificationsDBKey}.${targetClan.name}.${warningId}`, JSON.stringify(warningData));

        // Notify online clan members
        for (const player of world.getAllPlayers()) {
            if (Database.get(config.clanNameKey, player) === targetClan.name) {
                player.sendMessage("§c⚠ Your clan has received a warning from an administrator!");
            }
        }

        admin.sendMessage("§aWarning has been sent to the clan.");
    });
}

// Function to handle warning notifications
function handleWarningNotification(player, notification, warningData) {
    const ui = new CustomUI(1, "Administrative Warning");
    
    ui.body(
        `§cWarning from Administrator: §f${warningData.adminName}\n\n` +
        `§eMessage: §f${warningData.message}\n\n` +
        `§7Received: ${new Date(warningData.timestamp).toLocaleString()}`
    );

    ui.button("Acknowledge & Delete", 'textures/ui/confirm', false, 0);

    ui.show(player).then(result => {
        if (!result || result.selection !== 0) return;
        
        Database.delete(notification.key);
        player.sendMessage("§aWarning acknowledged and removed.");
    });
}


// Function to show deposit form
function showDepositForm(player, clan) {
    const moneyObj = world.scoreboard.getObjective(config.MONEY_SCORE);
    const playerBalance = moneyObj ? moneyObj.getScore(player) : 0;

    const form = new ModalFormData()
        .title("Deposit Clan Funds")
        .textField(
            `Your balance: $${playerBalance}\nEnter amount to deposit:`,
            "Enter amount",
            ""
        );

    form.show(player).then((response) => {
        if (!response || response.canceled || !response.formValues) return;

        const amount = parseInt(response.formValues[0]);
        if (isNaN(amount) || amount <= 0) {
            player.sendMessage("§cPlease enter a valid amount.");
            return;
        }

        if (amount > playerBalance) {
            player.sendMessage("§cYou don't have enough money.");
            return;
        }

        // Convert player money to clan funds
        const currentFunds = getClanFunds(clan);
        const inviteCode = Database.get(config.inviteCodeKey, player);
        
        // Update player's money
        moneyObj.setScore(player, playerBalance - amount);
        
        // Update clan funds
        Database.set(`${config.clansDBKey}.${inviteCode}_funds`, (currentFunds + amount).toString());

        player.sendMessage(`§aSuccessfully deposited $${amount} to clan funds.`);
        
        // Create and send notification to all clan members
        const notificationData = {
            type: 'fundTransaction',
            transactionType: 'deposit',
            playerName: player.name,
            amount: amount,
            timestamp: Date.now()
        };

        // Store notification for each clan member
        clan.members.forEach(memberName => {
            const notificationKey = `${config.notificationsDBKey}.${clan.name}.fund_${Date.now()}_${Math.random().toString(36).substr(2, 9)}`;
            Database.set(notificationKey, JSON.stringify(notificationData));
        });

        // Notify online members
        for (const member of world.getAllPlayers()) {
            if (Database.get(config.clanNameKey, member) === clan.name && member.name !== player.name) {
                member.sendMessage(`§a${player.name} deposited $${amount} to clan funds.`);
            }
        }
    }).catch(error => {
        console.error('Error in showDepositForm:', error);
        player.sendMessage('§cAn error occurred while processing your deposit.');
    });
}

// Function to show withdraw form
function showWithdrawForm(player, clan) {
    // Get player's rank in the clan
    const playerRank = getRole(player);
    const withdrawLimits = {
        'owner': Infinity,
        'admin': 100000,
        'officer': 50000,
        'member': 25000,
        'recruit': 10000
    };

    const withdrawLimit = withdrawLimits[playerRank] || 0;
    const clanFunds = getClanFunds(clan);
    
    const form = new ModalFormData()
        .title("Withdraw Clan Funds")
        .textField(
            `Clan funds: $${clanFunds}\n` +
            `Your withdrawal limit: ${withdrawLimit === Infinity ? 'Unlimited' : '$' + withdrawLimit}\n` +
            `Enter amount to withdraw:`,
            "Enter amount",
            ""
        );

    form.show(player).then((response) => {
        if (!response || response.canceled || !response.formValues) return;

        const amount = parseInt(response.formValues[0]);
        if (isNaN(amount) || amount <= 0) {
            player.sendMessage("§cPlease enter a valid amount.");
            return;
        }

        // Check withdrawal limit
        if (amount > withdrawLimit) {
            player.sendMessage(`§cYour rank (${playerRank}) can only withdraw up to $${withdrawLimit}.`);
            return;
        }

        if (amount > clanFunds) {
            player.sendMessage("§cNot enough clan funds.");
            return;
        }

        const moneyObj = world.scoreboard.getObjective(config.MONEY_SCORE);
        const playerBalance = moneyObj ? moneyObj.getScore(player) : 0;
        const inviteCode = Database.get(config.inviteCodeKey, player);

        // Update clan funds
        Database.set(`${config.clansDBKey}.${inviteCode}_funds`, (clanFunds - amount).toString());
        
        // Update player's money
        moneyObj.setScore(player, playerBalance + amount);

        player.sendMessage(`§aSuccessfully withdrew $${amount} from clan funds.`);
        
        // Create and send notification to all clan members
        const notificationData = {
            type: 'fundTransaction',
            transactionType: 'withdraw',
            playerName: player.name,
            amount: amount,
            timestamp: Date.now(),
            rank: playerRank
        };

        // Store notification for each clan member
        clan.members.forEach(memberName => {
            const notificationKey = `${config.notificationsDBKey}.${clan.name}.fund_${Date.now()}_${Math.random().toString(36).substr(2, 9)}`;
            Database.set(notificationKey, JSON.stringify(notificationData));
        });

        // Notify online members
        for (const member of world.getAllPlayers()) {
            if (Database.get(config.clanNameKey, member) === clan.name && member.name !== player.name) {
                member.sendMessage(`§e${player.name} (${playerRank}) withdrew $${amount} from clan funds.`);
            }
        }

        // Log the transaction
        logClanTransaction(clan.name, player.name, 'withdraw', amount, playerRank);

    }).catch(error => {
        console.error('Error in showWithdrawForm:', error);
        player.sendMessage('§cAn error occurred while processing your withdrawal.');
    });
}

// Add transaction logging function
function logClanTransaction(clanName, playerName, type, amount, rank) {
    try {
        const logKey = `${config.clansDBKey}.${clanName}.transactions`;
        let transactions = [];
        try {
            const existingLogs = Database.get(logKey);
            if (existingLogs) {
                transactions = JSON.parse(existingLogs);
            }
        } catch (error) {
            console.warn('Error parsing existing transaction logs');
        }

        // Add new transaction
        transactions.push({
            playerName,
            type,
            amount,
            rank,
            timestamp: Date.now()
        });

        // Keep only last 50 transactions
        if (transactions.length > 50) {
            transactions = transactions.slice(-50);
        }

        // Save updated logs
        Database.set(logKey, JSON.stringify(transactions));

    } catch (error) {
        console.error('Error logging clan transaction:', error);
    }
}

// Update manageClanFunds function to show withdraw button for everyone
function manageClanFunds(player) {
    const playerClanName = Database.get(config.clanNameKey, player);
    const clan = getClanData(playerClanName);
    
    if (!clan) {
        player.sendMessage("§cError: Could not find clan data.");
        return;
    }

    const clanFunds = getClanFunds(clan);
    const moneyObj = world.scoreboard.getObjective(config.MONEY_SCORE);
    const playerBalance = moneyObj ? moneyObj.getScore(player) : 0;
    const playerRank = getRole(player);
    const withdrawLimit = {
        'owner': 'Unlimited',
        'admin': '$100,000',
        'officer': '$50,000',
        'member': '$25,000',
        'recruit': '$10,000'
    }[playerRank] || '$0';

    const ui = new CustomUI(1, "Clan Funds Management");
    
    ui.body(
        `§eClan Funds: §a$${clanFunds}\n` +
        `§eYour Balance: §a$${playerBalance}\n` +
        `§eYour Rank: §f${playerRank}\n` +
        `§eWithdrawal Limit: §f${withdrawLimit}\n\n` +
        `§7All members can deposit and withdraw funds based on their rank.`
    );

    ui.button("Deposit Funds", 'textures/items/money-bag', false, 0);
    ui.button("Withdraw Funds", 'textures/items/money', false, 1);
    ui.button("Transaction History", 'textures/ui/invite_base', false, 2);

    ui.show(player).then(result => {
        if (!result || result.selection === null) return;

        switch (result.selection) {
            case 0:
                showDepositForm(player, clan);
                break;
            case 1:
                showWithdrawForm(player, clan);
                break;
            case 2:
                showTransactionHistory(player, clan);
                break;
        }
    }).catch(error => {
        console.error('Error in manageClanFunds:', error);
        player.sendMessage('§cAn error occurred while managing clan funds.');
    });
}

function showTransactionHistory(player, clan) {
    try {
        const logKey = `${config.clansDBKey}.${clan.name}.transactions`;
        const transactionsData = Database.get(logKey);
        let transactions = [];
        
        if (transactionsData) {
            transactions = JSON.parse(transactionsData);
        }

        if (transactions.length === 0) {
            player.sendMessage("§eNo transaction history available.");
            return;
        }

        const ui = new CustomUI(1, "Transaction History");
        
        let body = "§eRecent Transactions:\n\n";
        transactions.slice(-10).reverse().forEach((transaction, index) => {
            const date = new Date(transaction.timestamp).toLocaleString();
            const type = transaction.type === 'deposit' ? '§a+' : '§c-';
            body += `${type}$${transaction.amount} §7by §f${transaction.playerName}\n`;
            body += `§7${transaction.rank} • ${date}\n\n`;
        });

        ui.body(body);
        ui.button("Close", 'textures/ui/cancel', false, 0);
        
        // Add delete button only for clan owner
        const playerRole = getRole(player);
        if (playerRole === 'owner') {
            ui.button("§cDelete", 'textures/ui/trash', false, 1);
        }
        
        ui.show(player).then(result => {
            if (!result || result.selection === null) return;
            
            if (result.selection === 1 && playerRole === 'owner') {
                // Show confirmation dialog before deleting
                const confirmUI = new CustomUI(1, "Confirm Delete History");
                confirmUI.body("§c⚠ Warning!\n\n§fAre you sure you want to delete all transaction history?\n§7This action cannot be undone.");
                confirmUI.button("§cDelete History", 'textures/ui/trash', false, 0);
                confirmUI.button("Cancel", 'textures/ui/cancel', false, 1);
                
                confirmUI.show(player).then(confirmResult => {
                    if (!confirmResult || confirmResult.selection !== 0) return;
                    
                    // Delete the transaction history
                    Database.delete(logKey);
                    player.sendMessage("§aTransaction history has been cleared.");
                    
                    // Log this action
                    const adminLogKey = `${config.clansDBKey}.${clan.name}.adminLog`;
                    let adminLog = [];
                    try {
                        const existingLog = Database.get(adminLogKey);
                        if (existingLog) {
                            adminLog = JSON.parse(existingLog);
                        }
                    } catch (error) {
                        console.warn('Error parsing admin log');
                    }
                    
                    adminLog.push({
                        action: 'clear_transactions',
                        player: player.name,
                        timestamp: Date.now()
                    });
                    
                    // Keep only last 50 admin actions
                    if (adminLog.length > 50) {
                        adminLog = adminLog.slice(-50);
                    }
                    
                    Database.set(adminLogKey, JSON.stringify(adminLog));
                });
            }
        });

    } catch (error) {
        console.error('Error showing transaction history:', error);
        player.sendMessage('§cAn error occurred while viewing transaction history.');
    }
}
// Add function to handle fund notifications
function handleFundNotification(player, notification, transactionData) {
    const ui = new CustomUI(1, "Fund Transaction Details");
    
    const transactionType = transactionData.transactionType;
    const color = transactionType === 'deposit' ? '§a' : '§c';
    const action = transactionType === 'deposit' ? 'deposited' : 'withdrew';
    
    ui.body(
        `${color}${transactionData.playerName} ${action} $${transactionData.amount}\n\n` +
        `§7Transaction Time: ${new Date(transactionData.timestamp).toLocaleString()}\n\n` +
        `§eClick below to dismiss this notification`
    );

    ui.button("§aDismiss Notification", 'textures/ui/check', false, 0);

    ui.show(player).then(result => {
        if (!result || result.selection !== 0) return;
        
        // Delete the notification
        Database.delete(notification.key);
        player.sendMessage("§aNotification dismissed.");
        
        // Refresh notifications
        showNotifications(player);
    });
}


// Add this new function to check notifications
function checkPlayerNotifications(player) {
    try {
        const playerClan = Database.get(config.clanNameKey, player);
        if (!playerClan) return;

        // Get all notifications for the player's clan
        const notifications = Database.entries()
            .filter(entry => entry.key.startsWith(`${config.notificationsDBKey}.${playerClan}.`))
            .map(entry => {
                try {
                    return {
                        key: entry.key,
                        data: JSON.parse(entry.value)
                    };
                } catch (error) {
                    console.warn(`Invalid notification data for key ${entry.key}`);
                    return null;
                }
            })
            .filter(notification => notification !== null);

        if (notifications.length > 0) {
            // Show notification count to player
            player.sendMessage(`§e⚠ You have ${notifications.length} unread clan notification${notifications.length > 1 ? 's' : ''}!`);
            player.sendMessage('§7Use §6!notifications §7or §6!n §7to view them.');
            
            // Play notification sound
            player.playSound("random.orb");
            
            // Show action bar notification
            let actionBarTimer = 0;
            const actionBarInterval = system.runInterval(() => {
                if (actionBarTimer >= 5) { // Show for 5 seconds
                    system.clearRun(actionBarInterval);
                    return;
                }
                player.onScreenDisplay.setActionBar("§e⚠ New Clan Notifications Available!");
                actionBarTimer++;
            }, 20);

            // Automatically show notifications after a short delay
            system.runTimeout(() => {
                if (player.isValid()) {
                    showNotifications(player);
                }
            }, 60); // Show after 3 seconds (60 ticks)
        }

    } catch (error) {
        console.error('Error checking player notifications:', error);
    }
}

function getRole(player) {
    try {
        if (!player) return 'member';
        
        // Get clan data
        const playerClanName = Database.get(config.clanNameKey, player);
        if (!playerClanName) return 'member';

        const clanData = getClanData(playerClanName);
        if (!clanData) return 'member';

        // Check if player is the owner
        if (clanData.owner === player.name || clanData.creator === player.name) {
            // Remove any existing role tags
            const tags = player.getTags();
            tags.forEach(tag => {
                if (tag.startsWith('role:')) {
                    player.removeTag(tag);
                }
            });
            // Set owner tag
            player.addTag('role:owner');
            return 'owner';
        }

        // Check roles in clan data first
        if (clanData.roles && clanData.roles[player.name]) {
            const role = clanData.roles[player.name];
            player.addTag(`role:${role}`);
            return role;
        }

        // Check existing role tags
        const tags = player.getTags();
        for (const tag of tags) {
            if (tag.startsWith('role:')) {
                return tag.replace('role:', '');
            }
        }

        // Default to member if no role is found
        player.addTag('role:member');
        return 'member';

    } catch (error) {
        console.error('Error in getRole:', error);
        return 'member';
    }
}
// Add this helper function to ensure creator is owner when clan is created
function ensureCreatorIsOwner(player, clanName) {
    try {
        const clanData = getClanData(clanName);
        if (clanData && clanData.creator === player.name) {
            // Remove any existing role tags
            const tags = player.getTags();
            tags.forEach(tag => {
                if (tag.startsWith('role:')) {
                    player.removeTag(tag);
                }
            });
            // Set owner tag
            player.addTag('role:owner');
        }
    } catch (error) {
        console.error('Error in ensureCreatorIsOwner:', error);
    }
}



// Add a function to verify and fix roles if needed
function verifyAndFixRoles(player) {
    try {
        const playerClanName = Database.get(config.clanNameKey, player);
        if (!playerClanName) return;

        const clanData = getClanData(playerClanName);
        if (!clanData) return;

        if (clanData.creator === player.name) {
            ensureCreatorIsOwner(player, playerClanName);
        }
    } catch (error) {
        console.error('Error in verifyAndFixRoles:', error);
    }
}

// Add this to your player join event handler
world.afterEvents.playerSpawn.subscribe((event) => {
    const player = event.player;
    // Verify and fix roles when player joins
    verifyAndFixRoles(player);
});
const WAR_DATA_KEY = 'activeWarData';
const WAR_VOTES_KEY = 'surrenderVotes';
const TIMER_STATE_KEY = 'warTimerStates';
const WAR_PREP_KEY = 'warPrepStates';  // Add this new key for prep time states


function saveWarDataToDatabase() {
    try {
        const warDataToSave = {};
        const timerStates = {};
        const playerWarsObj = Object.fromEntries(activeWars);

        for (const [warId, data] of activeWarData.entries()) {
            // Save timer state with countdown info
            timerStates[warId] = {
                startTime: data.startTime,
                duration: data.duration,
                warStarted: data.warStarted,
                lastSaveTime: Date.now(),
                // Save countdown specific data
                countdownStartTime: data.countdownStartTime || Date.now(),
                prepTime: data.prepTime,
                countdownComplete: data.countdownComplete || false
            };

            // Get current scores for all players
            const clan1Scores = {};
            const clan2Scores = {};
            
            data.clan1Members.forEach(memberName => {
                const player = world.getAllPlayers().find(p => p.name === memberName);
                if (player) {
                    clan1Scores[memberName] = getScore(player, "clanWarKills") || 0;
                }
            });
            
            data.clan2Members.forEach(memberName => {
                const player = world.getAllPlayers().find(p => p.name === memberName);
                if (player) {
                    clan2Scores[memberName] = getScore(player, "clanWarKills") || 0;
                }
            });

            // Save war data
            warDataToSave[warId] = {
                ...data,
                warInterval: null,
                countdownInterval: null, // Don't save interval IDs
                clan1: data.clan1,
                clan2: data.clan2,
                clan1Members: data.clan1Members,
                clan2Members: data.clan2Members,
                clan1Kills: data.clan1Kills,
                clan2Kills: data.clan2Kills,
                clan1Scores: clan1Scores,
                clan2Scores: clan2Scores,
                betAmount: data.betAmount,
                prepTime: data.prepTime,
                countdownStartTime: data.countdownStartTime || Date.now(),
                countdownComplete: data.countdownComplete || false
            };
        }

        // Save all data to database
        Database.set(WAR_DATA_KEY, JSON.stringify(warDataToSave));
        Database.set(TIMER_STATE_KEY, JSON.stringify(timerStates));
        Database.set("activeWars", JSON.stringify(playerWarsObj));

        // Save surrender votes
        const votesToSave = {};
        for (const [warId, votes] of surrenderVotes.entries()) {
            votesToSave[warId] = Array.from(votes);
        }
        Database.set(WAR_VOTES_KEY, JSON.stringify(votesToSave));

    } catch (error) {
        console.error('Error saving war data:', error);
    }
}
function loadWarDataFromDatabase() {
    try {
        const savedWarData = Database.get(WAR_DATA_KEY);
        const savedTimerStates = Database.get(TIMER_STATE_KEY);
        const savedPlayerWars = Database.get("activeWars");
        const savedVotes = Database.get(WAR_VOTES_KEY);

        if (savedWarData && savedTimerStates) {
            const warData = JSON.parse(savedWarData);
            const timerStates = JSON.parse(savedTimerStates);

            Object.entries(warData).forEach(([warId, data]) => {
                const timerState = timerStates[warId];
                if (timerState) {
                    if (!timerState.warStarted && !timerState.countdownComplete) {
                        // Calculate remaining countdown time
                        const elapsedTime = (Date.now() - timerState.countdownStartTime) / 1000;
                        const remainingCountdown = Math.max(0, timerState.prepTime - elapsedTime);

                        if (remainingCountdown > 0) {
                            // Continue countdown
                            const continuedWarData = {
                                ...data,
                                countdownStartTime: timerState.countdownStartTime,
                                prepTime: timerState.prepTime,
                                countdownComplete: false,
                                warStarted: false,
                                clan1: data.clan1,
                                clan2: data.clan2,
                                clan1Members: data.clan1Members,
                                clan2Members: data.clan2Members,
                                clan1Kills: data.clan1Kills,
                                clan2Kills: data.clan2Kills,
                                betAmount: data.betAmount,
                                duration: data.duration
                            };
                            activeWarData.set(warId, continuedWarData);

                            // Schedule the countdown to continue
                            system.runTimeout(() => {
                                startWarCountdown(warId, continuedWarData);
                            }, 20); // Small delay to ensure everything is loaded

                            // Notify players that war countdown is continuing
                            const minutes = Math.floor(remainingCountdown / 60);
                            const seconds = Math.floor(remainingCountdown %60);
                            world.getAllPlayers().forEach(player => {
                                if (data.clan1Members.includes(player.name) || 
                                    data.clan2Members.includes(player.name)) {
                                    player.sendMessage(
                                        `§e⚔ War countdown continuing: ${minutes}:${seconds.toString().padStart(2, '0')} remaining\n` +
                                        `§6${data.clan1} §fvs §6${data.clan2}`
                                    );
                                }
                            });

                        } else {
                            // Countdown finished while offline, start war
                            startWarTimer(warId, data);
                        }
                    } else if (timerState.warStarted) {
                        // Handle ongoing war
                        const elapsedTime = (Date.now() - timerState.lastSaveTime) / 1000;
                        const remainingDuration = Math.max(0, timerState.duration - elapsedTime);

                        if (remainingDuration > 0) {
                            const continuedWarData = {
                                ...data,
                                startTime: timerState.startTime,
                                duration: timerState.duration,
                                warStarted: true,
                                countdownComplete: true
                            };
                            activeWarData.set(warId, continuedWarData);
                            startWarTimer(warId, continuedWarData);
                        } else {
                            // War ended while offline
                            if (data.clan1Kills === data.clan2Kills) {
                                handleWarDraw(data);
                            } else {
                                const winningClan = data.clan1Kills > data.clan2Kills ? data.clan1 : data.clan2;
                                const losingClan = data.clan1Kills > data.clan2Kills ? data.clan2 : data.clan1;
                                endClanWar(warId, winningClan, losingClan);
                            }
                        }
                    }
                }
            });
        }

        // Restore player war associations and surrender votes
        if (savedPlayerWars) {
            const playerWars = JSON.parse(savedPlayerWars);
            Object.entries(playerWars).forEach(([playerName, warData]) => {
                activeWars.set(playerName, warData);
            });
        }

        if (savedVotes) {
            const votes = JSON.parse(savedVotes);
            Object.entries(votes).forEach(([warId, votersList]) => {
                surrenderVotes.set(warId, new Set(votersList));
            });
        }

    } catch (error) {
        console.error('Error loading war data from database:', error);
    }
}

// Add new function to handle clan info editing
function showClanInfoEditForm(player, clan) {
    const form = new ModalFormData()
        .title("Edit Clan Info Display")
        .textField(
            "Clan Description", 
            "Enter new description or leave unchanged", 
            clan.description || config.defaultClanDescription
        )
        .toggle("Show Invite Code", clan.showInviteCode !== false)
        .toggle("Show Total Kills", clan.showKills !== false)
        .toggle("Show Total Games", clan.showGames !== false)
        .toggle("Show War Wins", clan.showWins !== false)
        .toggle("Show War Losses", clan.showLosses !== false)
        .toggle("Show Win Rate", clan.showWinRate !== false)
        .toggle("Show Clan Funds", clan.showFunds !== false)
        .toggle("Show Clan Rank", clan.showRank !== false);

    form.show(player).then(response => {
        if (!response || response.canceled) return;

        const [
            newDescription, 
            showInviteCode, 
            showKills,
            showGames,
            showWins,
            showLosses,
            showWinRate,
            showFunds,
            showRank
        ] = response.formValues;

        // Update clan settings
        clan.description = newDescription.trim() || clan.description;
        clan.showInviteCode = showInviteCode;
        clan.showKills = showKills;
        clan.showGames = showGames;
        clan.showWins = showWins;
        clan.showLosses = showLosses;
        clan.showWinRate = showWinRate;
        clan.showFunds = showFunds;
        clan.showRank = showRank;

        // Save updated clan data
        const inviteCode = Database.get(config.inviteCodeKey, player);
        Database.set(`${config.clansDBKey}.${inviteCode}`, JSON.stringify(clan));

        player.sendMessage("§aClan info display settings updated!");

        // Return to clan settings
        showClanSettings(player, clan);
    });
}

// Add this new function
function showAddonInfo(player) {
    const ui = new CustomUI(1, "§6JMAFK Clan System");
    
    const infoText = 
        "§e⭐ Created by §bJMAFKBEDROCK \n\n" +
        "§e⭐ Discord §bjm_afk \n" +
        "§eSubscribe: §bhttps://www.youtube.com/@JMAFKBedrock\n" +
        "\n§6HOW TO USE THIS ADDON\n\n" +
        "§e1. §fCreating a Clan:\n" +
        "   §b• Use §e'Clans Menu' §bto open menu\n" +
        "   §b• Choose 'Create' and set clan name/description\n" +
        "   §b• Select a clan logo\n\n" +
        "§e2. §fJoining a Clan:\n" +
        "   §b• Get an invite code from a clan\n" +
        "   §b• Use the 'Join' button and enter the code\n" +
        "   §b• Or you can view clan list and join a clan\n\n" +
        "§e3. §fClan Wars:\n" +
        "   §b• Only clan owners can initiate wars\n" +
        "   §b• Set bet amount and duration\n" +
        "   §b• Use §e!s §bor §e!surrender §bto forfeit\n\n" +
        "§e4. §fClan Management:\n" +
        "   §b• Owners can assign roles\n" +
        "   §b• Manage clan funds\n" +
        "   §b• Customize clan info display\n\n" +
        "§e5. §fClan Ranks:\n" +
        "   §b• Win wars to increase clan rank\n" +
        "   §b• Ranks: Rookie → Legend\n\n" +
        "§e6. §fNotifications:\n" +
        "   §b• War invites\n" +
        "   §b• Join requests\n" +
        "   §b• Fund transactions\n\n" +
        "§e7. §fAdmin Access:\n" +
        "   §b• Tag yourself admin to see new buttons\n" +
        "   §b• See new buttons in Clans Menu\n\n" +
        "§c⚠ Important Notes:\n" +
        "§7• Keep your invite code private\n" +
        "§7• Only owners can manage wars and funds\n" +
        "§7• Be active to maintain clan status\n\n" +
        "§6✨ Special Features:\n" +
        "§b• Custom clan logos\n" +
        "§b• War statistics tracking\n" +
        "§b• Clan fund management\n" +
        "§b• Role hierarchy system\n\n" +
        "§c⚠ Admin Scoreboards:\nWarning: Do not edit these scoreboards\n" +
        "§7• money §f- Player's money balance\n" +
        "§7• clanWarKills §f- Kills during clan wars\n" +
        "§7• clanKills §f- Total clan kills\n" +
        "§7• clanWins §f- Total clan war wins\n" +
        "§7• clanLosses §f- Total clan war losses\n" +
        "§7• clanGames §f- Total games played\n" +
        "§7• clanRank §f- Clan ranking points\n" +
        "§7• clanFunds §f- Clan treasury balance\n" +
        "§7• warTimer §f- Active war countdown\n" +
        "§7• surrenderVotes §f- War surrender votes\n\n" +
        "§d❤ Thank you for using JMAFK Clan System!\n" +
        "§d❤ Special thanks to Angel136381 for the Free Form UI!\n" +
        "§eRemember to like and subscribe for updates!";;

    ui.body(infoText);
    ui.button("§l§6Back to Menu", "textures/ui/arrow_left", false, 0);

    ui.show(player).then(result => {
        if (result && result.selection === 0) {
            showMainMenu(player);
        }
    });
}