import { world, system, Player, Dimension, ItemStack, EnchantmentType, Entity, Block } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";

import { CustomUI } from "./customForm.js";
import { getScore, setScore } from "./library/lib/index";

// Define the config object at the top of the file
const config = {
    clansDBKey: "clans", // Base key for clan data in the database
    clanMembersDBKey: "clanMembers", // Key for storing members data, if separate
    clanNameKey: "clanName", // Key for storing the player's clan name
    MONEY_SCORE: 'money', // Scoreboard objective for money
    WINS_SCORE: 'wins', // Scoreboard objective for wins
    LOSSES_SCORE: 'losses',
    JOB_DB_KEY: 'jobrequest', // Database key for storing job requests
    ShopTag: 'shop', // Tag for shop players
    formLeaveMessage: "§cMenu Closed", // Message displayed when a menu is closed
    shopItemsDBKey: 'ShopItems', // Database key for storing shop items
    REPORTS_DB_KEY: 'ShopReports', // Database key for storing item reports
    KILLS_SCORE: 'kills', // Scoreboard objective for kills
    typingJobCost: 5000,   // Cost to register for the job
    typingReward: 50,     // Reward for each correct code
    randomCodes: [
        'diamond', 'gold', 'iron', 'coal',
        'wood', 'lapis', 'dirt', 'glass', 'sand',
        'flint', 'wool', 'cake', 'fish', 'melon', 
        'beetroot', 'stone', 'ender_pearl', 'bone', 'gunpowder',
        'pork', 'bread', 'carrot', 'potato', 'sugar'
    ] // Closing the array
}; // Closing the config object

// Initialize scoreboards
function initializeScoreboards() {
    const objectives = [
        { id: config.MONEY_SCORE, displayName: "Money" },
        { id: config.WINS_SCORE, displayName: "Wins" },
        { id: config.LOSSES_SCORE, displayName: "Losses" },
        { id: "kills", displayName: "Kills" },
        { id: "points", displayName: "Points" },
        { id: "duelkills", displayName: "Duel Kills" }
    ];

    for (const obj of objectives) {
        try {
            if (!world.scoreboard.getObjective(obj.id)) {
                world.scoreboard.addObjective(obj.id, obj.displayName);
            }
        } catch (error) {
            console.error(`Error creating objective ${obj.id}:`, error);
        }
    }
}
// UUID-based rank system
const uuidRanks = [
    "", "", "", "", "", "", "",
    "", "", "", "", "", "", "", "",
    "", "", "", "", ""
];


// Call the function to initialize scoreboards
initializeScoreboards();

// Updated getScore function with better error handling
async function getScore(player, objective) {
    if (!player || !objective) {
        console.warn("Invalid player or objective");
        return 0;
    }
    
    try {
        const scoreboardObjective = world.scoreboard.getObjective(objective);
        if (!scoreboardObjective) {
            console.warn(`Objective ${objective} not found`);
            return 0;
        }
        
        // Check if player has a valid scoreboardIdentity
        if (!player.scoreboardIdentity) {
            console.warn(`No scoreboardIdentity for player ${player.name}`);
            return 0;
        }
        
        const score = scoreboardObjective.getScore(player.scoreboardIdentity);
        return typeof score === 'number' ? score : 0;
    } catch (error) {
        console.error(`Error getting score for ${objective}:`, error);
        return 0;
    }
}

// Updated setScore function with better error handling
async function setScore(player, objective, value) {
    if (!player || !objective) {
        console.warn("Invalid player or objective");
        return;
    }
    
    try {
        const scoreboardObjective = world.scoreboard.getObjective(objective);
        if (!scoreboardObjective) {
            console.warn(`Objective ${objective} not found`);
            return;
        }
        
        // Check if player has a valid scoreboardIdentity
        if (!player.scoreboardIdentity) {
            console.warn(`No scoreboardIdentity for player ${player.name}`);
            return;
        }
        
        // Ensure value is a number
        const numericValue = Number(value);
        if (isNaN(numericValue)) {
            console.warn(`Invalid score value: ${value}`);
            return;
        }
        
        scoreboardObjective.setScore(player.scoreboardIdentity, numericValue);
    } catch (error) {
        console.error(`Error setting score for ${objective}:`, error);
    }
}

// Updated hasEnoughBalance function
async function hasEnoughBalance(player, amount) {
    if (!player || typeof amount !== 'number') {
        console.warn("Invalid player or amount");
        return false;
    }
    
    try {
        const balance = await getScore(player, config.MONEY_SCORE);
        return balance >= amount;
    } catch (error) {
        console.error('Error in hasEnoughBalance:', error);
        return false;
    }
}



// Progress bar generator remains unchanged
function generateProgressBar(current, total, type) {
    let progress = '';
    const maxBarLength = 10; // Maximum length of the progress bar
    const segmentCount = Math.ceil((current / total) * maxBarLength);

    for (let i = 0; i < maxBarLength; i++) {
        progress += i < segmentCount ? (type === 'countdown' ? '§e|' : '§a|') : '§7|';
    }

    return progress;
}

// Function to retrieve a player's rank
function getRank(player) {
    const allTags = player.getTags();
    const foundTag = allTags.find(tag => tag.startsWith('rank:'));
    return foundTag ? foundTag.substring(5) : 'Member'; // Default to 'Member' if no rank found
}

// Function to set or update a player's rank
function setRank(player, rank) {
    // Remove any existing rank tag before adding a new one
    const rankTags = player.getTags().filter(tag => tag.startsWith('rank:'));
    rankTags.forEach(tag => player.removeTag(tag)); // Remove old rank tag(s)
    // Add the new rank as a tag
    player.addTag(`rank:${rank}`);
}

// Function to update player rank based on score
async function updatePlayerRank(player) {
    const kills = await getScore(player, "kills"); // Get player's current kill count
    const rankIndex = Math.floor(kills / 10); // Every 10 kills results in a rank increase
    const newRankIndex = Math.min(rankIndex, uuidRanks.length - 1); // Ensure the index is within bounds
    const newRank = uuidRanks[newRankIndex];
    const currentRank = getRank(player);

    if (newRank !== currentRank) {
        setRank(player, newRank);
        player.sendMessage(`§aCongratulations! You have achieved a new rank: §2${newRank}`);
    } else {
        // Handle demotion if necessary
        const currentRankIndex = uuidRanks.indexOf(currentRank);
        if (currentRankIndex > 0) {
            const lowerRankIndex = currentRankIndex - 1;
            if (kills < (lowerRankIndex * 10)) {
                const lowerRank = uuidRanks[lowerRankIndex];
                setRank(player, lowerRank);
                player.sendMessage(`§cYour rank has been demoted to: §4${lowerRank}`);
            }
        }
    }
}




//
///////
////////
///////////


// Function to create scoreboard if it doesn't exist
function createScoreboardIfNotExists(objectiveId, displayName) {
    let objective = world.scoreboard.getObjective(objectiveId);
    if (objective === undefined) {
        return world.scoreboard.addObjective(objectiveId, displayName);
    }
    return objective;
}

// Function to get the player's money using getScore from the database
async function getPlayerMoney(player) {
    return await getScore(player, "money"); // Fetch money score from the scoreboard
}

// Function to get the player's level from their XP
function getPlayerLevel(player) {
    return player.level; // Get the player's XP level
}
// Subscribe to the PlayerInteractWithEntityBeforeEvent
world.beforeEvents.playerInteractWithEntity.subscribe((event) => {
    const player = event.player;
    const target = event.target;

    // Log to confirm the event is triggered
    console.log("Player interacted with entity before action");

    // Check if the player is sneaking and the target is another player
    if (player.isSneaking) {
        if (target instanceof Player) {
            console.log(`${player.name} is sneaking and about to interact with ${target.name}`);

            // Call the function to handle player interaction and show the player info form
            handlePlayerInteraction(player, target);
        } else {
            console.log("Target is not a player");
        }
    } else {
        console.log("Player is not sneaking");
    }
});


// Function to handle player interaction and show the info form
function handlePlayerInteraction(viewerPlayer, targetPlayer) {
    showPlayerInfoForm(viewerPlayer, targetPlayer);
}


async function showPlayerInfoForm(viewerPlayer, targetPlayer) {
    try {
        // Retrieve player stats
        const kills = await getScore(targetPlayer, "kills");
        const money = await getPlayerMoney(targetPlayer);
        const level = getPlayerLevel(targetPlayer);
        const rank = getRank(targetPlayer);
        const wins = await getScore(targetPlayer, "wins");
        const losses = await getScore(targetPlayer, "losses");

        // Create a CustomUI for player info
        const ui = new CustomUI(1, `IGN: ${targetPlayer.name}`);

        ui.body(`§eName: §a${targetPlayer.name}\n` +
                `§eTotal Kills: §a${kills}\n` +
                `§eTotal Wins: §a${wins}\n` +
                `§eTotal Losses: §c${losses}\n` +
                `§eMoney: §e$${money}\n` +
                `§eXP: §a${level}\n` +
                `§eRank: §a${rank}\n\n` +
                `§bWhat would you like to do?`);

        // Add buttons for actions
        ui.button("§aInvite To Duel", 'textures/items/chat', false, 0);
        ui.button("§cExit", 'textures/ui/cancel.png', false, 1);

        // Show the UI to the viewer player
        const result = await ui.show(viewerPlayer);
        const selection = result.selection;

        if (selection === 0) {
            // Handle "Invite To Duel" button
            openInviteForm(viewerPlayer, targetPlayer);
        } else if (selection === 1) {
            // Handle "Exit" button
            viewerPlayer.sendMessage('You exited the player info form.');
        }
    } catch (error) {
        // Log any errors encountered while showing the form
        console.error('Error showing player info form:', error);
    }
}


//DUEL SHOP
//
//
system.runInterval(() => {
    for (let player of world.getPlayers()) {
        if (player.hasTag("addduelshop")) {
            openBuyItemsForm(player);
            player.removeTag("addduelshop");
        }
    }
});
let playerSneakingAndJumped = new Map(); // Map to track players who sneak and then jump

// Run the interval to check all players periodically
system.runInterval(() => {
    // Loop through all players in the world
    for (let player of world.getPlayers()) {
        // Check if the player is in a duel
        const duelData = activeDuels.get(player.name);
        if (!duelData) continue; // Skip if the player is not in a duel

        // Initialize the player's tracking if not already initialized
        if (!playerSneakingAndJumped.has(player)) {
            playerSneakingAndJumped.set(player, { sneaked: false, tagged: false });
        }

        let playerState = playerSneakingAndJumped.get(player);

        // Check if the player is sneaking
        if (player.isSneaking) {
            playerState.sneaked = true; // Mark player as sneaking
        }

        // Check if the player is jumping
        if (player.isJumping) {
            // If player is jumping and has previously sneaked, and hasn't been tagged yet
            if (playerState.sneaked && !playerState.tagged) {
                player.addTag("addduelshop"); // Tag the player when they jump after sneaking
                
                playerState.tagged = true; // Mark player as tagged
            }
        }

        // Reset sneaked state if the player stops sneaking
        if (!player.isSneaking && playerState.sneaked) {
            playerState.sneaked = false; // Reset sneaked after player stops sneaking
        }

        // Allow player to sneak and jump again by resetting 'tagged' when they're not jumping anymore
        if (!player.isJumping && playerState.tagged) {
            playerState.tagged = false; // Reset tagged once the player is no longer jumping
        }
    }
}, 3); // This runs every 20 ticks (1 second)


// Function to open the "Buy Items" form
function openBuyItemsForm(player) {
    const ui = new CustomUI(4, "§y§rBuy Items");

    // Add buttons for different items with descriptions and prices
  // Add buttons to the UI for each item
  ui.button("§bBuy Random Blocks\n§7Price: $5", 'textures/blocks/dirt.png', false, 0);
  ui.button("§bBuy Gold Sword\n§7Price: $5", 'textures/items/gold_sword.png', false, 1);
  ui.button("§bBuy Diamond Sword\n§7Price: $10", 'textures/items/diamond_sword.png', false, 2);
  ui.button("§bBuy Golden Apple\n§7Price: $40", 'textures/items/gold_apple.png', false, 3);
  ui.button("§bBuy Full Netherite Armor\n§7Price: $1000", 'textures/items/netherite_helmet.png', false, 4);
  ui.button("§bBuy Full Diamond Armor\n§7Price: $700", 'textures/items/diamond_helmet.png', false, 5);
  ui.button("§bBuy Full Iron Armor\n§7Price: $500", 'textures/items/iron_helmet.png', false, 6);
  ui.button("§bBuy Arrow and Bow\n§7Price: $10", 'textures/items/bow.png', false, 7);
  ui.button("§bBuy Shield\n§7Price: $5", 'textures/items/shield.png', false, 8);
  ui.button("§bBuy Splash Potion of Healing\n§7Price: $25", 'textures/items/splash_potion_of_healing.png', false, 9);
  ui.button("§bBuy Ender Pearls\n§7Price: $30", 'textures/items/ender_pearl.png', false, 10);
  ui.button("§bBuy Totem of Undying\n§7Price: $40", 'textures/items/totem_of_undying.png', false, 11);
  ui.button("§bBuy Potion of Strength\n§7Price: $20", 'textures/items/potion_strength.png', false, 12);
  ui.button("§bBuy Potion of Speed\n§7Price: $15", 'textures/items/potion_speed.png', false, 13);
  ui.button("§bBuy Chainmail Armor Set\n§7Price: $35", 'textures/items/chainmail_helmet.png', false, 14);
  ui.button("§bBuy Crossbow and Arrows\n§7Price: $15", 'textures/items/crossbow.png', false, 15);
  ui.button("§bBuy Wind Charge\n§7Price: $25", 'textures/items/fire_charge.png', false, 16);
  
    ui.show(player).then(result => {
        const selection = result.selection;
        // Call the function to handle buying the selected item
        handleBuyItem(player, selection);
    }).catch(error => {
        console.error("Error in openBuyItemsForm:", error);
    });
}
// Initialize the playerBoughtItems map
let playerBoughtItems = new Map();

// Function to handle the purchase of items
async function handleBuyItem(player, itemSelection) {
    let itemName = "";
    let price = 0;
    let items = [];
    let quantities = [];

    // Define items and prices based on selection
    switch(itemSelection) {
        case 0:
            itemName = "Random Blocks";
            price = 5;
            items = ['minecraft:dirt'];
            quantities = [64];
            break;
        case 1:
            itemName = "Gold Sword";
            price = 5;
            items = ['minecraft:golden_sword'];
            quantities = [1];
            break;
        case 2:
            itemName = "Diamond Sword";
            price = 10;
            items = ['minecraft:diamond_sword'];
            quantities = [1];
            break;
        case 3:
            itemName = "Golden Apple";
            price = 40;
            items = ['minecraft:golden_apple'];
            quantities = [1];
            break;
        case 4:
            itemName = "Full Netherite Armor";
            price = 100;
            items = [
                'minecraft:netherite_helmet',
                'minecraft:netherite_chestplate',
                'minecraft:netherite_leggings',
                'minecraft:netherite_boots'
            ];
            quantities = [1, 1, 1, 1];
            break;
        case 5:
            itemName = "Full Diamond Armor";
            price = 70;
            items = [
                'minecraft:diamond_helmet',
                'minecraft:diamond_chestplate',
                'minecraft:diamond_leggings',
                'minecraft:diamond_boots'
            ];
            quantities = [1, 1, 1, 1];
            break;
        case 6:
            itemName = "Full Iron Armor";
            price = 50;
            items = [
                'minecraft:iron_helmet',
                'minecraft:iron_chestplate',
                'minecraft:iron_leggings',
                'minecraft:iron_boots'
            ];
            quantities = [1, 1, 1, 1];
            break;
        case 7:
            itemName = "Arrow and Bow";
            price = 10;
            items = ['minecraft:bow', 'minecraft:arrow'];
            quantities = [1, 64];
            break;
        case 8:
            itemName = "Shield";
            price = 5;
            items = ['minecraft:shield'];
            quantities = [1];
            break;
        case 9:
            itemName = "Ender Pearls";
            price = 30; // Price for Ender Pearls
            items = ['minecraft:ender_pearl'];
            quantities = [5];
            break;
        case 10:
            itemName = "Totem of Undying";
            price = 40; // Price for Totem of Undying
            items = ['minecraft:totem_of_undying'];
            quantities = [1];
            break;
        case 11:
            itemName = "Crossbow and Arrows";
            price = 15; // Price for Crossbow and Arrows
            items = ['minecraft:crossbow', 'minecraft:arrow'];
            quantities = [1, 64];
            break;
        case 12:
            itemName = "Fishing Rod";
            price = 20; // Price for Fishing Rod (useful for pulling in opponents)
            items = ['minecraft:fishing_rod'];
            quantities = [1];
            break;
        case 13:
            itemName = "Flint and Steel";
            price = 15; // Price for Flint and Steel (good for setting traps)
            items = ['minecraft:flint_and_steel'];
            quantities = [1];
            break;
        case 14:
            itemName = "Bowl of Soup";
            price = 10; // Price for Bowl of Soup (good for healing)
            items = ['minecraft:mushroom_stew'];
            quantities = [1];
            break;
        case 15:
            itemName = "Fire Charge";
            price = 25; // Price for Fire Charge (useful for combat)
            items = ['minecraft:fire_charge'];
            quantities = [1];
            break;
        case 16:
            itemName = "Wind Charge"; // New Wind Charge Item
            price = 50; // Price for Wind Charge
            items = ['minecraft:wind_charge']; // Or you could use a custom item like a wind charge item
            quantities = [5]; // Quantity of Wind Charge
            break;
        default:
            player.sendMessage("§cInvalid selection!"); // Error message for invalid selection
            return; // Exit the function
    }


  // Process the purchase
  const points = getScore(player, "points");

  if (points >= price) {
      setScore(player, "points", points - price);

      // Store the purchased items in the playerBoughtItems map
      let purchasedItems = playerBoughtItems.get(player.name) || [];
      for (let i = 0; i < items.length; i++) {
          purchasedItems.push({ item: items[i], quantity: quantities[i] });
      }
      playerBoughtItems.set(player.name, purchasedItems);

      // Give the player the selected items
      const failedItems = [];
      for (let i = 0; i < items.length; i++) {
          const remaining = await giveItems(player, items[i], quantities[i]);
          if (remaining > 0) {
              failedItems.push(items[i]);
          }
      }

      if (failedItems.length > 0) {
          player.sendMessage(`§cYour inventory is full. Could not add: ${failedItems.join(", ")}.`);
      } else {
          player.sendMessage(`§aYou bought ${itemName} for $${price}.`);
      }
  } else {
      player.sendMessage(`§cYou don't have enough money to buy ${itemName}.`);
  }
}


// Function to give the player the selected items and auto-equip armor
async function giveItems(player, itemIdentifier, quantity) {
    // Check if this is part of a full armor set purchase or a totem
    const isFullArmorSet = isPartOfFullArmorSet(itemIdentifier);
    const isTotem = itemIdentifier === 'minecraft:totem_of_undying';
    
    // Create the item stack with the specified quantity
    const itemStack = new ItemStack(itemIdentifier, quantity);

    if (isFullArmorSet) {
        // If it's part of a full set, equip it directly
        return equipArmorPiece(player, itemIdentifier) ? 0 : quantity;
    } else if (isTotem) {
        // If it's a totem, equip it in the offhand slot
        return equipTotem(player) ? 0 : quantity;
    } else {
        // For non-armor items, add to inventory normally
        const inventory = player.getComponent("minecraft:inventory").container;
        const remaining = inventory.addItem(itemStack);
        return remaining ? remaining.count : 0;
    }
}

// Function to equip totem in offhand slot using replaceitem
function equipTotem(player) {
    try {
        // Use replaceitem command to place totem in offhand
        player.runCommandAsync('replaceitem entity @s slot.weapon.offhand 0 totem_of_undying 1')
            .then(() => {
                player.sendMessage("§aTotem of Undying equipped in offhand slot");
            })
            .catch(error => {
                console.error("Failed to execute replaceitem command:", error);
                // Fallback to inventory method if command fails
                const inventory = player.getComponent("minecraft:inventory").container;
                const totemStack = new ItemStack('minecraft:totem_of_undying', 1);
                inventory.setItem(1, totemStack);
            });
        return true;
    } catch (error) {
        console.error("Failed to equip totem:", error);
        return false;
    }
}

// Add an interval to check and replace totem if it's used
system.runInterval(() => {
    world.getPlayers().forEach(player => {
        // Check if player is in a duel
        const duelData = activeDuels.get(player.name);
        if (!duelData) return;

        // Check if player has bought a totem
        const boughtItems = playerBoughtItems.get(player.name) || [];
        const hasBoughtTotem = boughtItems.some(item => item.item === 'minecraft:totem_of_undying');

        if (hasBoughtTotem) {
            // Check offhand slot
            const inventory = player.getComponent("minecraft:inventory").container;
            const offhandItem = inventory.getItem(1); // Offhand slot

            // If totem is missing from offhand, replace it
            if (!offhandItem || offhandItem.typeId !== 'minecraft:totem_of_undying') {
                player.runCommandAsync('replaceitem entity @s slot.weapon.offhand 0 totem_of_undying 1')
                    .then(() => {
                        player.sendMessage("§aTotem of Undying automatically re-equipped");
                    })
                    .catch(error => {
                        console.error("Failed to re-equip totem:", error);
                    });
            }
        }
    });
}, 20); // Check every second (20 ticks)

// Function to check if an item is part of a full armor set purchase
function isPartOfFullArmorSet(itemIdentifier) {
    const fullArmorSets = [
        ['minecraft:netherite_helmet', 'minecraft:netherite_chestplate', 'minecraft:netherite_leggings', 'minecraft:netherite_boots'],
        ['minecraft:diamond_helmet', 'minecraft:diamond_chestplate', 'minecraft:diamond_leggings', 'minecraft:diamond_boots'],
        ['minecraft:iron_helmet', 'minecraft:iron_chestplate', 'minecraft:iron_leggings', 'minecraft:iron_boots'],
        ['minecraft:chainmail_helmet', 'minecraft:chainmail_chestplate', 'minecraft:chainmail_leggings', 'minecraft:chainmail_boots']
    ];

    return fullArmorSets.some(set => set.includes(itemIdentifier));
}

// Function to equip a single armor piece
function equipArmorPiece(player, itemIdentifier) {
    try {
        const inventory = player.getComponent("minecraft:inventory").container;
        let slotIndex;

        if (itemIdentifier.includes("helmet")) {
            slotIndex = 0;
        } else if (itemIdentifier.includes("chestplate")) {
            slotIndex = 1;
        } else if (itemIdentifier.includes("leggings")) {
            slotIndex = 2;
        } else if (itemIdentifier.includes("boots")) {
            slotIndex = 3;
        } else {
            return false;
        }

        // Create and equip the armor piece
        const armorPiece = new ItemStack(itemIdentifier, 1);
        inventory.setItem(slotIndex, armorPiece);
        
        // Send success message for each piece
        const itemName = itemIdentifier.split(":")[1].replace(/_/g, " ");
        player.sendMessage(`§aAuto-equipped ${itemName}`);
        return true;
    } catch (error) {
        console.error(`Failed to equip ${itemIdentifier}:`, error);
        return false;
    }
}


const POINTS_NEEDED = 10; // Points required to reload dirt
const SLOT_RANGE = { start: 0, end: 8 }; // Slot range to check

// Helper function to add items to the player's inventory
async function giveItems1(player, itemType, quantity, slot) {
    const inventory = player.getComponent("minecraft:inventory")?.container;
    if (!inventory) return quantity;

    // Create the item stack
    const itemStack = new ItemStack(itemType, quantity);

    try {
        inventory.setItem(slot, itemStack);
        return 0; // Successfully added all items
    } catch {
        return quantity; // Return the quantity that couldn't be added
    }
}

// Main interval function
system.runInterval(() => {
    world.getPlayers().forEach(async (player) => {
        // Check if the player is in a duel
        const duelData = activeDuels.get(player.name);
        if (!duelData) return; // Skip if the player is not in a duel

        const points = getScore(player, "points");
        let replenished = false; // Track if any dirt slot was replenished

        if (points >= POINTS_NEEDED) {
            for (let slot = SLOT_RANGE.start; slot <= SLOT_RANGE.end; slot++) {
                const inventory = player.getComponent("minecraft:inventory")?.container;
                if (!inventory) continue;

                const item = inventory.getItem(slot);
                if (item?.typeId === "minecraft:dirt" && item.amount < 3) {
                    const remaining = await giveItems1(player, "minecraft:dirt", 64, slot);
                    if (remaining === 0) {
                        replenished = true; // Mark replenishment
                    }
                }
            }

            // Deduct points and notify player if any slot was replenished
            if (replenished) {
                setScore(player, "points", points - POINTS_NEEDED);
                player.sendMessage("§aDirt has been replenished in your inventory.");
            }
        }
    });
}, 1); // Runs every 100 ticks (5 seconds)




////////
////////
///////

// Function to open the form to invite a player to a duel and set a bet
export async function openInviteForm(inviter, targetPlayer) {
    const form = new ModalFormData()
        .title("Duel Invitation")
        .textField("Enter your bet amount:", "Bet Amount")
        .dropdown("Select match duration:", [
            "15 seconds",
            "1 minute",
            "5 minutes",
            "10 minutes"
        ])
        .show(inviter)
        .then(async response => {
            if (response.canceled) {
                inviter.sendMessage("§cDuel invitation cancelled.");
                return;
            }

            const betAmount = parseInt(response.formValues[0], 10);
            const matchDurationOptions = [15, 60, 300, 600]; // Durations in seconds
            const matchDuration = matchDurationOptions[response.formValues[1]];

            if (isNaN(betAmount) || betAmount <= 0) {
                inviter.sendMessage("§cInvalid bet amount. Please enter a positive number.");
                return;
            }

            // Check if the inviter and target player have enough money to place the bet
            if (!(await hasEnoughBalance(inviter, betAmount))) {
                inviter.sendMessage("§cYou do not have enough money to place this bet.");
                return;
            }
            if (!(await hasEnoughBalance(targetPlayer, betAmount))) {
                inviter.sendMessage("§cThe target player does not have enough money to accept this bet.");
                return;
            }

            // Proceed to invite the target player with the bet and match duration
            inviteToDuel(inviter, targetPlayer, betAmount, matchDuration);
      
        
            playSoundThreeTimes(targetPlayer, "random.orb", 3, 200);
        
        });
}

function playSoundThreeTimes(player, sound) {
    let playCount = 0;
    const intervalId = system.runInterval(() => {
        if (playCount < 3) {
            player.playSound(sound);
            playCount++;
        } else {
            system.clearRun(intervalId); // Clear the interval when done
        }
    }, 3); // Delay between sounds in ticks (1 second = 20 ticks)
}

// Function to invite a player to a duel
function inviteToDuel(inviter, invitee, betAmount, matchDuration) {
    const ui = new CustomUI(1, "§y§rDuel Invitation");

    ui.body(`§r${inviter.name} has invited you to a duel!\n\n` +
            `§lBet Amount: §a$${betAmount}\n` +
            `§lMatch Duration: §a${formatTime(matchDuration)}\n\n` +
            `Do you accept?`);
            
    ui.button("§aAccept", 'textures/ui/confirm.png', false, 0);
    ui.button("§cDecline", 'textures/ui/cancel.png', false, 1);

    ui.show(invitee).then(result => {
        const selection = result.selection;

        if (selection === 0) {
            startDuel(inviter, invitee, betAmount, matchDuration);
            inviter.sendMessage(`§a${invitee.name} has accepted your duel invitation with a bet of $${betAmount}!`);
        } else if (selection === 1) {
            inviter.sendMessage(`§c${invitee.name} has declined your duel invitation.`);
        }
    }).catch(error => {
        console.error("Error in inviteToDuel:", error);
        invitee.sendMessage("§cAn error occurred while handling the duel invitation.");
    });
}



const activeDuels = new Map(); // Map to store active duels and bet amounts

// Initialize coordinates with default values or load from database
let coordinates = Database.get("duelCoordinates") ? 
    JSON.parse(Database.get("duelCoordinates")) : [
        { x: -202.25, y: 66.00, z: -130.39 },
        { x: -183.21, y: 66.00, z: -129.70 },
        { x: -183.33, y: 66.00, z: -101.77 },
        { x: -202.14, y: 66.00, z: -100.48 }
    ];

// Chat handler for setting coordinates
world.beforeEvents.chatSend.subscribe((event) => {
    const player = event.sender;
    const message = event.message.toLowerCase();

    // Check if player has admin permissions
    if (!player.hasTag('admin')) return;

    // Handle coordinate setting commands
    if (message.startsWith('!c')) {
        event.cancel = true; // Prevent command from showing in chat

        try {
            const coordNumber = parseInt(message.substring(2));
            if (coordNumber >= 1 && coordNumber <= 4) {
                // Get player's current position
                const pos = player.location;
                
                // Round coordinates to 2 decimal places for cleaner storage
                const newCoord = {
                    x: Math.round(pos.x * 100) / 100,
                    y: Math.round(pos.y * 100) / 100,
                    z: Math.round(pos.z * 100) / 100
                };

                // Update the coordinate in the array
                coordinates[coordNumber - 1] = newCoord;

                // Save updated coordinates to database
                Database.set("duelCoordinates", JSON.stringify(coordinates));

                // Confirm to player
                player.sendMessage(
                    `§aCoordinate ${coordNumber} set to:\n` +
                    `§eX: ${newCoord.x}\n` +
                    `§eY: ${newCoord.y}\n` +
                    `§eZ: ${newCoord.z}`
                );

                // Display all current coordinates
                player.sendMessage("\n§6Current Duel Coordinates:");
                coordinates.forEach((coord, index) => {
                    player.sendMessage(
                        `§bPosition ${index + 1}: ` +
                        `§fX: ${coord.x}, Y: ${coord.y}, Z: ${coord.z}`
                    );
                });
            }
        } catch (error) {
            console.error("Error setting coordinate:", error);
            player.sendMessage("§cError setting coordinate. Please try again.");
        }
    }
});

// Function to get coordinates (used in your existing duel code)
function getRandomCoordinates(excludeCoord) {
    // Load latest coordinates from database
    const savedCoords = Database.get("duelCoordinates");
    if (savedCoords) {
        coordinates = JSON.parse(savedCoords);
    }

    // Filter out the excluded coordinate (if any)
    const filteredCoords = excludeCoord
        ? coordinates.filter(coord => 
            coord.x !== excludeCoord.x || 
            coord.z !== excludeCoord.z)
        : coordinates;
    
    // Select a random coordinate
    const randomIndex = Math.floor(Math.random() * filteredCoords.length);
    return filteredCoords[randomIndex];
}


async function startDuel(player1, player2, betAmount, matchDuration) {
    const duelObjective = createScoreboardIfNotExists("duelkills", "Duel Kills");

    // Initial teleport positions
    const player1Start = getRandomCoordinates();
    const player2Start = getRandomCoordinates(player1Start); // Exclude player1's position
    
    await teleportPlayer(player1, player1Start);
    await teleportPlayer(player2, player2Start);

    // Set initial duel kills to 0 for both players
    setScore(player1, "duelkills", 0);
    setScore(player2, "duelkills", 0);

    // Store duel info with duelStarted as false for pre-duel countdown
    activeDuels.set(player1.name, { opponent: player2, betAmount, duelStarted: false });
    activeDuels.set(player2.name, { opponent: player1, betAmount, duelStarted: false });

    player1.sendMessage(`§eThe duel will start in 15 seconds!`);
    player2.sendMessage(`§eThe duel will start in 15 seconds!`);

    let countdownTimer = 15; // Countdown before duel starts

    // Countdown before the duel starts
    const countdownInterval = system.runInterval(() => {
        if (countdownTimer > 0) {
            // Display countdown in ActionBar
            player1.onScreenDisplay.setActionBar(`§bDuel starting in: §a${countdownTimer}s`);
            player2.onScreenDisplay.setActionBar(`§bDuel starting in: §a${countdownTimer}s`);

            teleportPlayer(player1, player1Start);
            teleportPlayer(player2, player2Start);

            countdownTimer--;
        } else {
            // End countdown and start duel
            system.clearRun(countdownInterval);

            // Set duelStarted to true
            activeDuels.get(player1.name).duelStarted = true;
            activeDuels.get(player2.name).duelStarted = true;

          
            // Duel has started, inform players
            player1.sendMessage(`§aDuel started! Duration: ${formatTime(matchDuration)}`);
            player2.sendMessage(`§aDuel started! Duration: ${formatTime(matchDuration)}`);

            let timer = matchDuration; // Set the match duration timer

            // Duel countdown
            const duelInterval = system.runInterval(() => {
                if (timer > 0) {
                    // Get scores for both players
                    const player1Kills = getScore(player1, "duelkills") || 0;
                    const player2Kills = getScore(player2, "duelkills") || 0;

                    // Generate progress bars for scores
                    const player1ScoreBar = generateProgressBar(player1Kills, 10, 'kills');
                    const player2ScoreBar = generateProgressBar(player2Kills, 10, 'kills');

                    // Display time and scores in ActionBar
                    player1.onScreenDisplay.setActionBar(
                        `§bYou: ${player1ScoreBar} ${formatTime(timer)} Enemy: ${player2ScoreBar}`
                    );
                    player2.onScreenDisplay.setActionBar(
                        `§bYou: ${player2ScoreBar} ${formatTime(timer)} Enemy: ${player1ScoreBar}`
                    );

                    // Add 1 point to both players every second during the duel
                    setScore(player1, "points", (getScore(player1, "points") || 0) + 1);
                    setScore(player2, "points", (getScore(player2, "points") || 0) + 1);

                    timer--; // Decrease timer each second
                } else {
                    system.clearRun(duelInterval); // End the duel timer

                    // Fetch final scores
                    const player1Kills = getScore(player1, "duelkills") || 0;
                    const player2Kills = getScore(player2, "duelkills") || 0;

                    // Determine the winner based on kills
                    if (player1Kills > player2Kills) {
                        declareWinner(player1, player2);
                    } else if (player2Kills > player1Kills) {
                        declareWinner(player2, player1);
                    } else {
                        // Tie
                        player1.sendMessage("§eThe duel ended in a tie!");
                        player2.sendMessage("§eThe duel ended in a tie!");
                    }

                    // Cleanup duel state
                    activeDuels.delete(player1.name);
                    activeDuels.delete(player2.name);

                    // Clear ActionBar for both players
                    player1.onScreenDisplay.setActionBar("");
                    player2.onScreenDisplay.setActionBar("");
                }
            }, 20); // 20 ticks = 1 second
        }
    }, 20); // 20 ticks = 1 second
}


// Helper function to teleport player
async function teleportPlayer(player, coord) {
    try {
        await player.teleport(
            { x: coord.x, y: coord.y, z: coord.z },
            { dimension: player.dimension }
        );
    } catch (error) {
        console.error(`Error teleporting player ${player.name}:`, error);
    }
}
// Utility function to format time in mm:ss
function formatTime(seconds) {
    const minutes = Math.floor(seconds / 60);
    const remainingSeconds = seconds % 60;
    return `${minutes}:${remainingSeconds < 10 ? '0' : ''}${remainingSeconds}s`;
}

// Declare the winner and update money, wins, and losses
async function declareWinner(winner, loser) {
    try {
        const duelData = activeDuels.get(winner.name);
        const betAmount = duelData.betAmount; // Retrieve the bet amount

        // Inform players of the result
        winner.sendMessage(`§aYou have won the duel against ${loser.name}! You win $${betAmount}.`);
        loser.sendMessage(`§cYou have lost the duel against ${winner.name}. You lost $${betAmount}.`);

        // Retrieve scoreboard objectives for money, wins, and losses
        let moneyObj = world.scoreboard.getObjective(config.MONEY_SCORE);
        let winsObj = world.scoreboard.getObjective(config.WINS_SCORE);
        let lossesObj = world.scoreboard.getObjective(config.LOSSES_SCORE);
        let pointsObj = world.scoreboard.getObjective("points");  // Access the points scoreboard

        if (!moneyObj || !winsObj || !lossesObj || !pointsObj) {
            console.error("Scoreboard objectives do not exist.");
            return;
        }

        // Get current balance of the winner and loser (default to 0 if undefined)
        let winnerBalance = moneyObj.getScore(winner) || 0;
        let loserBalance = moneyObj.getScore(loser) || 0;

        // Update the scoreboard with the new balances
        moneyObj.setScore(winner, winnerBalance + betAmount); // Add bet amount to winner's balance
        moneyObj.setScore(loser, Math.max(loserBalance - betAmount, 0)); // Deduct bet amount from loser's balance (ensure no negative balance)

        // Update wins and losses
        let winnerWins = winsObj.getScore(winner) || 0;
        let loserLosses = lossesObj.getScore(loser) || 0;

        winsObj.setScore(winner, winnerWins + 1); // Increment winner's wins
        lossesObj.setScore(loser, loserLosses + 1); // Increment loser's losses

        // Inform the winner of their new balance and wins
        winner.sendMessage(`§aYour new balance is $${winnerBalance + betAmount}. Total Wins: ${winnerWins + 1}`);
        loser.sendMessage(`§cYour new balance is $${Math.max(loserBalance - betAmount, 0)}. Total Losses: ${loserLosses + 1}`);

        // Reset points for both players to 0 after the duel ends
        await setScore(winner, "points", 0);
        await setScore(loser, "points", 0);

        // Clear the bought items from the winner and loser's inventories
        clearBoughtItems(winner);
        clearBoughtItems(loser);

        // Remove duel from active duels map
        activeDuels.delete(winner.name);
        activeDuels.delete(loser.name);

    } catch (error) {
        console.error('Error in declareWinner:', error);
    }
}
// Function to remove items from the player's inventory
async function removeItemFromPlayer(player, itemIdentifier, quantity) {
    try {
        // Get the player's inventory container
        const inventory = player.getComponent("minecraft:inventory").container;

        // Check if the inventory component exists
        if (!inventory) {
            player.sendMessage("§cError: Could not access inventory.");
            return;
        }

        // Attempt to remove the specified quantity of the item from the player's inventory
        const itemStack = new ItemStack(itemIdentifier, quantity); // Assuming ItemStack is defined correctly
        const removedCount = inventory.removeItemCount(itemStack); // Using removeItemCount instead of removeItem

        if (removedCount > 0) {
            player.sendMessage(`§cRemoved ${removedCount} of ${itemIdentifier.split(":")[1].replace(/_/g, " ")} from your inventory.`);
        } else {
            player.sendMessage("§cFailed to remove items from your inventory.");
        }
    } catch (error) {
        console.error("Error in removeItemFromPlayer:", error);
    }
}


// Function to clear the bought items from the player's inventory
function clearBoughtItems(player) {
    if (playerBoughtItems.has(player.name)) {
        const boughtItems = playerBoughtItems.get(player.name);

        // Remove each item from the player's inventory
        boughtItems.forEach(async ({ item, quantity }) => {
            try {
                await removeItemFromPlayer(player, item, quantity); // Call the removeItemFromPlayer function
            } catch (err) {
                console.error(`Error removing item ${item}:`, err);
            }
        });

        // Clear the purchased items after the duel ends
        playerBoughtItems.delete(player.name);

        // Optionally, inform the player that their items have been cleared
        player.sendMessage("§aYour bought items have been cleared after the duel.");
    }
}


// Inside the entityDie event callback
world.afterEvents.entityDie.subscribe(async function (data) {
    const deadEntity = data.deadEntity;
    const damagingEntity = data.damageSource.damagingEntity;

    // Ensure both the dead entity and damagingEntity are players
    if (deadEntity instanceof Player && damagingEntity instanceof Player) {
        // Check if both players are part of a duel
        const duelDataKiller = activeDuels.get(damagingEntity.name);
        const duelDataDead = activeDuels.get(deadEntity.name);

        if (duelDataKiller || duelDataDead) {
            const duelObjective = world.scoreboard.getObjective("duelkills");
            const deathCountObj = world.scoreboard.getObjective("deathcount");

            if (duelObjective && deathCountObj) {
                // Check if duel has started
                const duelData = duelDataKiller || duelDataDead;
                if (!duelData.duelStarted) {
                    // If duel has not started, ignore the kill
                    console.log(`Kill during countdown - ignoring for duel.`);
                    return;
                }

                try {
                    // Increment the death count for the dead player
                    const currentDeaths = await getScore(deadEntity, "deathcount") || 0;
                    await setScore(deadEntity, "deathcount", currentDeaths + 1);

                    // Update kills for the killer - ensure we get current kills first
                    const currentKills = await getScore(damagingEntity, "duelkills") || 0;
                    const newKills = currentKills + 1;
                    await setScore(damagingEntity, "duelkills", newKills);

                    // Get opponent's kills
                    const opponent = duelData.opponent;
                    const opponentKills = await getScore(opponent, "duelkills") || 0;

                    // Debug messages
                    console.warn(`Killer ${damagingEntity.name} kills updated: ${currentKills} -> ${newKills}`);
                    damagingEntity.sendMessage(`§aKills: ${newKills}`);

                    // Check if the killer has reached the required kills to declare the winner
                    if (newKills >= duelData.killsToWin) {
                        await declareWinner(damagingEntity, deadEntity);
                        // Clear duel kills after the duel is over
                        await setScore(damagingEntity, "duelkills", 0);
                        await setScore(deadEntity, "duelkills", 0);
                    } else {
                        // Only teleport if the duel isn't over
                        // Get a random coordinate excluding opponent's position
                        const opponentPos = damagingEntity.location;
                        const randomCoord = getRandomCoordinates({
                            x: opponentPos.x,
                            z: opponentPos.z
                        });

                        // Add a small delay before teleporting
                        system.runTimeout(async () => {
                            if (deadEntity.isValid()) {
                                await teleportPlayer(deadEntity, randomCoord);
                                deadEntity.sendMessage("§eYou have been teleported to a random duel position!");
                            }
                        }, 5); // 5 tick delay (0.25 seconds)
                    }

                    // Award 5 points to the killer for the kill
                    const currentPoints = await getScore(damagingEntity, "points") || 0;
                    await setScore(damagingEntity, "points", currentPoints + 5);

                } catch (error) {
                    console.error("Error updating scores:", error);
                    damagingEntity.sendMessage("§cError updating kill score.");
                }
            }
        }
    }
});





world.afterEvents.playerJoin.subscribe(async function (event) {
    const player = event.player;

    // Initialize wins and losses for new players
    const winsObj = world.scoreboard.getObjective(config.WINS_SCORE);
    const lossesObj = world.scoreboard.getObjective(config.LOSSES_SCORE);

    if (winsObj && lossesObj) {
        let playerWins = winsObj.getScore(player);
        let playerLosses = lossesObj.getScore(player);

        if (playerWins === null) {
            winsObj.setScore(player, 0);
        }
        if (playerLosses === null) {
            lossesObj.setScore(player, 0);
        }
    }
});

// Import necessary classes
const configs = {
    MOB_KILL_SCORE: 'mobkills' // Scoreboard objective for mob kills
};



//change name function


// Function to set the player's display name and name tag
function setPlayerDisplayName(player, newDisplayName) {
    if (player && player.isValid) {
        // Initialize the previous kills count
        let previousKills = getScore(player, "kills");

        // Construct the initial display name with kills included
        const displayNameWithKills = `${newDisplayName} KILLS: ${previousKills}`;

        // Set the custom display name (visible in chat)
        player.displayName = displayNameWithKills;

        // Set the name tag (visible above the player's head)
        player.nameTag = displayNameWithKills;

        // Notify the player of the change
        player.sendMessage(`Your display name has been changed to §e${displayNameWithKills}§r.`);

        // Set up an interval to update the display name
        const intervalId = system.runInterval(() => {
            if (player && player.isValid) {
                // Get the updated kills from the scoreboard
                const currentKills = getScore(player, "kills");

                // Check if the kills have changed
                if (currentKills !== previousKills) {
                    // Update the previous kills count
                    previousKills = currentKills;

                    // Construct the updated display name with kills included
                    const updatedDisplayName = `${newDisplayName} KILLS: ${currentKills}`;

                    // Update the custom display name and name tag
                    player.displayName = updatedDisplayName;
                    player.nameTag = updatedDisplayName;

                    // Optionally, send a message to confirm the update (comment this line out if not needed)
                    player.sendMessage(`Your display name has been updated to §e${updatedDisplayName}§r.`);
                }
            } else {
                // Clear the interval if the player is no longer valid
                system.clearInterval(intervalId); // Clear the interval when the player is not valid
            }
        });
    } else {
        console.error('Invalid player object.');
    }
}

// Function to open the main form with buttons
function openMainForm(player) {
    const ui = new CustomUI(4, "§y§rPlayer Menu");

    // Add buttons to the form
    ui.button("§aShow Scores", 'textures/ui/icon_steve.png', false, 13); // First button for showing scores
    ui.button("§bBuy Items", 'textures/ui/trade.png', false, 22); // Second button for buying items

    ui.show(player).then(result => {
        const selection = result.selection;

        if (selection === 13) {
            // Handle "Show Scores" button click
            showScores(player);
        } else if (selection === 22) {
            // Handle "Buy Items" button click
            openBuyItemsForm(player);
        }
    }).catch(error => {
        console.error("Error in openMainForm:", error);
    });
}

// Function to show scores
function showScores(player) {
    const killsToWin = 10; // Example kills needed for the win
    const playerKills = getScore(player, "duelkills"); // Assuming you have a scoreboard for kills
    const killsLeft = killsToWin - playerKills;

    player.sendMessage(`§aYou have ${killsLeft} kills left to win.`);
}


world.afterEvents.entityDie.subscribe(({ damageSource: killa, deadEntity: player }) => {
    if (player.typeId !== 'minecraft:player') return;
    if (player.hasTag('KeepInvOn')) return

    const inv = player.getComponent('inventory').container;
    const location = player.location;

    for (let i = 0; i < inv.size; i++) {
        const item = inv.getItem(i);
        if (item) player.dimension.spawnItem(item, location);
    }
    
    const equippable = player.getComponent("minecraft:equippable");
    ["Head", "Chest", "Legs", "Feet", "Offhand"].forEach((e) => {
        const item = equippable.getEquipment(e);
        if (item) player.dimension.spawnItem(item, location);
    });

    player.runCommandAsync('clear @s')
})








